/*
Company :		ZEFYR Technologies
File :			FFT1D.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fast Fourier Transform

Features : 
    dimension   :one
    data length :power of 2
    decimation  :frequency
    radix       :split-radix
    data        :inplace
    table       :use

Functions : 

    cdft: Complex Discrete Fourier Transform
    rdft: Real Discrete Fourier Transform

Necessary package : none

Comments : see file FFTs.txt


*/

#ifndef __ZEFYR_FFT1D_H
#define __ZEFYR_FFT1D_H

#include "customTypes.h"
#include "staticParameters.h"

#ifdef ZEF_SPLIT_RADIX

#ifdef __cplusplus
extern "C" {
#endif

// ******** main FFT routines ********

/** 1D complex Discrete Fourier Transform */
void	cdft(INT32 p_size, INT32 p_sign, REAL32 *p_data, INT32 *p_inPlace, REAL32 *p_trigo);

/** 1D real Discrete Fourier Transform */
void	rdft(INT32 p_size, INT32 p_sign, REAL32 *p_data, INT32 *p_inPlace, REAL32 *p_trigo);


// ******** initializing routines ********

/** Compute all precalc tables */
void	makewt(INT32 p_sizeW, INT32 *p_inPlace, REAL32 *p_trigo);

/** Compute table for in place processing */
void	makeipt(INT32 p_sizeW, INT32 *p_inPlace);

/** Compute trigo table */
void	makect(INT32 p_sizeC, INT32 *p_inPlace, REAL32 *p_trigoSub);

// ******** child routines ********

void	cftfsub(INT32 p_size, REAL32 *p_data, INT32 *p_inPlace, INT32 p_sizeW, REAL32 *p_trigo);
void	cftbsub(INT32 p_size, REAL32 *p_data, INT32 *p_inPlace, INT32 p_sizeW, REAL32 *p_trigo);
void	bitrv2(INT32 p_size, INT32 *p_inPlace, REAL32 *p_data);
void	bitrv2conj(INT32 p_size, INT32 *p_inPlace, REAL32 *p_data);
void	bitrv216(REAL32 *p_data);
void	bitrv216neg(REAL32 *p_data);
void	bitrv208(REAL32 *p_data);
void	bitrv208neg(REAL32 *p_data);
void	cftf1st(INT32 p_size, REAL32 *p_data, REAL32 *p_trigo);
void	cftb1st(INT32 p_size, REAL32 *p_data, REAL32 *p_trigo);
void	cftrec4(INT32 p_size, REAL32 *p_data, INT32 p_sizeW, REAL32 *p_trigo);
INT32	cfttree(INT32 p_size, INT32 p_posJ, INT32 p_posK, REAL32 *p_data, INT32 p_sizeW, REAL32 *p_trigo);
void	cftleaf(INT32 p_size, INT32 p_split, REAL32 *p_data, INT32 p_sizeW, REAL32 *p_trigo);
void	cftmdl1(INT32 p_size, REAL32 *p_data, REAL32 *p_trigo);
void	cftmdl2(INT32 p_size, REAL32 *p_data, REAL32 *p_trigo);
void	cftfx41(INT32 p_size, REAL32 *p_data, INT32 p_sizeW, REAL32 *p_trigo);
void	cftf161(REAL32 *p_data, REAL32 *p_trigo);
void	cftf162(REAL32 *p_data, REAL32 *p_trigo);
void	cftf081(REAL32 *p_data, REAL32 *p_trigo);
void	cftf082(REAL32 *p_data, REAL32 *p_trigo);
void	cftf040(REAL32 *p_data);
void	cftb040(REAL32 *p_data);
void	cftx020(REAL32 *p_data);
void	rftfsub(INT32 p_size, REAL32 *p_data, INT32 p_sizeC, REAL32 *p_trigoSub);
void	rftbsub(INT32 p_size, REAL32 *p_data, INT32 p_sizeC, REAL32 *p_trigoSub);

#ifdef __cplusplus
}
#endif

#endif //ZEF_SPLIT_RADIX

#endif //__ZEFYR_FFT1D_H
