/*
Company :		ZEFYR Technologies
File :			FFT1Db.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fast Fourier Transform

Features : 
    dimension   :one
    data length :power of 2
    decimation  :frequency
    radix       :4, 2
    data        :inplace
    table       :use

Functions : 

    cdft: Complex Discrete Fourier Transform
    rdft: Real Discrete Fourier Transform

Necessary package : none

Comments : see file FFTs.txt


*/

#ifndef __ZEFYR_FFT1D_B_H
#define __ZEFYR_FFT1D_B_H

#include "customTypes.h"
#include "staticParameters.h"

#if defined(ZEF_RADIX_2_4) && !defined(ZEF_FIXED_POINT)


#ifdef __cplusplus
extern "C" {
#endif

void cdft(INT32 n, INT32 isgn, REAL32 *a, INT32 *ip, REAL32 *w);
void rdft(INT32 n, INT32 isgn, REAL32 *a, INT32 *ip, REAL32 *w);
void makewt(INT32 nw, INT32 *ip, REAL32 *w);
void makect(INT32 nc, INT32 *ip, REAL32 *c);
void bitrv2(INT32 n, INT32 *ip, REAL32 *a);
void bitrv2conj(INT32 n, INT32 *ip, REAL32 *a);
void cftfsub(INT32 n, REAL32 *a, REAL32 *w);
void cftbsub(INT32 n, REAL32 *a, REAL32 *w);
void cft1st(INT32 n, REAL32 *a, REAL32 *w);
void cftmdl(INT32 n, INT32 l, REAL32 *a, REAL32 *w);
void rftfsub(INT32 n, REAL32 *a, INT32 nc, REAL32 *c);
void rftbsub(INT32 n, REAL32 *a, INT32 nc, REAL32 *c);
	
#ifdef __cplusplus
}
#endif

#endif // ZEF_RADIX_2_4

#endif //__ZEFYR_FFT1D_B_H
