/*
Company :		ZEFYR Technologies
File :			FFT2D.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fast Fourier Transform

Features :
    dimension   :two
    data length :power of 2
    decimation  :frequency
    radix       :split-radix, row-column
    data        :inplace
    table       :use

Functions :
    cdft2d: Complex Discrete Fourier Transform
    rdft2d: Real Discrete Fourier Transform

Necessary package
    FFT1D.c / FFT1D.h  : 1D-FFT package

Comments : see file FFTs.txt

*/

#ifndef __ZEFYR_FFT2D_H
#define __ZEFYR_FFT2D_H

#include "customTypes.h"
#include "staticParameters.h"

#ifdef __cplusplus
extern "C" {
#endif

// ******** main FFT routines ********

/** 2D complex Discrete Fourier Transform */
void cdft2d(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, REAL32 *p_data, REAL32 *p_temp, INT32 *p_inPlace, REAL32 *p_trigo);

/** 2D real Discrete Fourier Transform */
void rdft2d(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, REAL32 *p_data, REAL32 *p_temp, INT32 *p_inPlace, REAL32 *p_trigo);

/** Sorting routine to "rearrange" rdft2d output */
void rdft2dsort(INT32 p_sizeY, INT32 p_sizeX, INT32 isgn, REAL32 *a);



// ******** child routines ********

void cdft2d_sub(INT32 p_sizeY, INT32 p_sizeX, INT32 isgn, REAL32 *p_data, REAL32 *p_temp, INT32 *p_inPlace, REAL32 *p_trigo);
void rdft2d_sub(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, REAL32 *p_data);


// ******** helpful routine ********

/** returns the integer part of the base 2 logarithme of an integer parameter */
INT32 zLog2( INT32 p_number );

#ifdef __cplusplus
}
#endif

#endif //__ZEFYR_FFT2D_H
