/*******************************************************************************
*
*	Filename		:	FCGetImage.h
*	Description		:	define the SDK calls
*						Part of the SDK
*
*	Company			:	ATMEL-Grenoble
*	Project			:	FingerChip
*	Authors			:	Jean-Francois MAINGUET
*						Thomasson Nicolas
*						AC	Alain Chambron
*
*	Platform		:	independant
*	Language		:	C ANSI
*	Operating System:	independant
*
*	History
*	06-JUL-2001		:	OK
*	08-JUN-2002 PEY	:	3 API entries added :
*			(CIRIEL)	FC_CharacReadRegister
*						FC_CharacWriteRegister
*						FC_CharacGetSlice
*	31/07/2003 AC	:	turned C ANSI for Linux platforms
*	05/08/2003 AC	:	removed charac specific calls 
*						->FCCharac.h (common/include)
*						renamed FC_CharacReadRegister FC_ReadRegister
*						and FC_CharacWriteRegister FC_WriteRegister
*
*	Copyright (c)2001-2003  All rights reserved
******************************************************************************/

#ifndef __FCGETIMAGE__
#define __FCGETIMAGE__
/*
#ifdef __cplusplus
extern "C"
{
#endif // __cplusplus
*/
#ifdef WIN32
/*
* The following ifdef block is the standard way of creating macros which make
* exporting from a DLL simpler. All files within this DLL are compiled with
* the FCGETIMAGE_EXPORTS symbol defined on the command line. this symbol
* should not be defined on any project that uses this DLL. This way any other
* project whose source files include this file see FCGETIMAGE_API functions
* as being imported from a DLL, wheras this DLL sees symbols defined with this
* macro as being exported.
*/
#ifdef STATIC_LIB
#define FCGETIMAGE_API
#else
#ifdef FCGETIMAGE_EXPORTS
#define FCGETIMAGE_API __declspec(dllexport)
#else
#define FCGETIMAGE_API __declspec(dllimport)
#endif
#endif

#else // UNIX doesn't need call conventions
#define FCGETIMAGE_API
#endif

//Data structures definitions and error codes
#include "FCPublicStruct.h"
#include "FC_Error.h"

//This two macros could be used to empty a SliceInfo or ImageInfo structure
#define INIT_SLICE_INFO(pSliceInfo) memset(pSliceInfo,0,sizeof(pSliceInfo))
#define INIT_IMAGE_INFO(pImageInfo) memset(pImageInfo,0,sizeof(pImageInfo))

/****************************************************************************
*
* FC_GetSoftwareConfiguration (Exported function)
* FC_SetSoftwareConfiguration (Exported function)
* FC_GetDeviceInfo (Exported function)
* FC_SetDeviceInfo (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short	FC_GetSoftwareConfiguration( t_FC_SoftwareConfiguration * )
*					AFX_EXT_CLASS short	FC_SetSoftwareConfiguration( t_FC_SoftwareConfiguration * )
*					AFX_EXT_CLASS short FC_GetDeviceInfo( t_FC_DeviceInfo *)
*					AFX_EXT_CLASS short	FC_SetDeviceInfo( t_FC_DeviceInfo *)
*
* Description	:	The user can uses them to modify the t_FC_DeviceInfo
*					and the t_FC_SoftwareConfiguration structures.
*
* In			:	a pointer to the data to get or set
*
* PreCond		:	memory pointed was reserved by user
*
* Out			:	FC_OK if allright
*
* PostCond		:	Data was Get or Set
*					
*
* Remarks		:	FC_SetDeviceInfo return FC_BUSY if you call it when
*					the device is used. In this particular case only the
*					pixelDefaultValue and the reserved parameters are modified.
*
******************************************************************************/
FCGETIMAGE_API short FC_GetSoftwareConfiguration( t_FC_SoftwareConfiguration *pDestSoftConf ) ;
FCGETIMAGE_API short FC_SetSoftwareConfiguration( t_FC_SoftwareConfiguration *pSrcSoftConf ) ;
FCGETIMAGE_API short FC_GetDeviceInfo( t_FC_DeviceInfo *pDestDeviceInfo ) ;
FCGETIMAGE_API short FC_SetDeviceInfo( t_FC_DeviceInfo *pSrcDeviceInfo ) ;

/****************************************************************************
*
* FC_OpenDevice (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_OpenDevice()
*
* Description	:	The user can use it to open the device
*
* In			:	none
*
* PreCond		:	the device must not be opened by an other process.
*
* Out			:	FC_OK	if allright
*					FC_BUSY if already opened
*					FC_NOT_ENOUGH_MEMORY if not enough memory
*					FC_THRESHOLD_HIGH if computed finger detection threshold is
*										too high
*					FC_UNCONSISTENT_SLICE if slice are not correct slice
*					FC_NO_THERMAL if chip warming don't work properly
*
* PostCond		:	the device should be opened (if there is only warning)
*
* Remarks		:	if the device is already opened by the calling process
*					it is firstly closed
*
******************************************************************************/
FCGETIMAGE_API short FC_OpenDevice( void ) ;

/****************************************************************************
*
* FC_CloseDevice (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_CloseDevice()
*
* Description	:	The user can use it to close the device
*
* In			:	none
*
* PreCond		:	FC must be opened by the caller
*
* Out			:	FC_OK if all right
*
* PostCond		:	If the device was used by the calling process it's closed
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_CloseDevice( void ) ;

/****************************************************************************
*
* FC_GetInfo (Exported function)
* FC_SetInfo (Exported function)
* Prototypes	:	AFX_EXT_CLASS short	FC_GetInfo(t_FC_Info *pDestInfo)
*					AFX_EXT_CLASS short	FC_SetInfo(t_FC_Info *pSrcInfo)
* Description	:	The user can use it to set or retrieve the t_FC_Info
* In			:	a pointer do the data to get or set
* PreCond		:	memory pointed to have been reserved by user
*					parameters access NOT locked (LOCK)
* Out			:	FC_OK if allright
* PostCond		:	Data have been Get or Set
* Remarks		:	none
*
******************************************************************************/
short FC_GetInfo(t_FC_DeviceInfo *pDeviceInfo);

/*****************************************************************************
*
* FC_GetImage (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_GetImage(unsigned char*pImage,
*												 unsigned short xImage,
*												 unsigned short yImage,
*												 t_FC_ImageInfo *pImgNfo)
*
* Description	:	Acquire a fingerprint
*
* In			:	pImage	:	point to the image buffer
*					xImage	:	x size of the image
*					yImage	:	y size of the image
*					*pImgNfo:	point to a t_FC_ImageInfo structure
*
* PreCond		:	pImage must point to a xImage*yImage sized buffer
*
* Out			:	FC_OK	if allright
*					FC_BUSY	if opened by someone else
*					FC_NO_IMAGE_MEMORY if slice image is null
*					FC_IMAGE_TOO_SMALL if the image buffer is not big enough
*					FC_IMAGE_TOO_BIG if the image buffer is too big
*					FC_DRIVER_NOT_FOUND if unable to connect to the driver
*					FC_TRUNCATED if the image was truncated
*					FC_ERROR : otherwise
*
* PostCond		:	If all right, the image is acquired. 
*					If pImgNfo is provided (!=0) the result t_FC_ImageInfo is 
*					copied to.
*
* Remarks		:	none
*
*****************************************************************************/
FCGETIMAGE_API short FC_GetImage(unsigned char *pImage,
								 unsigned short xImage,
								 unsigned short yImage,
								 t_FC_ImageInfo *pImageInfo);

/****************************************************************************
*
* FC_IsFinger (Exported function)
*
* Prototypes  : AFX_EXT_CLASS short FC_IsFinger(signed long*pXMove,
*											 signed long*pYMove,
*											 unsigned long*pStandardDeviation,
*											 unsigned long timeOut,
*											 unsigned long pollingValue,
*											 t_FC_SliceInfo *pSliceNfo)
*
* Description	: Detect finger presence and displacement.
*
* In			:	pXMove				:	point to the x result displacement
*					pYMove				:	point to the y result displacement
*					pStandardDeviation	:	point to the result std deviation
*											of the tested slice
*					timeOut				:	timeout of the function
*					pollingValue		:	period between two fingers tests
*					pSliceNfo			:	pointer to a slices info structure
*
* PreCond		:	pXMove, pYMove must be initialized.
*
* Out			:	FC_OK	if allright
*					FC_BUSY	if opened by someone else
*					FC_NOT_INITIALIZED if a parameter is not correctly 
*						initialysed
*					FC_DRIVER_NOT_FOUND if unable to connect to the driver
*					FC_ERROR : otherwise
*
* PostCond		:	If all right, the image is acquired. If pImgNfo is provided
*					(!=0) the result t_FC_ImageInfo is copied to.
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_IsFinger(signed long*pXMove,
								 signed long*pYMove,
								 unsigned long*pStandardDeviation,
								 unsigned long timeOut,
								 unsigned long pollingValue,
								 t_FC_SliceInfo *pSliceNfo);


/****************************************************************************
*
* FC_StopAcquisition (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_StopAcquisition()
*
* Description	:	Stop actual acquisition
*
* In			:	none
*
* PreCond		:	caller must have the open the device himself
*					Current process opened the device.
*
* Out			:	FC_OK	if allright
*					FC_DRIVER_NOT_FOUND if unable to connect to the driver
*
* PostCond		:	Current acquisition is stopped
*
* Remarks		:	nothing happen if the device is not acquiring.
*
******************************************************************************/
FCGETIMAGE_API short FC_StopAcquisition();


/****************************************************************************
*
* FC_DebugFileGeneration (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_DebugFileGeneration(
*									 unsigned short level,char *pDebugFile)
*
* Description	:	Enable/Disable the debug file generation
*
* In			:	level :	describe the behavior
*							FC_LOG_DISABLED		: no debug file
*							FC_LOG_ENABLE_ERROR	: debug file on error
*							FC_LOG_ENABLE_ALWAYS: debug file always
*					pDebugFile : pointer to a ASCIIZ file string
*
* PreCond		:	none
*
* Out			:	FC_OK	if allright
*					FC_BUSY if already opened
*					FC_NOT_ENOUGH_MEMORY if not enough memory
*					FC_DIRECTORY_MISSING if directory is missing
*
* PostCond		:	the device should be opened
*
* Remarks		:	FC_LOG_ERASE may be mixed with a logical or operator
*					to empty the debug file
*
******************************************************************************/
FCGETIMAGE_API short FC_DebugFileGeneration(unsigned short Level,
											char *pDebugFile);


/****************************************************************************
*
* FC_GetSlice (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_GetSlice(unsigned char*pSlice,
*											 	 unsigned long sliceSize,
*												 t_FC_SliceInfo*pSliceNfo)
*
* Description	:	Acquire slices.
*
* In			:	pSlice			:	point to the slice buffer
*					sliceSize		:	size of the slice buffer
*					*pSliceNfo		:	point to a t_FC_SliceInfo structure
*
* PreCond		:	pSlice must point to a slice buffer.
*					the slice buffer must be sliceSize large.
*
* Out			:	FC_OK	if allright
*					FC_BUSY if opened by someone else
*					FC_NOT_INITIALIZED if slice memory is null
*					FC_ERROR : otherwise
*
* PostCond		:	If all right, the slices was acquired. If pSliceNfo is 
*					provided (!=0) the result t_FC_SliceInfo is copied to.
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_GetSlice(unsigned char *pSlice, 
								 unsigned long LSice,
								 t_FC_SliceInfo *pSliceInfo);


/****************************************************************************
*
* FC_ReconstructImage (Exported function)
*
* Prototypes   : AFX_EXT_CLASS short FC_ReconstructImage(
*										unsigned char *pImage,
*										unsigned short xImage,
*										unsigned short yImage,
*										unsigned char *pSlice,
*										unsigned long sliceSize,
*										 t_FC_ImageInfo *pImgNfo)
*
* Description	: Reconstruct an image from provided slices.
*
* In			:	pImage		:	point to the image buffer
*					xImage		:	x size of the image
*					yImage		:	y size of the image
*					pSlice		:	point to the slice buffer
*					sliceSize	:	size of the slice buffer
*					*pImgNfo	:	point to a t_FC_ImageInfo structure
*
* PreCond		:	pImage must point to a xImage*yImage sized buffer
*					pSlice must point to a slice buffer.
*					the slice buffer must be sliceSize large.
*
* Out			:	FC_OK	if allright
*					FC_NO_SLICE_MEMORY if slice memory is null
*					FC_NO_IMAGE_MEMORY if slice image is null
*					FC_NOT_ENOUGH_GOOD_SLICE if slice memory is too little
*					FC_IMAGE_TOO_SMALL if the image buffer is not big enough
*					FC_IMAGE_TOO_BIG if the image buffer is too big
*					FC_ERROR : otherwise
*
* PostCond		: If all right, the image is reconstructed. If pImgNfo is 
*					provided (!=0) the result t_FC_ImageInfo is copied to.
*
* Remarks		: this function assume that slices were unoised during GetSlice.
*				  acquisition and reconstruction should be done with the same 
*				  device configuration to avoid slice size conflicts
*
******************************************************************************/
FCGETIMAGE_API short FC_ReconstructImage(unsigned char *pImage,
										 unsigned short XImage,
										 unsigned short YImage,
										 unsigned char *pSlice,
										 unsigned long LSlice,
										 t_FC_ImageInfo *pImageInfo);

/****************************************************************************
*
* FC_GetCurrentTemperature (Exported function)
*
* Prototypes	:	AFX_EXT_CLASS short FC_GetCurrentTemperature
*							 (long* pTemperature)
*
* Description	:	Retrieve the current device temperature.
*
* In			:	pTemperature   :	point to the temperature buffer
*
* PreCond		:	Device must be open.
*
* Out			:	FC_OK	if allright
*					FC_BUSY if already used by someone else
*					FC_ERROR : otherwise
*
* PostCond		:	If all right, value pointed by pTemperature contains 
*					the temperature
*
* Remarks		:	if the device is not open or if the function is not able 
*					to open the device, (return value is FC_BUSY for example)
*					*temperature is equal to the last acquired temperature
*					(temperature is extract from the last slice read maybe
*					by an other process).
*					otherwise a slice is acquired to read the temperature.
*
******************************************************************************/
FCGETIMAGE_API short FC_GetCurrentTemperature(long *temperature);

/*****************************************************************************
*
* FC_SetTemperatureManagement (Exported function)
*
* Prototype		: AFX_EXT_CLASS short
*					FC_SetTemperatureManagement(enum eThermalManag ThermalManag)
*
* Description	:	The user can use it to change the policy of management of 
*					the temperature. The new policy of management is saved in
*					the parameters
*					(to see PSoftwareConfiguration-> autoThermalManag). 
*					This function does not work if the device is already opened
*					by someone else. 
*
* In			:	ThermalManag	: the new thermal managment
*
* PreCond		:	none
*
* Out			:	FC_OK if allright
*
* PostCond		:	Thermal managment is changed
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_SetTemperatureManagement(eThermalManag ThermalManag);




/*****************************************************************************
*
* FC_SetLedMode (Exported function)
*
* Prototype		: AFX_EXT_CLASS short
*					FC_SetLedMode(enum eLedMode ledMode)
*
* Description	:	The user can use it to change the Les control mode. 
*					The new Led control mode is saved in the parameters
*					(to see PSoftwareConfiguration-> ledMode). 
*					This function does not work if the device is already opened
*					by someone else. 
*
* In			:	ledMode	: the new Led control mode
*
* PreCond		:	none
*
* Out			:	FC_OK if allright
*
* PostCond		:	Led management is changed
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_SetLedMode(eLedMode LedMode);



/*****************************************************************************
*
* FC_FormatMessage (Exported function)
*
* Prototype		: AFX_EXT_CLASS short FC_FormatMessage(short retVal
*											,unsigned char *pMessage
*											,unsigned long messageSize)
*
* Description	:	The user can use it to retrieve an error message string
*					according to the error code.
*
* In			:	retVal			: the return code value to analyse
*					pMessage		: pointer to a buffer
*					messageSize		: size of the pointed buffer
*
* PreCond		:	pMessage must not be null
*					message size must be > 0
*				
*
* Out			:	FC_OK if allright
*					FC_NOT_INITIALIZED if parameters are not correctly set
*
* PostCond		:	pMessage buffer contains the error message
*
* Remarks		:	It use FCProcess::FormatIt function to copy the message
*
******************************************************************************/
FCGETIMAGE_API short FC_FormatMessage(short retVal,unsigned char *pMessage
									 ,unsigned long messageSize);


/*****************************************************************************
*
* FC_StartNav(Exported function)
* FC_StopNav(Exported function)
* FC_GetNav(Exported function)
*
* Prototypes    : AFX_EXT_CLASS short FC_StartNav()
*                 AFX_EXT_CLASS short FC_StopNav()
*                 AFX_EXT_CLASS short FC_GetNav(t_FC_NavigationInfo * pNavInfo)
*
* Description  :  Get slice with synchronisation data
*
* In           :  pNavInfo      : Returned navigation information
*
* PreCond      :  pSlice must point to a slice buffer.
*                 the slice buffer must be sliceSize large.
*
* Out          :  FC_OK  if allright
*                 FC_BUSY if opened by someone else
*                 FC_NOT_INITIALIZED if slice memory is null
*                 FC_ERROR : otherwise
*                 FC_NOT_SUPPORTED : if not FC 104
*
* PostCond     :  If all right, the slices was acquired. If pSliceNfo is
*                 provided (!=0) the result t_FC_SliceInfo is copied to.
*
* Remarks      :  none
*
******************************************************************************/
FCGETIMAGE_API short FC_StartNav();
FCGETIMAGE_API short FC_StopNav();
FCGETIMAGE_API short FC_GetNav(t_FC_NavigationInfo * pNavInfo);

/*****************************************************************************
*
* FC_WriteRegister (Exported function)
*
* Prototype    : AFX_EXT_CLASS short FC_WriteRegister(
*                                            unsigned char  ucVirtualPage,
*                                            unsigned short unAddress,
*                                            unsigned char  ucRegSize,
*                                            unsigned long  ulValue)
*
* Description  : The user can use it to access to the registers
*
* In           : ucVirtualPage : 0 for physical device
*                unAddress     : register address
*                ucRegSize     : size of the register in byte (1,2 or 4)
*                ulValue       : Value to put in the register
*
* PreCond      : unAddress but be a valid address for the device
*
*
* Out			:	FC_OK	if allright
*                FC_NOT_INITIALIZED if parameters are not correctly set
*
* PostCond		:	none
*
* Remarks		:	none
*
******************************************************************************/
FCGETIMAGE_API short FC_WriteRegister( unsigned char  ucVirtualPage,
                                       unsigned short unAddress,
                                       unsigned char  ucRegSize,
                                       unsigned long  ulValue);

/*****************************************************************************
*
* FC_ReadRegister (Exported function)
*
* Prototype    : AFX_EXT_CLASS short FC_ReadRegister(
*                                            unsigned char  ucVirtualPage,
*                                            unsigned short unAddress,
*                                            unsigned char  ucRegSize,
*                                            unsigned long *ulValue)
*
* Description  :  The user can use it to access to the registers
*
* In           :  ucVirtualPage : 0 for physical device
*                 unAddress     : register address
*                 ucRegSize     : size of the register in byte (1,2 or 4)
*                 ulValue       : Pointer to the output value
*
* PreCond      :  unAddress but be a valid address for the device
*        
*
* Out          :  FC_OK if allright
*                 FC_NOT_INITIALIZED if parameters are not correctly set
*
* PostCond     :  none
*
* Remarks      :  none
*
******************************************************************************/
FCGETIMAGE_API short FC_ReadRegister( unsigned char  ucVirtualPage,
                                      unsigned short unAddress,
                                      unsigned char  ucRegSize,
                                      unsigned long *ulValue);

/*

#ifdef __cplusplus
}
#endif // __cplusplus
*/
#endif // __FCGETIMAGE__
