/*
Company :		compatibility.h
Author :		Matthieu Perreira Da Silva 
First release :	15/04/2003

Description : functions for testing new version of the algorithm with the old template format 

Features : template convertion and display functions	
    
Functions : 
	 
	convertToOldGabV2 :	convert a new version template to an old version template
	displayMinutiae :	display a new template to an unsigned char array
	
Structures : 

	CoordRect :	old format for coordinate description
	Minutie :	old format for minutia description
	InfoLocale : old format for local infos
	GabaritV2 : old format for templates

Necessary package : 
	customTypes.h
	errorCodes.h
	tools.h
	zMaths.h

Comments :just for tests

*/

#ifndef __ZEF_COMPATIBILITY_H
#define __ZEF_COMPATIBILITY_H

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/** Not used anymore */
typedef struct _CoordRect
{
	INT16 m_x;
	INT16 m_y;
} CoordRect;

/** Not used anymore */
typedef struct _Minutie
{
	CoordRect		m_coord;
	UINT8			m_orient;
	UINT8			m_quality;
} Minutie;

/** Not used anymore */
typedef struct _InfoLocale
{
	UINT8			 m_orient;			// orientation moyenne du bloc 24x24
	UINT8			 m_qualAndVarOri;	//  qualit moyenne du bloc && ecart type des orientations
} InfoLocale;

/** Not used anymore */
typedef struct _GabaritV2
{
	INT16				m_xMin,m_xMax,m_yMin,m_yMax;
	CoordRect			m_cores[2];
	UINT8				m_nbCores;
	CoordRect			m_deltas[2];
	UINT8				m_nbDeltas;
	Minutie				m_minuties[50];
	INT32					m_nbMinuties;
	InfoLocale			m_tabLocal[256]; // tableau contenant des infos locales sur l'image (blocs 24x24)
	UINT8				m_lTabX,m_hTabY; // taille en x et y du tableau
	INT16				m_decXtab,m_decYtab; // dcalage du tableau d'infos locales par rapport au core
	INT16				m_freqMoy;
} GabaritV2;

/**
 * Convert a current template buffer to an oldTemplateV2 buffer.
 * What usedfull for testing when refactoring the BioEngine code
 * This function is not used anymore
 * 
 * @param[in]		p_src		source template in actual template format
 * @param[out]		p_dst		destination template, converted to the old template format
 * 
 * @return			ZEF_OK if everything was ok, an error code otherwise.
 */
INT32 convertToOldGabV2( UINT8 *p_src, UINT8 *p_dst );

/**
 * Places minutiaes in a image using information from a template structure.
 * Debuging function.
 * 
 * @param[in]		p_src		source image representing the image skeleton
 * @param[out]		p_dst		destination image representing the skeleton and the minutiae + quality map
 * @param[in]		p_qual		quality map of the source image
 * @param[in]		p_width		Width of the source and destination images
 * @param[in]		p_height	Height of the source and destination images
 * @param[in]		p_template	Template structure from which minutiae positions and orientation will be read
 * @param[in]		p_xCore		X position (in p_src image) of the main image core
 * @param[in]		p_yCore		Y position (in p_src image) of the main image core
 * 
 * @return			ZEF_OK if everything was ok, an error code otherwise.
 */
INT32 displayMinutiae( UINT8 *p_src, UINT8 *p_dst, UINT8 *p_qual,  UINT32 p_width, UINT32 p_height, Template *p_template,
							UINT32	p_xCore, UINT32 p_yCore );

#ifdef __cplusplus
}
#endif


#endif // __ZEF_COMPATIBILITY_H
