/*
Company :		ZEFYR Technologies
File :			dpBio.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	19/06/2003

Description : digiPass BioEngine functions

Features : bioEngine...
    
Functions : 

  	dpInitEnroll :	initialize everything
	dpUnInitEnroll :	free everything
	getMaxTemplateSize : return the max size a template can be (with the actual parameters)		
	getTemplateSize	:	return the size of a given template
	generateTemplate : generate a template from an image
	getTemplateQuality : return the quality of a given template
	loadParameters : load the custom bioEngine parameters


Necessary package : 

  none

Comments : none

*/

#ifndef __ZEFYR_DP_BIO_H
#define __ZEFYR_DP_BIO_H


// warnings
#define WARNING_OFFSET			2
#define ZEF_NO_MATCH			WARNING_OFFSET
#define	ZEF_TOO_MANY_MINUTIAE	WARNING_OFFSET+1
#define ZEF_CORE_CLOSE_TO_BORDER	WARNING_OFFSET+2

// common errors

#define ZEF_OK						1
#define ZEF_NULL					(void*)0;
#define	ZEF_NULL_POINTER			-1
#define ZEF_BUFFER_TO_SMALL			-2
#define ZEF_OUT_OF_MEMORY			-3
#define	ZEF_MATHS_ERROR				-4
#define	ZEF_CHECKSUM_FAILED			-5
#define	ZEF_BAD_VERSION				-6
#define ZEF_BAD_SECU_THRESHOLD		-7
#define	ZEF_BAD_MAX_SEARCH			-8
#define	ZEF_ENGINE_NOT_INITIALIZED	-9
#define	ZEF_BAD_SIZE				-10
#define ZEF_NOT_BLOC_MULTIPLE		-11
#define ZEF_BAD_SPEED_PARAM			-12
#define ZEF_STOPPED					-13

// ProcessPreview flags
#define ZEF_FLAG_FINGER_DETECTED	1
#define ZEF_FLAG_REMOVE_FINGER		2
#define ZEF_FLAG_BEST_IMG			4
#define ZEF_FLAG_MOVE_UP			8
#define ZEF_FLAG_MOVE_DOWN			16
#define ZEF_FLAG_MOVE_LEFT			32
#define ZEF_FLAG_MOVE_RIGHT			64
#define ZEF_FLAG_PRESS_STRONGER		128

#ifndef WINAPI
	#define WINAPI __stdcall
#endif

#ifdef __cplusplus
	extern "C" {
#endif

// functions

int				WINAPI	dpInitEnroll( void );
int				WINAPI	dpUnInitEnroll( void );
unsigned int	WINAPI	getMaxTemplateSize( void );
unsigned int	WINAPI	getTemplateSize( unsigned char *p_templateBuffer );
int				WINAPI	generateTemplate( unsigned char *p_imgData, unsigned int p_width, unsigned int p_height, unsigned char *p_templateBuffer ); 
unsigned int	WINAPI	getTemplateQuality( unsigned char *p_templateBuffer );
int				WINAPI	loadParameters( unsigned char *p_buffer, unsigned int p_size );
int				WINAPI  dpProcessPreview( unsigned char *p_imgData, unsigned int p_width, 
										 unsigned int p_height, unsigned char *p_preview, unsigned int *p_flags );	

#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_DP_BIO_H