/*
Company :		ZEFYR Technologies
File :			dpBio.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	19/06/2003

Description : digiPass BioEngine functions

Features : bioEngine...
    
Functions : 

  	dpInitMatcher :	initialize everything
	dpUnInitMatcher :	free everything
	setSecurity	: set the global security threshold
	getSecurity	: get the global security threshold
	identifyTemplate : compared a template to many others
	setDbMaxSearch	: set maximum seach depth during identification
	verifyTemplate : Function comparing two templates and returning if they match
	verifyAndRefineTemplate : Matches two templates and if the matching score is good makes a composite template
	loadParameters : load the custom bioEngine parameters


Necessary package : 

  none

Comments : none

*/

#ifndef __ZEFYR_DP_BIO_H
#define __ZEFYR_DP_BIO_H


// warnings
#define WARNING_OFFSET			2
#define ZEF_NO_MATCH			WARNING_OFFSET
#define	ZEF_TOO_MANY_MINUTIAE	WARNING_OFFSET+1
#define ZEF_CORE_CLOSE_TO_BORDER	WARNING_OFFSET+2

// common errors

#define ZEF_OK						1
#define ZEF_NULL					(void*)0;
#define	ZEF_NULL_POINTER			-1
#define ZEF_BUFFER_TO_SMALL			-2
#define ZEF_OUT_OF_MEMORY			-3
#define	ZEF_MATHS_ERROR				-4
#define	ZEF_CHECKSUM_FAILED			-5
#define	ZEF_BAD_VERSION				-6
#define ZEF_BAD_SECU_THRESHOLD		-7
#define	ZEF_BAD_MAX_SEARCH			-8
#define	ZEF_ENGINE_NOT_INITIALIZED	-9
#define	ZEF_BAD_SIZE				-10
#define ZEF_NOT_BLOC_MULTIPLE		-11
#define ZEF_BAD_SPEED_PARAM			-12
#define ZEF_STOPPED					-13

#ifndef WINAPI
	#define WINAPI __stdcall
#endif

#ifdef __cplusplus
	extern "C" {
#endif

// functions

int				WINAPI	dpInitMatcher( void );
int				WINAPI	dpUnInitMatcher( void );
int				WINAPI	verifyTemplate( unsigned char *p_templateBuffer1, unsigned char *p_templateBuffer2, unsigned int *p_score );
int				WINAPI	verifyAndRefineTemplate( unsigned char *p_templateBuffer1, unsigned char *p_templateBuffer2, unsigned int *p_score, char *p_refined );
int				WINAPI	identifyTemplate( unsigned char **p_templatesArray, unsigned int *p_nbTemplates, unsigned char *p_templateBuffer, unsigned int *p_score );
int				WINAPI	loadParameters( unsigned char *p_buffer, unsigned int p_size );
int				WINAPI	setSecurity( unsigned int p_securityLevel );
int				WINAPI	getSecurity( unsigned int *p_securityLevel );
int				WINAPI	setDbMaxSearch( unsigned int p_maxSearch );
int				WINAPI	dpStopIdentification( unsigned char *p_templateBuffer );
	
#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_DP_BIO_H