/*
Company :		ZEFYR Technologies
File :			dynamicParameters.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	05/05/2003

Description : run time parameters for fingerprint processing

Features : defines some parameters for all the processing routines		
    
Functions : 

	saveParameters : save the parameters contained in BioEngineParams structure
	loadParameters : load the parameters in BioEngineParams structure

Structures : 

	BioEngineParams : structure containing the parameters for the actual version of the algorithm
	BioEngineData : structure for datas that are passed across differents modules
	PreCompData	: preCompiled or preAllocated arrays

Necessary package : 

	customTypes.h
	staticParameters.h

Comments :none

*/

#ifndef __ZEFYR_DYNAMIC_PARAMETERS_H
#define __ZEFYR_DYNAMIC_PARAMETERS_H

#include "customTypes.h"
#include "staticParameters.h"
#include "portableMutex.h"

#ifdef __cplusplus
extern "C" {
#endif

// params struct version
#define		BIO_ENGINE_PARAM_STRUCT_VERSION		0x0100

/** Structure containing the parameters for the actual version of the algorithm.
 *  Default values and description of all the fields are in staticParameters.h */
typedef struct _bioEngineParams
{													// DEFAULT TO :
	/** @see MAX_NUMBER_OF_DETECTED_MINUTAE */
	UINT32			m_maxNumberOfDetectedMinutiae;	// 500

	/** @see MAX_NUMBER_OF_MINUTIAE */
	UINT32			m_maxNumberOfMinutiae;			// 50

	/** @see MAX_NUMBER_OF_LOCAL_INFOS */
	UINT32			m_maxNumberOfLocalInfos;		// 256

	/** @see MAX_NUMBER_OF_CORES */
	UINT32			m_maxNumberOfCores;				// 2 

	/** @see NORMALISATION_MEAN */
	UINT32			m_normalizationMean;			// 127

	/** @see NORMALISATION_STD_DEV */
	UINT32			m_normalizationStdDev;			// 50

	/** @see QUALITY_THRESHOLD */
	UINT32			m_qualityThreshold;				// 64

	/** @see FREQ_QUAL_THRESHOLD */
	FIXED32			m_freqQualThreshold;			// 0.5

	/** @see FREQ_QUAL_THRESHOLD_INC */
	FIXED32			m_freqQualThresholdInc;			// 0.5

	/** @see SPECTRUM_MULT_FACTOR */
	FIXED32			m_spectrumMultFactor;			// 0.25

	/** @see GABOR_WIDTH */
	FIXED32			m_gaborWidth;					// 0.5

	/** @see GABOR_FREQ_MULT */
	FIXED32			m_gaborFreqMult;				// 0.9

	/** @see GABOR_FUSION_THRESHOLD */
	FIXED32			m_gaborFusionThreshold;			// 4.0

	/** @see REGION_DIV_COEF */
	FIXED32			m_regionDivCoef;				// 10.0

	/** @see NB_CLASSES_SEGMENTATION */
	UINT32			m_nbClassesSegmentation;		// 4

	/** @see NB_CLASSES_REJECT_SEGMENTATION */
	UINT32			m_nbClassesRejectSegmentation;	// 1

	/** @see SEGM_MEDIAN_FILTER_SIZE */
	UINT32			m_segmMedianFilterSize;			// 3

	/** @see REGION_PERIMETER_COEF */
	UINT32			m_regionPerimeterCoef;			// 10

	/** @see REGION_SURFACR_COEF */
	UINT32			m_regionSurfaceCoef;			// 1

	/** @see ORI_FUSION_MEDIAN_SIZE */
	UINT32			m_oriFusionMedianSize;			// 3

	/** @see NB_PTS_FUSION_THRESHOLD */
	UINT32			m_nbPtsFusionThreshold;			// 2

	/** @see MULT_FACTOR_REGION_FUSION */
	FIXED32			m_multFactorRegionFusion;		// 1.25

	/** @see FUSION_ERODE_SIZE */
	UINT32			m_fusionErodeSize;				// 3

	/** @see FINAL_FUSION_MEDIAN_SIZE */
	UINT32			m_finalFusionMedianSize;		// 7

	/** @see FUSION_INTENSITY_MULT_FACTOR */
	FIXED32			m_fusionIntensityMultFactor;	// 0.5

	/** @see BINARY_MEDIAN_SIZE */
	UINT32			m_binaryMedianSize;				// 3

	/** @see MINIMUM_CURVATURE_THRESHOLD */
	UINT32			m_minimumCurvatureThreshold;	// 25

	/** @see LOCAL_INFOS_BLOC_SIZE */
	INT32			m_localInfosBlocSize;			// 24

	/** @see MINUTIAE_NEIBORHOOD_SIZE */
	UINT32			m_minutiaeNeiborhoodSize;		// 16

	/** @see BRANCH_MIN_SIZE */
	UINT32			m_branchMinSize;				// 8 

	/** @see MAX_RECONNECT_SIZE */
	UINT32			m_maxReconnectSize;				// 16

	/** @see MIN_RECONNECTION_ANGLE */
	UINT32			m_minReconnectionAngle;			// (PI/8)

	/** @see MAX_SUPPR_ANGLE */
	UINT32			m_maxSupprAngle;				// (PI/3)

	/** @see POINCARE_SIZE */
	UINT32			m_poincareSize;					// 3

	/** @see MAX_REGISTRATION_ANGLE */
	INT32			m_maxRegistrationAngle;			// 48

	/** @see FIRST_PASS_ANGLE_STEP */
	INT32			m_fisrtPassAngleStep;			// 8

	/** @see REGISTER_TRANSLATION_TOLERANCE */
	INT32			m_registerTranslationTolerance;	// 12

	/** @see TRANSLATION_SHIFT */
	UINT32			m_translationShift;				// 4	// 1<<4 = 16 pixels

	/** @see TRANSLATION */
	UINT32			m_translation;					// (1<<TRANSLATION_SHIFT)	// 1<<4 = 16 pixels
	
	/** @see MAX_TRANSLATION_WIDTH_SHIFT_1 */
	UINT32			m_maxTranslationWidthShift1;	// 5
	
	/** @see MAX_TRANSLATION_WIDTH_1 */
	INT32			m_maxTranslationWidth1;			// (1<<MAX_TRANSLATION_WIDTH_SHIFT)	// (-32)<<4 < dx < (+32)<<4 ==> -512 < dx < 512
	
	/** @see MAX_TRANSLATION_WIDTH_SHIFT_2 */
	UINT32			m_maxTranslationWidthShift2;	// 6
	
	/** @see MAX_TRANSLATION_WIDTH_2 */
	INT32			m_maxTranslationWidth2;			// (1<<MAX_TRANSLATION_WIDTH_SHIFT)	// (-32)<<4 < dx < (+32)<<4 ==> -512 < dx < 512
	
	/** @see ACCU_FILL */
	INT32			m_accuFill;						// 8
	
	/** @see TRANSLATION_MAX_THRESHOLD */
	FIXED32			m_translationMaxThreshold;		// 0.8
	
	/** @see ANGLE_TOLERANCE */
	INT32			m_angleTolerance;				// 25
	
	/** @see TRANSLATION_TOLERANCE */
	UINT32			m_translationTolerance;			// 16
	
	/** @see SPLINE_STIFFNESS */
	FIXED32			m_splineStiffness;				// 2000.0
	
	/** @see MIN_NUMBER_OF_MINUTIAE */
	UINT32			m_minNumberOfMinutiae;			// 3
	
	/** @see ORI_MATCH_NORMALIZE */
	FIXED32			m_oriMatchNormalize;			// 24.0
	
	/** @see MAX_SCORE */
	UINT32			m_maxScore;						// 1000
	
	/** @see SCORE_NORMALIZE */
	FIXED32			m_scoreNormalize;				// 16.0
	
	/** @see POW_FFT_FLAG */
	BOOL8			m_powFftFlag;					// FALSE
	
	/** @see MAX_DEFORMATION */
	UINT32			m_maxDeformation;				// 1500
	
	/** @see NB_FREQ_POS */
	UINT32			m_nbFreqPos;					// 8
	
	/** Frequency bands used for frequency estimation */
	UINT32			m_freqPos[MAX_NB_OF_FREQ];	
	
	/** @see DIST_FIND_CORE_ANGLE */
	UINT32			m_distFindCoreAngle;			// 24
	
	/** @see DIV_DIST_COEF */
	FIXED32			m_distDivCoef;					// 3.0
	
	/** @see MINUTIAE_BOOST */
	BOOL8			m_minutiaeBoost;				// TRUE
	
	/** @see DEFAULT_SECURITY */
	UINT32			m_defaultSecurity;				// 75
	
	/** @see MAX_DB_SEARCH */
	UINT32			m_maxDbSearch;					// 100
	
	/** @see MAX_IMG_WIDTH */
	UINT32			m_max_img_width;				// 512
	
	/** @see MAX_IMG_HEIGHT */
	UINT32			m_max_img_height;				// 512	
	
	/** @see GLOBAL_IMG_SCALING */
	UINT32			m_globalImgScaling;				// 256 
	
	/** @see GLOBAL_TEMPLATE_SCALING */
	INT32			m_globalTemplateScaling;		// 256
	
	/** @see FFT_BLOC_OVERLAP */
	UINT32			m_fftBlocOverlap;				// 32
	
	/** @see NUM_OF_ORIENTATION */
	UINT32			m_numOfOrientation;				// 8
	
	/** @see ORI_STEP_SIZE */
	UINT32			m_oriStepSize;					// 1	
	
	/** @see FILTER_SELECTION_THRESHOLD */
	UINT32			m_filterSelectionThreshold;		// 0
} BioEngineParams,*PBioEngineParams;

/** Structure for datas that are passed across differents modules */
typedef struct _bioEngineData
{
	Template		m_template;			/**< Template that fill be filled by different functions during the fingerprint processing */
	UINT32			m_width;			/**< width of m_workImg */
	UINT32			m_height;			/**< height of m_workImg */
	UINT8			*m_image;			/**< image comming from the sensor, it will be transformed by the algorithms */
	UINT8			*m_workImg;			/**< image used for processings */
	UINT8			*m_ori;				/**< orientation map */
	UINT8			*m_quality;			/**< quality map / mask */

} BioEngineData, PBioEngineData;


#ifndef ZEF_FIXED_POINT
/** preCompiled or preAllocated arrays.
 * Floating point version */
	typedef struct _preCompData
	{
		REAL32		*m_fftTemp;
		INT32		*m_fftInPlace;
		REAL32		*m_fftTrigo;
		REAL32		*m_butterworth;
	} PreCompData, *PPreCompData;
#else
/** preCompiled or preAllocated arrays.
 * Fixed point version */
	typedef struct _preCompData
	{
		FIXED16		*m_fftTemp;
		INT32		*m_fftInPlace;
		FIXED16		*m_fftTrigo;
		FIXED16		*m_butterworth;
	} PreCompData, *PPreCompData;
#endif // ZEF_FIXED_POINT

// declaration of the global structures
extern BioEngineParams	g_params;
extern PreCompData		g_preCompData;
extern MUTEX_OBJECT		g_paramsMutex;

// global data to abort identification
extern MUTEX_OBJECT		g_idCancelMutex;
extern UINT8			*g_idCancelPointer;

// functions

/**
 * Initializes the global parameter structure with default parameters
 * 
 * @return			ZEF_OK if everything was ok, an error code otherwise.
 *
 */
INT32 initParameters( void );

/**
 * Save all BioEngine parameters in a buffer.
 * Buffer must be allocated by caller !!!
 * 
 * @param[out]	p_buffer	buffer which will receive all the parameters
 * @param[in]	p_size		size of p_buffer
 * 
 * @return			ZEF_OK if everything was ok, an error code otherwise.
 *
 */
INT32 saveParameters( UINT8 *p_buffer, UINT32 *p_size );

/**
 * Load all the parameters of the bioEngine from a buffer
 * 
 * @param[in]	p_buffer	buffer which contains all the parameters
 * @param[in]	p_size		size of p_buffer
 * 
 * @return			ZEF_OK if everything was ok, an error code otherwise.
 *
 */
INT32 WINAPI dpLoadParameters( UINT8 *p_buffer, UINT32 p_size );

#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_DYNAMIC_PARAMETERS_H
