/*
Company :		ZEFYR Technologies
File :			enrollExport.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	09/05/2003

Description : SDK enrolment exported functions

Features : external interface functions		
    
Functions : 

	dpInitEnroll : initialize enrolment part
	dpUnInitEnroll : uninitialize enrolment part
	dpGetMaxTemplateSize : return the max size a template can be (with the actual parameters)		
	dpGetTemplateSize	:	return the size of a given template
	dpGenerateTemplate : generate a template from an image
	dpGetTemplateQuality : return the quality of a given template
	dpLoadParameters : load engine parameters

Necessary package : 

	zMaths.h
	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	preProcessings.h
	FFT2D.h
	enhancements.h
	coresDeltas.h
	localParameters.h
	skeleton.h
	minutiae.h
	tools.h

Comments : none

*/

#ifndef __ZEFYR_ENROLL_EXPORT_H
#define __ZEFYR_ENROLL_EXPORT_H

#include "dynamicParameters.h"

#ifdef __cplusplus
extern "C" {
#endif

// ********  ProcessPreview flags ******** (see description in "dpBio.h" file )
#define ZEF_FLAG_FINGER_DETECTED	1
#define ZEF_FLAG_REMOVE_FINGER		2
#define ZEF_FLAG_BEST_IMG			4
#define ZEF_FLAG_MOVE_UP			8
#define ZEF_FLAG_MOVE_DOWN			16
#define ZEF_FLAG_MOVE_LEFT			32
#define ZEF_FLAG_MOVE_RIGHT			64
#define ZEF_FLAG_PRESS_STRONGER		128

// ******** Function prorotypes ******** 

// see descritpion of these fonctions in "enhancements.h"
INT32	WINAPI dpInitEnroll( void );
INT32	WINAPI dpUnInitEnroll( void );

// For a description of these functions, see "dpBio.h" file
INT32	WINAPI dpGetMaxTemplateSize( void );
INT32	WINAPI dpGetTemplateSize( UINT8 *p_templateBuffer );
INT32	WINAPI dpGenerateTemplate( UINT8 *p_imgData, UINT32 p_width, UINT32 p_height, UINT8 *p_templateBuffer ); 
INT32	WINAPI dpGetTemplateQuality( UINT8 *p_templateBuffer );
INT32	WINAPI dpLoadParameters( UINT8 *p_buffer, UINT32 p_size );
INT32	WINAPI dpProcessPreview( UINT8 *p_imgData, UINT32 p_width, UINT32 p_height, UINT8 *p_preview, UINT32 *p_flags );

#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_ENROLL_EXPORT_H
