/*
Company :		ZEFYR Technologies
File :			fxEnhancements.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	09/05/2003

Description : fixed point fingerprint enhancements

Features : fixed point versions of differents fingerprint specific enhancements
    
Functions : 

	enhance : main enhancement routine
	fillBlocArray :	cuts the source image into overlapping blocs and store those blocs in a array
	fftBlocArray :	compute the FFT of all blocs in a bloc array
	polarSpectrum : compute the polar spectrum of each bloc of a bloc array
	freqSpectrum :	compute the main frequency of each spectrum bloc of an array of blocs
	powBlocFFT :	enhance some blocs of an array of bloc by multipl
	applyGaborFilter : filter each bloc with N oriented gabor filters tuned the the frequency processed in freqSpectrum function
	gaborFusion : use the N filterd image to find the best orientation map for the image
	findRegions : find regions of connected pixels with the same orientation
	fillRegions : replace regions that are to different from their neighboors
	findPixels :  find pixels that are too different from their neighboors
	imageFusion : reconstruct the final image based on the orientation map
	qualityThreshold : threshold the quality map to define a mask

Necessary package : 

	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	FFT2D.h
	zMaths.h
	filters.h
	greyLevelTrans.h
	geometry.h
	quicksort.h

Comments : none

*/

#ifndef __ZEFYR_FX_ENHANCEMENTS_H
#define __ZEFYR_FX_ENHANCEMENTS_H

#ifdef ZEF_FIXED_POINT

#include "customTypes.h"
#include "dynamicParameters.h"

#ifdef __cplusplus
extern "C" {
#endif

// Fixed point versions of the functions defined in ""enhancements.h"
// see "enhancements.h" for the description of these fonctions
INT32 initEnhance( void );
INT32 unInitEnhance( void );
INT32 enhance( BioEngineData *p_data );
INT32 fillBlocArray( UINT8 *p_src, FIXED16 **p_blocArray, UINT32 p_width, UINT32 p_height );
INT32 fftBlocArray( FIXED16 **p_blocArray, UINT32 p_nbBlocsX, UINT32 p_nbBlocsY );
INT32 polarSpectrum( FIXED16 **p_blocArray, UINT32 p_nbBlocsX, UINT32 p_nbBlocsY, FIXED16	**p_spectrum );
INT32 freqSpectrum( FIXED16 **p_spectrum, UINT32 p_nbBlocsX, UINT32 p_nbBlocsY, FIXED32 *p_freq, FIXED32 *p_qual, FIXED32 *p_meanFreq );
INT32 powBlocFFT( FIXED16 **p_spectrum, UINT32 p_nbBlocsX, UINT32 p_nbBlocsY, FIXED32 *p_qual );
INT32 applyGaborFilter(  FIXED16 **p_blocArray, INT8 *p_images, UINT32 p_nbBlocsX, UINT32 p_nbBlocsY, FIXED32 *p_freq );
INT32 gaborFusion( INT8 *p_images, UINT32 p_width, UINT32 p_height, UINT8 *p_quality, UINT8 *p_orient );
INT32 findRegions( UINT8 *p_orient, UINT32 *p_regions, UINT32 p_width, UINT32 p_height, FIXED32 p_threshold  );
INT32 fillRegions( UINT8 *p_orient, UINT8 *p_quality, UINT32 *intensities, UINT32 *regions, UINT32 miniWidth, UINT32 miniHeight );
INT32 findPixels( UINT8 *p_img, UINT32 *p_pixels, UINT32 p_width, UINT32 p_height, FIXED32 p_threshold );
INT32 imageFusion( INT8 *p_images, UINT8 *p_orient, UINT8 *p_dst, UINT32 p_width, UINT32 p_height );
INT32 qualityThreshold( UINT8 *p_quality, UINT32 p_width, UINT32 p_height, UINT32 p_threshold );
INT32 maskImage( UINT8 *p_src, UINT8 *p_dst, UINT32 p_width, UINT32 p_height, UINT8 *p_mask );

#if defined(USE_FX_PRECOMP)
	/** Initializes specific fixed point part of enhancement pre-computed data */
	INT32	fxInitEnhance( void );
#endif // USE_FX_PRECOMP

#ifdef __cplusplus
}
#endif

#endif // ZEF_FIXED_POINT

#endif // __FX_ZEFYR_ENHANCEMENTS_H

