/*
Company :		ZEFYR Technologies
File :			fxFFT1D.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fixed PoINT32 Fast Fourier Transform

Features : 
    dimension   :one
    data length :power of 2
    decimation  :frequency
    radix       :split-radix
    data        :inplace
    table       :use

Functions : 

    fxcdft: Complex Discrete Fourier Transform
    fxrdft: Real Discrete Fourier Transform

Necessary package : none

Comments : see file FFTs.txt


*/


#ifndef __ZEFYR_FX_FFT1D_H
#define __ZEFYR_FX_FFT1D_H

#include "customTypes.h"
#include "fixedPoint.h"
#include "staticParameters.h"

#ifdef ZEF_SPLIT_RADIX

#ifdef __cplusplus
extern "C" {
#endif

/* -------- main FFT routines -------- */

/** 1D fixed point complex Discrete Fourier Transform */
void fxcdft(INT32 p_size, INT32 p_sign, FIXED16 *p_data, INT32 *p_inPlace, FIXED16 *p_trigo);
/** 1D fixed point real Discrete Fourier Transform */
void fxrdft(INT32 p_size, INT32 p_sign, FIXED16 *p_data, INT32 *p_inPlace, FIXED16 *p_trigo);


/* -------- initializing routines -------- */

/** Compute all fixed point precalc tables */
void fxmakewt(INT32 p_sizeW, INT32 *p_inPlace, FIXED16 *p_trigo);

/** Compute fixed point table for in place processing */
void fxmakeipt(INT32 p_sizeW, INT32 *p_inPlace);

/** Compute fixed point trigo table */
void fxmakect(INT32 p_sizeC, INT32 *p_inPlace, FIXED16 *p_trigoSub);

/* -------- child routines -------- */

void	fxcftfsub(INT32 p_size, FIXED16 *p_data, INT32 *p_inPlace, INT32 p_sizeW, FIXED16 *p_trigo);
void	fxcftbsub(INT32 p_size, FIXED16 *p_data, INT32 *p_inPlace, INT32 p_sizeW, FIXED16 *p_trigo);
void	fxbitrv2(INT32 p_size, INT32 *p_inPlace, FIXED16 *p_data);
void	fxbitrv2conj(INT32 p_size, INT32 *p_inPlace, FIXED16 *p_data);
void	fxbitrv216(FIXED16 *p_data);
void	fxbitrv216neg(FIXED16 *p_data);
void	fxbitrv208(FIXED16 *p_data);
void	fxbitrv208neg(FIXED16 *p_data);
void	fxcftf1st(INT32 p_size, FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftb1st(INT32 p_size, FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftrec4(INT32 p_size, FIXED16 *p_data, INT32 p_sizeW, FIXED16 *p_trigo);
INT32	fxcfttree(INT32 p_size, INT32 p_posJ, INT32 p_posK, FIXED16 *p_data, INT32 p_sizeW, FIXED16 *p_trigo);
void	fxcftleaf(INT32 p_size, INT32 p_split, FIXED16 *p_data, INT32 p_sizeW, FIXED16 *p_trigo);
void	fxcftmdl1(INT32 p_size, FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftmdl2(INT32 p_size, FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftfx41(INT32 p_size, FIXED16 *p_data, INT32 p_sizeW, FIXED16 *p_trigo);
void	fxcftf161(FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftf162(FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftf081(FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftf082(FIXED16 *p_data, FIXED16 *p_trigo);
void	fxcftf040(FIXED16 *p_data);
void	fxcftb040(FIXED16 *p_data);
void	fxcftx020(FIXED16 *p_data);
void	fxrftfsub(INT32 p_size, FIXED16 *p_data, INT32 p_sizeC, FIXED16 *p_trigoSub);
void	fxrftbsub(INT32 p_size, FIXED16 *p_data, INT32 p_sizeC, FIXED16 *p_trigoSub);

#ifdef __cplusplus
}
#endif

#endif // ZEF_SPLIT_RADIX

#endif //__ZEFYR_FX_FFT1D_H
