/*
Company :		ZEFYR Technologies
File :			fxFFT1Db.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fixed Point Fast Fourier Transform

Features : 
    dimension   :one
    data length :power of 2
    decimation  :frequency
    radix       :4, 2
    data        :inplace
    table       :use

Functions : 

    cdft: Complex Discrete Fourier Transform
    rdft: Real Discrete Fourier Transform

Necessary package : none

Comments : see file FFTs.txt


*/

#ifndef __ZEFYR_FFT1D_B_H
#define __ZEFYR_FFT1D_B_H

#include "customTypes.h"
#include "staticParameters.h"

#ifdef ZEF_RADIX_2_4

#ifdef __cplusplus
extern "C" {
#endif

void fxcdft(INT32 n, INT32 isgn, FIXED16 *a, INT32 *ip, FIXED16 *w);
void fxrdft(INT32 n, INT32 isgn, FIXED16 *a, INT32 *ip, FIXED16 *w);
void fxmakewt(INT32 nw, INT32 *ip, FIXED16 *w);
void fxmakect(INT32 nc, INT32 *ip, FIXED16 *c);
void fxbitrv2(INT32 n, INT32 *ip, FIXED16 *a);
void fxbitrv2conj(INT32 n, INT32 *ip, FIXED16 *a);
void fxcftfsub(INT32 n, FIXED16 *a, FIXED16 *w);
void fxcftbsub(INT32 n, FIXED16 *a, FIXED16 *w);
void fxcft1st(INT32 n, FIXED16 *a, FIXED16 *w);
void fxcftmdl(INT32 n, INT32 l, FIXED16 *a, FIXED16 *w);
void fxrftfsub(INT32 n, FIXED16 *a, INT32 nc, FIXED16 *c);
void fxrftbsub(INT32 n, FIXED16 *a, INT32 nc, FIXED16 *c);
	
#ifdef __cplusplus
}
#endif

#endif // ZEF_RADIX_2_4

#endif //__ZEFYR_FFT1D_B_H
