/*
Company :		ZEFYR Technologies
File :			fxFFT2D.h
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	15/04/2003

Description : Fixed PoINT32 Fast Fourier Transform

Features :
    dimension   :two
    data length :power of 2
    decimation  :frequency
    radix       :split-radix, row-column
    data        :inplace
    table       :use

Functions :
    fxcdft2d: Complex Discrete Fourier Transform
    fxrdft2d: Real Discrete Fourier Transform

Necessary package
    fxFFT1D.c / fxFFT1D.h  : 1D-FFT package

Comments : see file FFTs.txt

*/

#ifndef __ZEFYR_FX_FFT2D_H
#define __ZEFYR_FX_FFT2D_H

#include "customTypes.h"
#include "staticParameters.h"

#ifdef __cplusplus
extern "C" {
#endif

/* -------- main FFT routines -------- */

/** 2D fixed point complex Discrete Fourier Transform */
void fxcdft2d(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, FIXED16 *p_data, FIXED16 *p_temp, INT32 *p_inPlace, FIXED16 *p_trigo);

/** 2D fixed point real Discrete Fourier Transform */
void fxrdft2d(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, FIXED16 *p_data, FIXED16 *p_temp, INT32 *p_inPlace, FIXED16 *p_trigo);

/** Sorting routine to "rearrange" fxrdft2d output */
void fxrdft2dsort(INT32 p_sizeY, INT32 p_sizeX, INT32 isgn, FIXED16 *a);


/* -------- child routines -------- */

void fxcdft2d_sub(INT32 p_sizeY, INT32 p_sizeX, INT32 isgn, FIXED16 *p_data, FIXED16 *p_temp, INT32 *p_inPlace, FIXED16 *p_trigo);
void fxrdft2d_sub(INT32 p_sizeY, INT32 p_sizeX, INT32 p_sign, FIXED16 *p_data);

/* -------- helpful routine -------- */

INT32 fxlog2( INT32 p_number );

#ifdef __cplusplus
}
#endif

#endif //__ZEFYR_FX_FFT2D_H
