/*
Company :		ZEFYR Technologies
File :			matcher.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	02/07/2003

Description : Minutiae matching algorithm
Features : Fast matching !!!
    
Functions : 

	fastVerifyTemplate : same as verifyTemplate, but using a fast core based registration algorithm
	registerMinutiae : Main registering function, align to minutiae sets
	register1stPass : first step of minutiae alignment, finds a approximate alignment
	register2ndPass : second step of minutiae alignment, refines the fisrt estimation
	applyRegistration : modifies a template using  the registration informations (rotation, translation) 
	findMatches : searches the corresponding minutiae between two aligned templates
	estimateDeformations : uses a thin plate spline model to estimate the local deformation of the fingerprints
	applyDeformations : deforms a template using the model calculated with estimateDeformations
	matchOri : calculate a matching score based on bloc orientations
	computeScore : Main function to calculate the matching score of two aligned templates 
	findRotation : Fast registration algorithm used in fastVerifyTemplate
	updateTemplate : Build a composite template with two matching and aligned templates
	updateOri : build a composite bloc orentation map
	updateCores : keep the best cores for a composite template
	scoreIndex : processes a distance score used for linear classification during identification
	sortTemplates : sort an array of template according to their distance score

Necessary package : 

	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	zMaths.h
	tools.h
	tpsWarper.h
	quicksort.h

Comments : none

*/

#ifndef __ZEFYR_FX_MATCHER_H
#define __ZEFYR_FX_MATCHER_H

#include "customTypes.h"

#ifdef ZEF_FIXED_POINT

#ifdef __cplusplus
extern "C" {
#endif

// Fixed point version of the functions defined in "matcher.h"
// See "matcher.h" for detailed description
INT32	matchTemplate( Template *p_template1, Template *p_template2, INT32 **p_matches, 
						INT32 *p_nbMatches, UINT32 *p_score );
INT32	fastVerifyTemplate( UINT8 *p_templateBuffer1, UINT8 *p_templateBuffer2, UINT32 *p_score );
INT32	registerMinutiae( Template *templ1, Template *templ2, INT32 *dx, INT32 *dy, INT32 *dTheta );
INT32	register1stPass( Template *templ1, Template *templ2, INT32 *dx, INT32 *dy, INT32 *dTheta );
INT32	register2ndPass( Template *templ1, Template *templ2, INT32 *dx, INT32 *dy, INT32 *dTheta );
INT32	applyRegistration( Template *p_templ, INT32 p_dx, INT32 p_dy, INT32 p_dTheta );
INT32	findMatches( Template *p_templ1, Template *p_templ2, INT32 **p_matches, INT32 *p_nbMatches );
INT32   estimateDeformations( Template *p_templ1, Template *p_templ2, INT32 **p_matches, UINT32 p_cptMatches, FIXED32 p_alpha, TpsData *p_data );
INT32	applyDeformations( Template *p_templ, UINT32 *p_diffs, TpsData *p_data );
UINT32  matchOri( Template *p_templ1, Template *p_templ2 );
INT32   computeScore( Template *p_templ1, Template *p_templ2, INT32 **p_matches, INT32 p_nbMatches, UINT32 *p_score );
INT32	findRotation( Template *p_templ1, Template *p_templ2, INT32 *p_dTheta );
UINT32	scoreIndex( Template *p_templ1, Template *p_templ2 );
INT32	sortTemplates( UINT8 *p_templateBuffer, UINT8 **p_templateBufferArray, UINT32 p_nbTemplates,
					 UINT32 *p_coreIndexes, UINT32 *p_coreDist, UINT32 *p_noCoreIndexes, UINT32 *p_noCoreDist );
#ifdef __cplusplus
}
#endif

#endif // ZEF_FIXED_POINT

#endif //__ZEFYR_MATCHER_H
