/*
Company :		ZEFYR Technologies
File :			fxTpsWarper.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	04/07/2003

Description : calculates a non rigid deformation between two sets of points using thin plate splines

Features : estimates local deformation of fingerprints (fixed point)
    
Functions : 

	initTpsWarper : initialize a new estimation with new data
	initMatrixA : build tyhe A matrix for TPS system resolution
	unInitTpsWarper : free memory used during TPS warping
	warp : calculate the new position of a poirn according to the deformation calculated with initTpsWarper
	getDiffs : get the "energy" needed for the deformation

Necessary package : 

	customtypes.h
	errorCodes.h
	linAlgebra.h
	zMaths.h

Comments : none

*/

#ifndef __ZEFYR_FX_TPS_WARPER_H
#define __ZEFYR_FX_TPS_WARPER_H

#ifdef ZEF_FIXED_POINT

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// Fixed point version of the functions defined in "tpsWarper.h"
// See "tpsWarper.h" for detailled description
INT32	initTpsWarper( INT32 p_nbPts, INT32 *p_srcX, INT32 *p_srcY, INT32 *p_dstX, INT32 *p_dstY, FIXED32 p_alpha, TpsData *p_data);
INT32	initMatrixA( INT32 p_nbPts, INT32 *p_srcX, INT32 *p_srcY, FIXED32 p_alpha, TpsData *p_data );
INT32	unInitTpsWarper( TpsData *p_data );
INT32	warp( INT32 *p_x, INT32 *p_y, TpsData *p_data );
UINT32  getDiffs( TpsData *p_data );

#ifdef __cplusplus
}
#endif

#endif // ZEF_FIXED_POINT

#endif //__ZEFYR_TPS_WARPER_H
