/*
Company :		ZEFYR Technologies
File :			linAlgebra.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	04/07/2003

Description : linear system resolution

Features : finds the solutyions of a linear sysem
    
Functions : 

	luDecompose : transform a matrix into its LU decomposition (used to solve a linear system)
	luBackSubstitute : resolve a system using its LU decomposition

Necessary package : 

	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	zMaths.h

Comments : none

*/

#ifndef __ZEFYR_LIN_ALGEBRA_H
#define __ZEFYR_LIN_ALGEBRA_H

#ifndef ZEF_FIXED_POINT

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/** LU decomposition of a Matrix
 *  Given a matrix p_matrix[1..n][1..n], this routine replaces it by the LU decomposition of a rowwise permutation of itself.  
 *  This routine is used in combination with luBackSubstitute to solve linear equations or invert a matrix.
 * 
 *	@param[in,out]	p_matrix		Input matrix [in], LU decomposition of p_matrix [out]
 *	@param[in]		p_size			Size of the input matrix (Square matrix)
 *	@param[out]		p_index			Output vector that records the row permutation effected by the partial pivoting;
 *	@param[out]		p_evenOdd		Is output as 1 depending on whether the number of row interchanges was even or odd, respectively.		
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise				
*/
INT32 luDecompose(float **p_matrix, int p_size, int *p_index, float *p_evenOdd);

/** Solves the set of n linear equations AX = B. 
 *  p_matrix, p_size, and p_index are not modified by this routine and can be left in place 
 *  for successive calls with different right-hand sides p_rightSide. 
 *  This routine takes into account the possibility that p_rightSide will begin with many zero elements,
 *  so it is efficient for use in matrix inversion.
 * 
 *	@param[in]		p_matrix			LU decomposition of matrix A, determined by the routine "luDecompose".	
 *	@param[in]		p_size				Size of p_matrix
 *	@param[in]		p_index				Permutation vector returned by "luDecompose"
 *	@param[in,out]	p_rightSide[]		The right-hand side vector B [in], returns with the solution vector X [out]
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise	
 *
 *	@see			luDecompose
*/
INT32 luBackSubstitute(float **p_matrix, int p_size, int *p_index, float p_rightSide[]);

#ifdef __cplusplus
}
#endif

#endif // ZEF_FIXED_POINT

#endif //__ZEFYR_LIN_ALGEBRA_H
