/*
Company :		ZEFYR Technologies
File :			matchExport.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	09/05/2003

Description : SDK matching exported functions

Features : external interface functions		
    
Functions : 

	dpInitMatcher :	initialize matcher
	dpUnInitMatcher :	free matcher
	dpSetSecurity	: set the global security threshold
	dpGetSecurity	: get the global security threshold
	dpIdentifyTemplate : compared a template to many others
	dpSetDbMaxSearch	: set maximum seach depth during identification
	dpVerifyTemplate : Main function comparing two templates and returning if they match
	dpVerifyAndRefineTemplate : Matches two templates and if the matching score is good makes a composite template
	dpLoadParameters : load engine parameters

Necessary package : 

	zMaths.h
	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	tools.h
	matcher.h

Comments : none

*/

#ifndef __ZEFYR_MATCH_EXPORT_H
#define __ZEFYR_MATCH_EXPORT_H

#include "dynamicParameters.h"

#ifdef __cplusplus
extern "C" {
#endif


/**	Initializes precomputed data needed for matching
 *
 *  @return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise	
*/
INT32	WINAPI dpInitMatcher( void );

/**	Unitializes and deallocated precomputed data needed for matching 
 *
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise	
*/	
INT32	WINAPI dpUnInitMatcher( void );

// See "dpBio.h" for detailed description of these functions
INT32	WINAPI dpSetSecurity( UINT32 p_securityLevel );
INT32	WINAPI dpGetSecurity( UINT32 *p_securityLevel );
INT32	WINAPI dpIdentifyTemplate( UINT8 **p_templatesArray, UINT32 p_nbTemplates, UINT8 *p_templateBuffer, 
								  UINT32 *p_score, UINT32 *p_templatePos );
INT32	WINAPI dpSetDbMaxSearch( UINT32 p_maxSearch );
INT32	WINAPI dpVerifyTemplate( UINT8 *p_templateBuffer1, UINT8 *p_templateBuffer2, UINT32 *p_score );
INT32	WINAPI dpVerifyAndRefineTemplate( UINT8 *p_templateBuffer1, UINT8 *p_templateBuffer2, UINT32 *p_score, BOOL8 *p_refined );
INT32	WINAPI dpLoadParameters( UINT8 *p_buffer, UINT32 p_size );
INT32	WINAPI dpStopIdentification( UINT8 *p_templateBuffer );


#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_MATCH_EXPORT_H

