/*
Company :		ZEFYR Technologies
File :			matchRefine.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	02/07/2003

Description : Template enhancement routines
Features : Makes a composite template from 2 templates
    
Functions : 

	updateTemplate : Build a composite template with two matching and aligned templates
	updateOri : build a composite bloc orentation map
	updateCores : keep the best cores for a composite template

Necessary package : 

	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	zMaths.h
	tools.h
	tpsWarper.h
	quicksort.h

Comments : none

*/

#ifndef __ZEFYR_MATCH_REFINE_H
#define __ZEFYR_MATCH_REFINE_H

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/**	Mid level template "enhancement/compositing" function.
 *  Takes two aligned (matching) templates as input + a list of mathcing minutiae and outputs
 *	a new template which is a "mix" of the best part of the two input templates
 * 
 *	@param[out]		p_refinedTemplate		Enhanced composite template	
 *	@param[in]		p_templ1				Reference template		
 *	@param[in]		p_templ2				Aligned candidate template
 *	@param[in]		p_matches				2d Array of matching minutiae between p_templ1 and p_templ2
 *	@param[in]		p_nbMatches				number of matching minutiae pairs in p_matches
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32	updateTemplate( Template *p_refinedTemplate, Template *p_templ1, Template *p_templ2, INT32 **p_matches, INT32 p_nbMatches );

/**	Low level template "enhancement/compositing" function.
 *  Updates the Local orientation array part of the "refined" template.
 * 
 *	@param[in,out]	p_refinedTemplate		Enhanced composite template	
 *	@param[in]		p_templ1				Reference template	
 *	@param[in]		p_templ2				Aligned candidate template
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32	updateOri( Template *p_refinedTemplate, Template *p_templ1, Template *p_templ2 );

/**	Low level template "enhancement/compositing" function.
 *  Updates the core and deltas the "refined" template.
 * 
 *	@param[in,out]	p_refinedTemplate		Enhanced composite template	
 *	@param[in]		p_templ1				Reference template	
 *	@param[in]		p_templ2				Aligned candidate template
 *	@param[in]		p_matches				2d Array of matching minutiae between p_templ1 and p_templ2. 
 *											We don't need the size of this array as the 6 first array cells are
 *											reserved for core and delta matching pairs.
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32	updateCores( Template *p_refinedTemplate, Template *p_templ1, Template *p_templ2, INT32 **p_matches );

#ifdef __cplusplus
}
#endif

#endif //__ZEFYR_MATCH_REFINE_H
