/*
Company :		ZEFYR Technologies
File :			memMonitor.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	17/06/2005

Description : memory monitoring tools

Features : check if no memory leak was left + max mem usage, etc...	
    
Functions :

		MALLOC	: our version of malloc
		FREE	: our veriosn of free

Necessary package : 

	staticParameters.h

Comments :none

*/

#ifndef __ZEFYR_MEM_MONITOR_H
#define __ZEFYR_MEM_MONITOR_H

#include "customTypes.h"
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

// memory management MACROS
#if defined(MEM_DEBUG)
	/** Replacement macro for calloc that allow memory leaks tracking during debuging */
	#define CALLOC(nb, size) myCalloc( nb, size, __FILE__, __LINE__ )

	/** Replacement macro for malloc that allow memory leaks tracking during debuging */
	#define MALLOC(bloc) myMalloc( bloc, __FILE__, __LINE__ )

	/** Replacement macro for free that allow memory leaks tracking during debuging */
	#define FREE(bloc) myFree( bloc, __FILE__, __LINE__ )

	/** Macro that displays (on the console) the mamimum amount of memory that the application has allocated */
	#define GET_PEAK_MEM getPeakMem()

	/** Macro that displays (on the console) the current total amount of allocated memory */
	#define GET_ALLOCATED_MEM getAllocatedMem()

	
	/** Debugging malloc function
	 * 
	 *	@param[in]		p_size		Memory size requested
	 *	@param[in]		p_file		File where the memory allocation has be requested
	 *	@param[in]		p_line		Line number in file where memory allocation has be requested
	 * 
	 *	@return			A pointer to the allocated memory bloc			
	*/
	void	*myMalloc( size_t p_size, char *p_file, int p_line );

	/**	Debugging calloc function
	 * 
	 *	@param[in]		p_nb		Number of elements requested
	 *	@param[in]		p_size		Size of each element
	 *	@param[in]		p_file		File where the memory allocation has be requested	
	 *	@param[in]		p_line		Line number in file where memory allocation has be requested
	 * 
	 *	@return			A pointer to the allocated memory bloc					
	*/
	void	*myCalloc( size_t p_nb, size_t p_size,  char *p_file, int p_line );

	/**	Debugging free function
	 * 
	 *	@param[in]		p_pointer	Pointer whose memory must be freed	
	 *	@param[in]		p_file		File where the memory freeing has be requested	
	 *	@param[in]		p_line		Line number in file where memory freeing has be requested			
	*/
	void	myFree( void *p_pointer, char *p_file, int p_line );

	/**	Displays (on the console) the mamimum amount of memory that the application has allocated					
	*/
	void	getPeakMem();

	/**	Displays (on the console) the current total amount of allocated memory					
	*/
	void	getAllocatedMem();

#else

	/** Replacement macro for calloc that allow memory leaks tracking during debuging 
	 *  This one is a link to the builtin calloc function
	*/
	#define CALLOC(nb, size) calloc( nb, size)

	/** Replacement macro for malloc that allow memory leaks tracking during debuging 
	 *  This one is a link to the builtin malloc function
	*/
	#define MALLOC(bloc) malloc( bloc )

	/** Replacement macro for free that allow memory leaks tracking during debuging 
	 *  This one is a link to the builtin free function
	*/
	#define FREE(bloc) free( bloc )

	/** Macro that displays (on the console) the mamimum amount of memory that the application has allocated 
	 *  Doesn't do anything
	*/
	#define GET_PEAK_MEM 0

	/** Macro that displays (on the console) the current total amount of allocated memory 
	 *  Doesn't do anything
	*/
	#define GET_ALLOCATED_MEM 0

#endif


#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_MEM_MONITOR_H
