/*
Company :		ZEFYR Technologies
File :			portableMutex.h
Author :		Matthieu Perreira Da Silva
Fisrt release :	07/09/2005

Description : Portable mutex header library

Features : Allows to use mutex on different platform in order to have a thread safe bioEngine
  

Functions : 

	CREATE_MUTEX	: initializes the mutex
	LOCK_MUTEX		: take mutex ownership
	UNLOCK_MUTEX	: Release mutex ownership
	DESTROY_MUTEX	: Unitializes the mutex

Necessary package : none

Comments : ther will be different sets of macro for differents OSs / Platforms


*/

#ifndef __ZEFYR_PORTABLE_MUTEX_H
#define __ZEFYR_PORTABLE_MUTEX_H


#ifdef THREAD_SAFE

/*-------------------- Win32 mutex --------------------------*/
#ifdef WIN32
#include "windows.h"

/** mutex object */
#define MUTEX_OBJECT	HANDLE

/** mutex create macro */
#define CREATE_MUTEX( mutex )	mutex = CreateMutex( NULL, 0, NULL )

/** mutex locking macro */
#define LOCK_MUTEX( mutex )	WaitForSingleObject( (mutex), INFINITE )

/** mutex unlocking macro */
#define UNLOCK_MUTEX( mutex )	ReleaseMutex( (mutex) )

/** mutex freeing macro */
#define DESTROY_MUTEX( mutex )	CloseHandle ( (mutex) )

#endif //WIN32
/*------------------ end Win32 mutex ------------------------*/



/*-------------------- POSIX mutex --------------------------*/
#ifdef __GNUC__
#include "pthread.h"

/** mutex object */
#define MUTEX_OBJECT	pthread_mutex_t

/** mutex create macro */
#define CREATE_MUTEX( mutex )	pthread_mutex_init( &(mutex), NULL )

/** mutex locking macro */
#define LOCK_MUTEX( mutex )	pthread_mutex_lock( &(mutex) )

/** mutex unlocking macro */
#define UNLOCK_MUTEX( mutex )	pthread_mutex_unlock( &(mutex) )

/** mutex freeing macro */
#define DESTROY_MUTEX( mutex )	pthread_mutex_destroy ( &(mutex) )

#endif // __GNUC__
/*------------------ end POSIX mutex ------------------------*/

#else

/*------------------------- no mutex ------------------------*/

// we don't use mutex, but we still need to define marcos in order to compile correctly
// of course these macro don't do anything at all !

/** mutex object */
#define MUTEX_OBJECT	int

/** mutex create macro */
#define CREATE_MUTEX()	0

/** mutex locking macro */
#define LOCK_MUTEX( mutex )

/** mutex unlocking macro */
#define UNLOCK_MUTEX( mutex )

/** mutex freeing macro */
#define DESTROY_MUTEX( mutex )

/*----------------------- end no mutex ----------------------*/

#endif // THREAD_SAFE


#endif //__ZEFYR_PORTABLE_MUTEX_H