/*
Company :		ZEFYR Technologies
File :			skeleton.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	13/06/2003

Description : Skeletonization algorithm

Features : fast skeletonization		
    
Functions : 

	analyzeNeighborhood : internal function used to give information about a pixel (Black to White transition in neighboorhoud, etc.)
	skeletonization : main skeletonisation function, should be quite fast

Function prototypes :

	UINT32 analyzeNeighborhood( UINT8 *p_src, UINT32 p_width, UINT32 *p_nbNeighbors, UINT32 *p_nbTransitions );
	INT32 skeletonization( UINT8 *p_src, UINT8 *p_dst, UINT32 p_width, UINT32 p_height );

Necessary package : 

	customTypes.h
	staticParameters.h
	dynamicParameters.h
	errorCodes.h

Comments : none

*/

#ifndef __ZEFYR_SKELETON_H
#define __ZEFYR_SKELETON_H

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// ******** some usefll macros ********

/** Used by skeletonization to determine if a pixel can be erase or not. @see skeletonization */
#define C(p) ((((p)&80)==80)&&(((~(p))&39)==39)) || ((((p)&65)==65)&&(((~(p))&156)==156)) 
/** Used by skeletonization to determine if a pixel can be erase or not. @see skeletonization */
#define D(p) ((((p)&5)==5)&&(((~(p))&114)==114)) || ((((p)&20)==20)&&(((~(p))&201)==201)) 
/** Used by skeletonization to determine if a pixel can be erase or not. @see skeletonization */
#define E(p) !(((p)&20)&&(((p)&65)==65))
/** Used by skeletonization to determine if a pixel can be erase or not. @see skeletonization */
#define F(p) !(((p)&65)&&(((p)&20)==20))

// ******* Function prototypes ********

// Warning : this function returns a value that is not an error code.
// This value describes the neighborhood of a pixel

/**	Analyses the neighborhood of a pixel
 * 
 *	@param[in]		p_src				Address of the pixel to be analysed
 *	@param[in]		p_width				Width of the source image
 *	@param[out]		p_nbNeighbors		Number of neighboring pixels	
 *	@param[out]		p_nbTransitions		Number of black to white transitions
 * 
 *	@return			A code describing how the 3x3 pixel neighborhood is organized			
*/
UINT32 analyzeNeighborhood( UINT8 *p_src, UINT32 p_width, UINT32 *p_nbNeighbors, UINT32 *p_nbTransitions );


/**	Fast skeletonisation function.
 *  Uses a modified NWG algorithm to perform a fast image skeletonisation
 * 
 *	@param[in]		p_src		Source image
 *	@param[out]		p_dst		Skeletonized image
 *	@param[in]		p_width		Width of p_src
 *	@param[in]		p_height	Height of p_src	
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32 skeletonization( UINT8 *p_src, UINT8 *p_dst, UINT32 p_width, UINT32 p_height );

#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_SKELETON_H
