/*
Company :		ZEFYR Technologies
File :			tools.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	13/06/2003

Description : some usefull routines

Features : many	
    
Functions :

	templatePacker : fill an array of butes with template structure informations
	templateUnpacker : fill a template structure information with the content of an array of bytes
	templateInit : init template strucuture
	templateUninit : free memory allocated for template structure

Necessary package : 

	customTypes.h
	minutiae.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h

Comments :none

*/

#ifndef __ZEFYR_TOOLS_H
#define __ZEFYR_TOOLS_H

#include "customTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// ******** macros to write data to disk consistantly on every cpu architecture ********

/** write an INT32 value to an address using little indian bytes ordering (on any cpu architecture) */
#define WRITE_INT32( addr, data ) (addr)[0] = (data)&255; (addr)[1] = ((data)>>8)&255; (addr)[2] = ((data)>>16)&255; (addr)[3] = ((data)>>24)&255;
/** write an INT16 value to an address using little indian bytes ordering (on any cpu architecture) */
#define WRITE_INT16( addr, data ) (addr)[0] = (data)&255; (addr)[1] = ((data)>>8)&255;
/** write an INT16 value to an address using little indian bytes ordering (on any cpu architecture) */
#define WRITE_INT8( addr, data ) (addr)[0] = (data);

// ******** macros to read data from disk consistantly on every cpu architecture ********

/** read an INT32 value from an address using little indian bytes ordering (on any cpu architecture) */
#define READ_INT32( dest, addr ) (dest) = (addr)[0]; (dest) += ((addr)[1]<<8); (dest) += ((addr)[2]<<16); (dest) += ((addr[3])<<24);
/** read an INT16 value from an address using little indian bytes ordering (on any cpu architecture) */
#define READ_INT16( dest, addr ) (dest) = (addr)[0]; (dest) += ((addr)[1]<<8);
/** read an INT8 value from an address using little indian bytes ordering (on any cpu architecture) */
#define READ_INT8( dest, addr ) (dest) = (addr)[0];

// ******** function prototypes ********

/**	Pack a template structure into a UINT8 array.
 *  The maximum size of a packed template can be retrieved using the dpGetMaxTemplateSize function
 * 
 *	@param[in]		p_template	Source template	structure
 *	@param[out]		p_buffer	Destination packed buffer	
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise
 * 
 *	@see			dpGetMaxTemplateSize
*/
INT32 templatePacker( Template *p_template, UINT8 *p_buffer );

/**	Unpack data from a packed UINT8 array to a template structure
 * 
 *	@param[in]		p_buffer	Source packed buffer
 *	@param[out]		p_template	Destimation template structure	
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32 templateUnpacker( UINT8 *p_buffer, Template *p_template );

/**	Copy a template struture to another template structure
 * 
 *	@param[out]		p_dest	Destination template structure	
 *	@param[in]		p_src	Source template structure	
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32 templateCopy( Template *p_dest, Template *p_src );

/**	Initilializes a template structure
 * 
 *	@param[in,out]	p_template	Template structure that wil be initialized
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32 templateInit( Template *p_template );

/**	Uninitialize a template structure
 * 
 *	@param[in,out]	p_template	Template structure that wil be un-initialized
 * 
 *	@return			ZEF_OK if everything was OK. An error code listed in "errorCodes.h" otherwise			
*/
INT32 templateUninit( Template *p_template );

#ifdef __cplusplus
}
#endif

#endif // __ZEFYR_TOOLS_H
