/*
Company :		ZEFYR Technologies
File :			main.c
Author :		Matthieu Perreira Da Silva (based on Takuya OOURA (email: ooura@kurims.kyoto-u.ac.jp) algorithms) 
Fisrt release :	24/03/2004

Description : Main for fixed point tests


Functions : 

    main: main program

Function prototypes : 

    void main( void )

Necessary package : none

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "fixedPoint.h"
#include "staticParameters.h"
#include "FFT1D.h"
#include "FFT1Db.h"
#include "FFT2D.h"
#include "fxFFT1D.h"
#include "fxFFT1Db.h"
#include "fxFFT2D.h"
#include "BMPfiles.h"


void main( void )
{
	float *buffer;	
	FIXED16	*fxbuffer;
	unsigned char	*img;
	float	w[64];
	FIXED16	fxw[64];
	int		i,j,ip[64];
	int		width,height,size;
	clock_t	t1,t2;

	getBmpSize("test.bmp",&size);

	img = (unsigned char *)malloc(size*sizeof(unsigned char));
	buffer = (float *)malloc(2*size*sizeof(float));
	fxbuffer = (FIXED16 *)malloc(2*size*sizeof(FIXED32));

	loadBmp("test.bmp",img,size,&width,&height);

	for( i=0; i<size; i++ )
	{
		buffer[i] = (float)img[i]/16384.0f;
		fxbuffer[i] = img[i];
	}

	/*for( i=0; i<64; i++ )
	{
		fxbuffer[i] = 8*(rand()%256);
		buffer[i] = fxbuffer[i]/16384.0f;
		printf("%d\n",fxbuffer[i]);
	}*/

	for(i=0; i<64; i++ )
	{
		ip[i] = 0;
		fxw[i] = 0;
	}

	t1 = clock();
	for( j=0; j<1; j++ )
	{
		fxrdft2d(64,64, 1, fxbuffer, NULL, ip, fxw);
		fxbuffer[0] = 0;
		fxrdft2d(64,64, -1, fxbuffer, NULL, ip, fxw); 
		//fxrdft(64, 1, fxbuffer, ip, fxw);
		//fxrdft(64, -1, fxbuffer, ip, fxw); 
	}
	t2 = clock();
	printf("time fixed = %d\n",t2-t1);
	saveBmpS16( "testFx.bmp", fxbuffer, width, height,1,0 );

	for(i=0; i<64; i++ )
	{
		ip[i] = 0;
		w[i] = 0;
	}

	t1 = clock();
	for( j=0; j<1; j++ )
	{
		rdft2d(64,64, 1, buffer,NULL, ip, w );
		buffer[0] = 0;
		rdft2d(64,64, -1, buffer,NULL, ip, w );
		//rdft(64, 1, buffer, ip, w );
		//rdft(64, -1, buffer, ip, w );
	}
	t2 = clock();
	printf("time float = %d\n",t2-t1);
	saveBmpR( "testF.bmp", buffer, width, height );

	for( i=0; i<64; i++ ) 
	{
		printf("%f - %f\n", buffer[i], 2.0*TOFLT(fxbuffer[i],Q1_14));
	}

	free( buffer );
	free( img );
	free( fxbuffer);

}
