// TestMatchingView.cpp : implementation of the CTestMatchingView class
//

#include "stdafx.h"
#include "customTypes.h"
#include "staticParameters.h"
#include "TestMatching.h"
#include "TestMatchingDoc.h"
#include "TestMatchingView.h"
#include "gdiplus.h"
#include "math.h"
#include "monCode.h"
#include "matcher.h"
#include "fxMatcher.h"
#include "tpsWarper.h"
#include "fxTpsWarper.h"
#include "exported.h"
#include "enrollExport.h"
#include "matchExport.h"
#include "tools.h"
#include "dynamicParameters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace Gdiplus;

/////////////////////////////////////////////////////////////////////////////
// CTestMatchingView

IMPLEMENT_DYNCREATE(CTestMatchingView, CFormView)

BEGIN_MESSAGE_MAP(CTestMatchingView, CFormView)
	//{{AFX_MSG_MAP(CTestMatchingView)
	ON_BN_CLICKED(IDC_OPEN1, OnOpen1)
	ON_BN_CLICKED(IDC_OPEN2, OnOpen2)
	ON_BN_CLICKED(IDC_TRANSFERT, OnTransfert)
	ON_BN_CLICKED(IDC_REGISTER1, OnRegister1)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_ZOOM, OnReleasedcaptureZoom)
	ON_WM_TIMER()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_BN_CLICKED(IDC_CORESPONDANCES, OnCorespondances)
	ON_EN_CHANGE(IDC_DIST_CORES, OnChangeDistCores)
	ON_EN_CHANGE(IDC_THETA_CORES, OnChangeThetaCores)
	ON_BN_CLICKED(IDC_WARPER, OnWarper)
	ON_EN_CHANGE(IDC_ALPHA, OnChangeAlpha)
	ON_EN_CHANGE(IDC_NB_ITERATIONS, OnChangeNbIterations)
	ON_BN_CLICKED(IDC_DEFORM, OnDeform)
	ON_BN_CLICKED(IDC_QUALITY, OnQuality)
	ON_BN_CLICKED(IDC_ORIENT, OnOrient)
	ON_BN_CLICKED(IDC_ET_ORIENT, OnEtOrient)
	ON_EN_CHANGE(IDC_DX, OnChangeDx)
	ON_EN_CHANGE(IDC_DY, OnChangeDy)
	ON_EN_CHANGE(IDC_THETA, OnChangeTheta)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestMatchingView construction/destruction

CTestMatchingView::CTestMatchingView()
	: CFormView(CTestMatchingView::IDD)
{
	//{{AFX_DATA_INIT(CTestMatchingView)
	m_register1Msg = _T("");
	m_zoomTxt = _T("");
	m_distance = _T("");
	m_offset = _T("");
	m_distCores = 0;
	m_thetaCores = 0;
	m_nbCorres = _T("");
	m_nbIterations = 0;
	m_file1 = _T("");
	m_file2 = _T("");
	m_diffs = _T("");
	m_dx = 0;
	m_dy = 0;
	m_angle = 0.0f;
	m_qual1 = _T("");
	m_qual2 = _T("");
	m_freq = _T("");
	m_alpha = 0.0f;
	m_scoreIndex = _T("");
	m_index1 = _T("");
	m_index2 = _T("");
	m_nbMinuties1 = _T("");
	m_nbMinuties2 = _T("");
	//}}AFX_DATA_INIT
	
	dpInit();

	m_gabLoaded[0] = m_gabLoaded[1] = false;
	m_moving = false;
	m_initGdiPlus.Initialize();

	m_gabarits[0] = new unsigned char[10*dpGetMaxTemplateSize()];
	m_gabarits[1] = new unsigned char[10*dpGetMaxTemplateSize()];

	m_concordances = new int*[256];
	for( int i=0; i<256; i++ )
		m_concordances[i] = new int[2];
	
}

CTestMatchingView::~CTestMatchingView()
{
	delete[] m_gabarits[0];
	delete[] m_gabarits[1];

	for( int i=0; i<256; i++ )
		delete[] m_concordances[i];
	delete[] m_concordances;

	dpUnInit();
	m_initGdiPlus.Deinitialize();

}

void CTestMatchingView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTestMatchingView)
	DDX_Control(pDX, IDC_AFFICHE_LOCAL, m_afficheLocaux);
	DDX_Control(pDX, IDC_CORESPONDANCES, m_coresp);
	DDX_Control(pDX, IDC_DISPLAY2, m_display2);
	DDX_Control(pDX, IDC_DISPLAY1, m_display1);
	DDX_Control(pDX, IDC_REGISTER1, m_register1);
	DDX_Control(pDX, IDC_TRANSFERT, m_transfert);
	DDX_Control(pDX, IDC_ZOOM, m_sliderZoom);
	DDX_Text(pDX, IDC_ZOOM_TXT, m_zoomTxt);
	DDX_Text(pDX, IDC_DISTANCE, m_distance);
	DDX_Text(pDX, IDC_OFFSET, m_offset);
	DDX_Text(pDX, IDC_DIST_CORES, m_distCores);
	DDV_MinMaxInt(pDX, m_distCores, 0, 255);
	DDX_Text(pDX, IDC_THETA_CORES, m_thetaCores);
	DDV_MinMaxInt(pDX, m_thetaCores, 0, 255);
	DDX_Text(pDX, IDC_NB_CORRES, m_nbCorres);
	DDX_Text(pDX, IDC_NB_ITERATIONS, m_nbIterations);
	DDV_MinMaxInt(pDX, m_nbIterations, 1, 100);
	DDX_Text(pDX, IDC_FILE1, m_file1);
	DDX_Text(pDX, IDC_FILE2, m_file2);
	DDX_Text(pDX, IDC_DIFFS, m_diffs);
	DDX_Text(pDX, IDC_DX, m_dx);
	DDX_Text(pDX, IDC_DY, m_dy);
	DDX_Text(pDX, IDC_THETA, m_angle);
	DDX_Text(pDX, IDC_QUAL1, m_qual1);
	DDX_Text(pDX, IDC_QUAL2, m_qual2);
	DDX_Text(pDX, IDC_FREQ, m_freq);
	DDX_Text(pDX, IDC_ALPHA, m_alpha);
	DDV_MinMaxFloat(pDX, m_alpha, 0.f, 100000.f);
	DDX_Text(pDX, IDC_SCORE_INDEX, m_scoreIndex);
	DDX_Text(pDX, IDC_INDEX1, m_index1);
	DDX_Text(pDX, IDC_INDEX2, m_index2);
	DDX_Text(pDX, IDC_NB_MINUTIES1, m_nbMinuties1);
	DDX_Text(pDX, IDC_NB_MINUTIES2, m_nbMinuties2);
	//}}AFX_DATA_MAP
}

BOOL CTestMatchingView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CTestMatchingView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

	m_sliderZoom.SetRange(1,100);
	m_sliderZoom.SetPos(50);

	m_decX=m_decY=0;

	SetTimer(123456,50,NULL);

	m_offset.Format("Offset : x = %d , y = %d",0,0);
	m_zoomTxt.Format("Echelle : %.2f",1.0);
	m_distance.Format("Distance : %.2f",0);
	m_diffs.Format("Diffs : %.2f",0);
	m_nbIterations = 1;
	m_alpha = 0.5;
	m_distCores = 12;
	m_thetaCores = 24;

	UpdateData(false);
}

/////////////////////////////////////////////////////////////////////////////
// CTestMatchingView printing

BOOL CTestMatchingView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTestMatchingView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTestMatchingView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CTestMatchingView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// CTestMatchingView diagnostics

#ifdef _DEBUG
void CTestMatchingView::AssertValid() const
{
	CFormView::AssertValid();
}

void CTestMatchingView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CTestMatchingDoc* CTestMatchingView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTestMatchingDoc)));
	return (CTestMatchingDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestMatchingView message handlers

void CTestMatchingView::OnOpen1() 
{
	int nb,q;
	Coord16		centre;
	float		angle;
	CString		str="";

	m_file1 = chargerGabarit( m_gabarits[0] );
	if( m_file1.Compare("Erreur") )
	{
		q=dpGetTemplateQuality(m_gabarits[0]);
		m_qual1.Format("Qualit : %d",q);
		m_gabLoaded[0] = true;
		recalcTransform();
		RedrawWindow();	
	}
	UpdateData(false);
}

void CTestMatchingView::OnOpen2() 
{
	int nb,q;
	Coord16		centre;
	float		angle;


	m_file2 = chargerGabarit( m_gabarits[1] );
	if( m_file2.Compare("Erreur")  )
	{
		q=dpGetTemplateQuality(m_gabarits[1]);
		m_qual2.Format("Qualit : %d",q);
		m_gabLoaded[1] = true;
		recalcTransform();
		RedrawWindow();	
	}
	UpdateData(false);
}

void CTestMatchingView::OnDraw(CDC* pDC) 
{	
	PAINTSTRUCT	ps;
	Pen			pen(Color(255,196,196),1.0);
	SolidBrush	brush(Color(255,255,255));
	int			i,nb,oldNb;
	int			xCenter=200,yCenter=200;
	int			displays[2] = {IDC_DISPLAY1,IDC_DISPLAY2};
	CDC			*dc;
	CRect		rect;
	Template	temp[2];
	float		zoom = 50.0/float(m_sliderZoom.GetPos());
	float		txFreq,alpha;
	TpsData		tpsData;
	CString		str,str2;
	unsigned int	diffs,diffsTmp,score;

	if( m_gabLoaded[0] && m_gabLoaded[1] ) 
	{
		templateInit(&(temp[0]));
		templateInit(&(temp[1]));
		templateUnpacker(m_gabarits[0], &(temp[0]) );
		templateUnpacker(m_gabarits[1], &(temp[1]) );

		m_nbMinuties1.Format("Nb Minuties : %d",temp[0].m_nbMinutiae );
		m_nbMinuties2.Format("Nb Minuties : %d",temp[1].m_nbMinutiae );
		UpdateData(false);
			
		str="";
		for( i=0; i<16; i++ )
		{
			str2.Format("%d - ",temp[0].m_sign[i]-127);
			str += str2;
		}
		m_index1.Format("Index1 : %s",str);

		str="";
		for( i=0; i<16; i++ )
		{
			str2.Format("%d - ",temp[1].m_sign[i]-127);
			str += str2;
		}
		m_index2.Format("Index2 : %s",str);

		nb = scoreIndex( &(temp[0]), &(temp[1]) );
		m_scoreIndex.Format("Score Index : %d", nb );
			
		txFreq = abs(temp[0].m_meanFreq-temp[1].m_meanFreq);
		m_freq.Format("Ecart Frequence : %.2f (%d;%d)",txFreq/100,temp[0].m_meanFreq,temp[1].m_meanFreq);

		// on effectue les transfos
		if( IsDlgButtonChecked( IDC_REGISTER1 ) || IsDlgButtonChecked( IDC_WARPER ) )
		{
			applyRegistration( &(temp[1]), m_dx, m_dy, m_angle );
		
			diffs = 0;
			nb = findMatches( &(temp[0]),&(temp[1]), (int**)m_concordances, &nb );
			if( nb-(2*g_params.m_maxNumberOfCores) > g_params.m_minNumberOfMinutiae )
			{
				if( IsDlgButtonChecked( IDC_WARPER ) )
				{
					oldNb = nb;
#ifdef ZEF_FIXED_POINT
alpha = FTOFIX32(m_alpha,Q15_16); // on doit passer un paramtre en Q15_16  la version virguel fixe de estimateDeformations
#else
alpha = m_alpha;
#endif // ZEF_FIXED_POINT
					estimateDeformations( &(temp[0]), &(temp[1]), (int**)m_concordances, nb, alpha, &tpsData );
					applyDeformations( &(temp[1]), &diffsTmp,&tpsData );
					diffs+=diffsTmp;
					findMatches( &(temp[0]),&(temp[1]),(int**)m_concordances, &nb );	
					while( nb > oldNb )
					{
						oldNb = nb;
						estimateDeformations( &(temp[0]), &(temp[1]), (int**)m_concordances, nb, alpha, &tpsData );
						applyDeformations( &(temp[1]), &diffsTmp,&tpsData );
						diffs+=diffsTmp;
						findMatches( &(temp[0]),&(temp[1]),(int**)m_concordances, &nb );	
					}
				}	
				
			}

			findMatches( &(temp[0]),&(temp[1]),(int**)m_concordances, &nb );

			computeScore(&(temp[0]),&(temp[1]),(int**)m_concordances,nb,&score);
		
			int nb1 = matchOri(&(temp[0]),&(temp[1]));
			m_nbCorres.Format("Score : %d - %d",score,nb1);
			m_diffs.Format("Diffs : %d",diffs);
			
			UpdateData(false);
		
		}	
		// boucle sur les 2 gabarits
		for( i=0; i<2; i++ )
		{
			// si le gabarit est charg
			if( m_gabLoaded[i] )
			{
				CDC			MemDC;
				CBitmap		MemBitmap;

				GetDlgItem(displays[i])->InvalidateRect(NULL);
				dc = GetDlgItem(displays[i])->BeginPaint( &ps );
				GetDlgItem(displays[i])->GetClientRect(&rect);

				MemDC.CreateCompatibleDC(dc);
				MemBitmap.CreateCompatibleBitmap(dc,rect.right,rect.bottom);
				MemDC.SelectObject(&MemBitmap);

				Graphics	graphics(MemDC.m_hDC);

				xCenter = rect.right/2+m_decX;
				yCenter = rect.bottom/2+m_decY;

				// rectangle blanc
				graphics.Clear(Color(255,255,255));

				// quality
				if( (GetCheckedRadioButton( IDC_QUALITY, IDC_ET_ORIENT )) == IDC_QUALITY )
					afficherQuality(temp[i],graphics,xCenter,yCenter, zoom );

				if( (GetCheckedRadioButton( IDC_QUALITY, IDC_ET_ORIENT )) == IDC_ORIENT )
					afficherOrient(temp[i],Color(255,255,128),graphics,xCenter,yCenter, zoom );

				// lignes de repre
				pen.SetColor( Color(255,196,196) );
				graphics.DrawLine(&pen,0,yCenter,rect.right,yCenter);
				graphics.DrawLine(&pen,xCenter,0,xCenter,rect.bottom);

				// affichage des minuties
				if( i== 0 )
				{
					afficherMinuties( temp[i], graphics, Color(0,255,0), xCenter,yCenter, zoom );
					if( IsDlgButtonChecked( IDC_TRANSFERT ) )
					{
						if( (GetCheckedRadioButton( IDC_QUALITY, IDC_ET_ORIENT )) == IDC_ORIENT )
							afficherOrient(temp[i+1],Color(255,200,255),graphics,xCenter,yCenter, zoom );
						afficherMinuties( temp[i+1], graphics, Color(0,0,255), xCenter,yCenter, zoom );
					}
				}
				else
				{
					if( IsDlgButtonChecked( IDC_DEFORM ) && IsDlgButtonChecked( IDC_WARPER ) )
						afficherDeformations(graphics,xCenter,yCenter,zoom);
					afficherMinuties( temp[i], graphics, Color(0,0,255), xCenter,yCenter, zoom );
				}

				if( IsDlgButtonChecked( IDC_CORESPONDANCES ) && m_gabLoaded[0] && m_gabLoaded[1] && IsDlgButtonChecked( IDC_TRANSFERT ) )
				{
					if( i== 0 )
					{
						afficherMinutiesConcordantes( m_concordances,i+1,temp[i+1],temp[i],graphics, xCenter,yCenter, zoom );
						afficherMinutiesConcordantes( m_concordances,i,temp[i],temp[i+1],graphics, xCenter,yCenter, zoom );
					}
					else
						afficherMinutiesConcordantes( m_concordances,i,temp[i],temp[i-1],graphics, xCenter,yCenter, zoom );
				}

					// on affiche les concordances
				if( i==0 && IsDlgButtonChecked( IDC_CORESPONDANCES ) && m_gabLoaded[0] && m_gabLoaded[1] && IsDlgButtonChecked( IDC_TRANSFERT ) )
				{
					afficherConcordances(m_concordances,temp[0],temp[1],graphics,xCenter,yCenter,zoom);	
				}

				dc->BitBlt(0,0,rect.right,rect.bottom,&MemDC,0,0,SRCCOPY);

				GetDlgItem(displays[i])->EndPaint( &ps );
				//GetDlgItem(displays[i])->ReleaseDC(dc);
			}
		}

		templateUninit(&(temp[0]));
		templateUninit(&(temp[1]));

	}
}

void CTestMatchingView::OnTransfert() 
{
	OnDraw(NULL);	
}

void CTestMatchingView::OnRegister1() 
{
	Coord16		centre;
	float		angle;

	if( IsDlgButtonChecked( IDC_REGISTER1 ) )
	{
		recalcTransform();
	}
	else
	{
		recalcTransform();

		m_angle = 0;
		m_dx = 0;
		m_dy = 0;

		m_register1Msg.Format("Angle=%.2f  -  dx=%d  -  dy=%d",m_angle,m_dx,m_dy);

		UpdateData(false);
	}
	

	OnDraw(NULL);
}

void CTestMatchingView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRect	rect1,rect2;

	m_display1.GetWindowRect(&rect1);
	m_display2.GetWindowRect(&rect2);

	ClientToScreen(&point);

	if( rect1.PtInRect(point) || rect2.PtInRect(point) )
	{
		m_mouseXL = point.x;
		m_mouseYL = point.y;
		m_moving = true;
	}
	
	CFormView::OnLButtonDown(nFlags, point);
}

void CTestMatchingView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	OnDraw(NULL);
	m_moving = false;
	
	CFormView::OnLButtonUp(nFlags, point);
}

void CTestMatchingView::OnMouseMove(UINT nFlags, CPoint point) 
{
	float dx,dy;
	CPoint myPoint1,myPoint2;

	myPoint1 = point;
	ClientToScreen(&point);
	if( nFlags&MK_RBUTTON  && m_moving )
	{
		m_decX += point.x - m_mouseX;
		m_decY += point.y - m_mouseY;
		m_mouseX = point.x;
		m_mouseY = point.y;

		m_offset.Format("Offset : x = %d , y = %d",m_decX,m_decY);
		UpdateData(false);
		OnDraw(NULL);
	}

	if( nFlags&MK_LBUTTON  && m_moving )
	{
		myPoint2.x = m_mouseXL;
		myPoint2.y = m_mouseYL;

		ScreenToClient(&myPoint2);

		dx = (point.x - m_mouseXL) / (50.0/float(m_sliderZoom.GetPos()));
		dy = (point.y - m_mouseYL) / (50.0/float(m_sliderZoom.GetPos()));

		OnDraw(NULL);
		Graphics	graphics(GetDC()->m_hDC);
		Pen			pen(Color(255,128,128),1.0);
		graphics.DrawLine(&pen,myPoint1.x,myPoint1.y,myPoint2.x,myPoint2.y);


		m_distance.Format("Distance : %.2f",sqrt(dx*dx + dy*dy));
		UpdateData(false);
	}

	CFormView::OnMouseMove(nFlags, point);
}

void CTestMatchingView::OnReleasedcaptureZoom(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDraw(NULL);

	m_zoomTxt.Format("Echelle : %.2f",50.0/float(m_sliderZoom.GetPos()));

	UpdateData(false);
	
	*pResult = 0;
}

void CTestMatchingView::OnTimer(UINT nIDEvent) 
{
	static float oldZoom=1.0;
	float zoom;

	if( nIDEvent = 123456 )
	{
		zoom = 50.0/float(m_sliderZoom.GetPos());
		if( zoom != oldZoom )
		{
			oldZoom = zoom;
			m_zoomTxt.Format("Echelle : %.2f",zoom);
			UpdateData(false);
			OnDraw(NULL);
		}
	}
	
	CFormView::OnTimer(nIDEvent);
}

void CTestMatchingView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CRect	rect1,rect2;

	m_display1.GetWindowRect(&rect1);
	m_display2.GetWindowRect(&rect2);

	ClientToScreen(&point);

	if( rect1.PtInRect(point) || rect2.PtInRect(point) )
	{
		m_mouseX = point.x;
		m_mouseY = point.y;
		m_moving = true;
	}
	
	CFormView::OnRButtonDown(nFlags, point);
}

void CTestMatchingView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	m_moving = false;	
	
	CFormView::OnRButtonUp(nFlags, point);
}

void CTestMatchingView::OnCorespondances() 
{
	UpdateData(true);
	recalcTransform();
	OnDraw(NULL);	
}

void CTestMatchingView::OnChangeDistCores() 
{
	UpdateData(true);
	g_params.m_translationTolerance = m_distCores;
	recalcTransform();
	OnDraw(NULL);
	
}

void CTestMatchingView::OnChangeThetaCores() 
{
	UpdateData(true);
	g_params.m_angleTolerance = m_thetaCores;
	recalcTransform();
	OnDraw(NULL);

}

void CTestMatchingView::recalcTransform()
{
	int			nb,dx,dy,angle;
	unsigned int score;	
	Template	temp[2];

	if( m_gabLoaded[0] && m_gabLoaded[1] )
	{

		templateInit(&(temp[0]));
		templateInit(&(temp[1]));
		templateUnpacker(m_gabarits[0], &(temp[0]));
		templateUnpacker(m_gabarits[1], &(temp[1]));

		if( IsDlgButtonChecked( IDC_REGISTER1 ) )
		{
			registerMinutiae(&(temp[0]), &(temp[1]), &dx , &dy, &angle );
			//findRotation( &(temp[0]), &(temp[1]), &angle );
			m_angle = angle;
			m_dx = dx;
			m_dy = dy;
		
			applyRegistration(&temp[1], dx, dy, angle );
			//m_register1Msg.Format("Angle=%.2f  -  dx=%d  -  dy=%d",m_angle,m_dx,m_dy);
		}
		if( IsDlgButtonChecked( IDC_CORESPONDANCES ) )
		{
			nb = findMatches( &(temp[0]),&(temp[1]), (int**)m_concordances, &nb );
			if( !IsDlgButtonChecked( IDC_WARPER ) )
			{
				if( nb-6 > 3 )
					nb = computeScore(&(temp[0]),&(temp[1]),(int**)m_concordances,nb,&score);
				else
					nb = 0;
				m_nbCorres.Format("Score : %d",nb);
			}
		}

		templateUninit(&(temp[0]));
		templateUninit(&(temp[1]));

		UpdateData(false);

	}

}

void CTestMatchingView::OnWarper() 
{
	CheckDlgButton(IDC_CORESPONDANCES,1);
	UpdateData(true);
	recalcTransform();
	OnDraw(NULL);
}

void CTestMatchingView::OnChangeAlpha() 
{
	UpdateData(true);
	recalcTransform();
	OnDraw(NULL);	
}

void CTestMatchingView::OnChangeNbIterations() 
{
	UpdateData(true);
	recalcTransform();
	OnDraw(NULL);
	
}

void CTestMatchingView::OnDeform() 
{
	UpdateData(true);
	recalcTransform();
	OnDraw(NULL);
}

void CTestMatchingView::OnQuality() 
{
	OnDraw(NULL);
	
}

void CTestMatchingView::OnOrient() 
{
	OnDraw(NULL);	
}

void CTestMatchingView::OnEtOrient() 
{
	OnDraw(NULL);	
}

void CTestMatchingView::OnChangeDx() 
{
	UpdateData(true);
	OnDraw(NULL);	
	
}

void CTestMatchingView::OnChangeDy() 
{
	UpdateData(true);
	OnDraw(NULL);	
	
}

void CTestMatchingView::OnChangeTheta() 
{
	UpdateData(true);
	OnDraw(NULL);	
	
}
