/*	Fichier monCode.cpp
	Fonctions pour TestMatching
	Zefyr Technologie 2000
	Matthieu Perreira Da Silva
*/

#include "stdafx.h"
#include "customTypes.h"
#include "gdiplus.h"
#include "matcher.h"
#include "zMaths.h"

#define M_PI 3.14159265358979323846

using namespace Gdiplus;

CString chargerGabarit( unsigned char *p_gabarit )
{
	CFileDialog	fileDialog(true,".bin",NULL,OFN_FILEMUSTEXIST,"Gabarits ZEFYR (*.bin)|*.bin||",NULL);
	CFile		file;
	unsigned int size;

	if( fileDialog.DoModal() == IDOK )
	{
		file.Open(fileDialog.GetPathName(),CFile::modeRead);
		file.Read(&size,sizeof(unsigned int));
		file.SeekToBegin();
		file.Read(p_gabarit,size);
		file.Close();
		return fileDialog.GetFileName();
	}

	return CString("Erreur");
}

void afficherMinuties( Template &p_gabarit, Graphics &p_graphics, Color &p_color, int p_xCentre, int p_yCentre, float zoom )
{
	//Pen			pen(p_color,2.0);
	Pen			bordure(p_color,1.0);
	Pen			blackPen( Color(0,0,0), 2.0 );
	int			j,x1,y1,x2,y2;
	float		theta;
	unsigned int colorValue;

	x1 = p_xCentre + zoom*(p_gabarit.m_boundBox.m_left);
	y1 = p_yCentre + zoom*(p_gabarit.m_boundBox.m_top);
	x2 = zoom*(p_gabarit.m_boundBox.m_right-p_gabarit.m_boundBox.m_left);
	y2 = zoom*(p_gabarit.m_boundBox.m_bottom-p_gabarit.m_boundBox.m_top);
	p_graphics.DrawRectangle(&bordure,x1,y1, x2,y2);

	// on parcourt toutes les minutiae pour tracer les orientations
	for( j=0; j<p_gabarit.m_nbMinutiae; j++ )
	{	
		theta = M_PI*(p_gabarit.m_minutiae[j].m_ori)/128.0;

		colorValue = p_color.GetValue();
		colorValue &= 0xFFFFFF;
		colorValue |= (p_gabarit.m_minutiae[j].m_qual)<<24;
		Pen pen(Color(colorValue), 2.0 );
		


		// ligne horizontale
		x1 = p_xCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_x);
		y1 = p_yCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_y);
		x2 = p_xCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_x + 8*cos(theta));
		y2 = p_yCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_y + 8*sin(theta));
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);

	}

	// on parcourt toutes les minutiae pour les tracer 
	for( j=0; j<p_gabarit.m_nbMinutiae; j++ )
	{	
		colorValue = 0;
		colorValue |= (p_gabarit.m_minutiae[j].m_qual)<<24;
		Pen pen(Color(colorValue), 2.0 );


		// ligne horizontale
		x1 = p_xCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_x -2);
		x2 = p_xCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_x +2);
		y1 = y2 = p_yCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_y);
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);

		// ligne verticale
		y1 = p_yCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_y -2);
		y2 = p_yCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_y +2);
		x1 = x2 = p_xCentre + zoom*(p_gabarit.m_minutiae[j].m_coord.m_x);
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);
	}

	// on affiche les cores
	for( j=0 ; j<p_gabarit.m_nbCores; j++ )
	{
		Pen			pen(p_color,2.0);
		x1 = p_xCentre + zoom*(p_gabarit.m_cores[j].m_x-2);
		y1 = p_yCentre + zoom*(p_gabarit.m_cores[j].m_y-2);
		x2 = zoom*4;
		y2 = zoom*4;
		p_graphics.DrawEllipse(&pen, x1,y1,x2,y2 );
	}

	// draw core Ori
	if( p_gabarit.m_nbCores > 0 )
	{
		Pen			pen(p_color,2.0);
		x1 = p_xCentre + zoom*(p_gabarit.m_cores[0].m_x-2);
		y1 = p_yCentre + zoom*(p_gabarit.m_cores[0].m_y-2);
		x2 = p_xCentre + zoom*(p_gabarit.m_cores[0].m_x-2+16*cos(M_PI*p_gabarit.m_oriCore/128));
		y2 = p_yCentre + zoom*(p_gabarit.m_cores[0].m_y-2+16*sin(M_PI*p_gabarit.m_oriCore/128));
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);
	}

	// et les deltas
	for( j=0 ; j<p_gabarit.m_nbDeltas; j++ )
	{
		Pen			pen(p_color,2.0);
		x1 = p_xCentre + zoom*(p_gabarit.m_deltas[j].m_x-2);
		y1 = p_yCentre + zoom*(p_gabarit.m_deltas[j].m_y-2);
		x2 = zoom*4;
		y2 = zoom*4;
		p_graphics.DrawRectangle(&pen, x1,y1,x2,y2 );
	}

}



void afficherConcordances( int **p_concordances, Template &p_gabarit1, Template &p_gabarit2, Graphics &p_graphics, int p_xCentre, int p_yCentre, float zoom )
{
	Pen			pen(Color(255,140,0),2.0);
	int			i=0,x1,y1,x2,y2;
	int			maxNbCores,maxNbDeltas;

	maxNbCores = (p_gabarit1.m_nbCores > p_gabarit2.m_nbCores) ? p_gabarit1.m_nbCores : p_gabarit2.m_nbCores;
	maxNbDeltas = (p_gabarit1.m_nbDeltas> p_gabarit2.m_nbDeltas) ? p_gabarit1.m_nbDeltas : p_gabarit2.m_nbDeltas;

	// les cores
	for( i=0; i<maxNbCores; i++ )
	{
		if( p_concordances[i][0] != -1 )
		{
			x1 = p_xCentre + zoom*(p_gabarit1.m_cores[p_concordances[i][0]].m_x);
			x2 = p_xCentre + zoom*(p_gabarit2.m_cores[p_concordances[i][1]].m_x);

			y1 = p_yCentre + zoom*(p_gabarit1.m_cores[p_concordances[i][0]].m_y);
			y2 = p_yCentre + zoom*(p_gabarit2.m_cores[p_concordances[i][1]].m_y);

			p_graphics.DrawLine(&pen,x1,y1,x2,y2);
		}
	}

	// les deltas
	for( i=maxNbCores; i<maxNbCores+maxNbDeltas; i++ )
	{
		if( p_concordances[i][0] != -1 )
		{
			x1 = p_xCentre + zoom*(p_gabarit1.m_deltas[p_concordances[i][0]].m_x);
			x2 = p_xCentre + zoom*(p_gabarit2.m_deltas[p_concordances[i][1]].m_x);

			y1 = p_yCentre + zoom*(p_gabarit1.m_deltas[p_concordances[i][0]].m_y);
			y2 = p_yCentre + zoom*(p_gabarit2.m_deltas[p_concordances[i][1]].m_y);

			p_graphics.DrawLine(&pen,x1,y1,x2,y2);
		}
	}

	// on parcourt toutes les minutiae pour tracer les orientations
	i=maxNbCores+maxNbDeltas;
	while( p_concordances[i][0] != -1 )
	{
		x1 = p_xCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][0]].m_coord.m_x);
		x2 = p_xCentre + zoom*(p_gabarit2.m_minutiae[p_concordances[i][1]].m_coord.m_x);

		y1 = p_yCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][0]].m_coord.m_y);
		y2 = p_yCentre + zoom*(p_gabarit2.m_minutiae[p_concordances[i][1]].m_coord.m_y);

		p_graphics.DrawLine(&pen,x1,y1,x2,y2);

		i++;
	}
}

void afficherMinutiesConcordantes( int **p_concordances, int p_index, Template &p_gabarit1, Template &p_gabarit2, Graphics &p_graphics, int p_xCentre, int p_yCentre, float zoom )
{
	Pen			pen(Color(255,0,0),1.0);
	int			i,x1,y1,x2,y2;
	int			maxNbCores,maxNbDeltas;

	maxNbCores = (p_gabarit1.m_nbCores > p_gabarit2.m_nbCores) ? p_gabarit1.m_nbCores : p_gabarit2.m_nbCores;
	maxNbDeltas = (p_gabarit1.m_nbDeltas> p_gabarit2.m_nbDeltas) ? p_gabarit1.m_nbDeltas : p_gabarit2.m_nbDeltas;

	i = maxNbCores+maxNbDeltas;

	// on parcourt toutes les minutiae pour les tracer 
	while( p_concordances[i][p_index] != -1 )
	{	
		// ligne horizontale
		x1 = p_xCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_x -2);
		x2 = p_xCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_x +2);
		y1 = y2 = p_yCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_y);
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);

		// ligne verticale
		y1 = p_yCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_y -2);
		y2 = p_yCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_y +2);
		x1 = x2 = p_xCentre + zoom*(p_gabarit1.m_minutiae[p_concordances[i][p_index]].m_coord.m_x);
		p_graphics.DrawLine(&pen,x1,y1,x2,y2);

		i++;
	}

}

void afficherDeformations( Graphics &p_graphics, int p_xCentre, int p_yCentre, float zoom )
{
	Pen		pen(Color(196,196,196),1.0);
	int		y,x;
	float	yF,xF;
	float   oldY,oldX;

	for( y=-300; y<300; y+=16 )
	{
		oldY = p_yCentre + zoom*y;
		oldX = p_xCentre + zoom*(-300);
		for( x=-300; x<300; x+=16 )
		{
			yF = y; xF = x;
			//warp( xF,yF );
			xF = p_xCentre + zoom*xF;
			yF = p_yCentre + zoom*yF;
			p_graphics.DrawLine(&pen,oldX,oldY,xF,yF);
			oldX = xF;
			oldY = yF;
		}
	}

	for( x=-300; x<300; x+=16 )
	{
		oldX = p_xCentre + zoom*x;
		oldY = p_yCentre + zoom*(-300);
		for( y=-300; y<300; y+=16 )
		{
			yF = y; xF = x;
			//warp( xF,yF );
			xF = p_xCentre + zoom*xF;
			yF = p_yCentre + zoom*yF;
			p_graphics.DrawLine(&pen,oldX,oldY,xF,yF);
			oldX = xF;
			oldY = yF;
		}
	}
}

void afficherQuality( Template &p_gabarit, Graphics &p_graphics, int p_xCentre, int p_yCentre, float zoom )
{
	int		y,x;
	float	yF,xF;
	int		val;
	
	for( y=0; y<p_gabarit.m_localH; y++ )
	{
		for(x=0; x<p_gabarit.m_localW; x++ )
		{
			val = p_gabarit.m_qual[y*p_gabarit.m_localW+x]*16;
			Color	   color(255-val,255-val,255-val);
			SolidBrush brush(color);

			xF = p_gabarit.m_localShiftX + (x-p_gabarit.m_localW/2)*p_gabarit.m_blocSize;
			yF = p_gabarit.m_localShiftY + (y-p_gabarit.m_localH/2)*p_gabarit.m_blocSize;

			xF = p_xCentre + zoom*xF;
			yF = p_yCentre + zoom*yF;

			p_graphics.FillRectangle(&brush,xF,yF,p_gabarit.m_blocSize*zoom,p_gabarit.m_blocSize*zoom);
		}
	}
	
	
}


void afficherOrient( Template &p_gabarit, Color &p_color, Graphics &p_graphics, int p_xCentre, int p_yCentre, float zoom )
{
	int		y,x;
	float	yF,xF;
	float	val;
	int		size2, size4;

	size2 = p_gabarit.m_blocSize/2;
	size4 = p_gabarit.m_blocSize/4;
	
	for( y=0; y<p_gabarit.m_localH; y++ )
	{
		for(x=0; x<p_gabarit.m_localW; x++ )
		{
				Color	   color(196,196,196);

				if( p_gabarit.m_qual[y*p_gabarit.m_localW+x] != 0 )
					color = p_color;
	
				Pen			pen(color,2);

				val = p_gabarit.m_ori[y*p_gabarit.m_localW+x];
				

				xF = p_gabarit.m_localShiftX + (x-p_gabarit.m_localW/2)*p_gabarit.m_blocSize+p_gabarit.m_blocSize/2;
				yF = p_gabarit.m_localShiftY + (y-p_gabarit.m_localH/2)*p_gabarit.m_blocSize+p_gabarit.m_blocSize/2;

				xF = p_xCentre + zoom*xF;
				yF = p_yCentre + zoom*yF;

				p_graphics.TranslateTransform(xF,yF);
				p_graphics.RotateTransform(val*180/255.0);
				p_graphics.ScaleTransform(zoom,zoom);
				p_graphics.DrawLine(&pen,-size2,-size4,size2,-size4);
				p_graphics.DrawLine(&pen,-size2,0,size2,0);
				p_graphics.DrawLine(&pen,-size2,+size4,size2,+size4);
				p_graphics.ResetTransform();
		}
	}
	
	
}