// BenchPage.cpp : implementation file
//

#include "stdafx.h"
#include "dpBioDemo.h"
#include "BenchPage.h"
#include "dpBioDemoView.h"
#include "FolderDlg.h"
#include "myFuncs.h"
#include "ProgressDlg.h"
#include "HiResCounter.h"
#include "dpBio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CDpBioMainView* m_demoView;

/////////////////////////////////////////////////////////////////////////////
// CBenchPage property page

IMPLEMENT_DYNCREATE(CBenchPage, CPropertyPage)

CBenchPage::CBenchPage() : CPropertyPage(CBenchPage::IDD)
{
	//{{AFX_DATA_INIT(CBenchPage)
	m_benchType = -1;
	m_rule1Nb = 0;
	m_sourceType = -1;
	m_ruleType = -1;
	//}}AFX_DATA_INIT
}

CBenchPage::~CBenchPage()
{
}

void CBenchPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBenchPage)
	DDX_CBIndex(pDX, IDC_BENCH_TYPE, m_benchType);
	DDX_Text(pDX, IDC_RULE1_NB, m_rule1Nb);
	DDX_Radio(pDX, IDC_IMAGES_BT, m_sourceType);
	DDX_Radio(pDX, IDC_RULE1, m_ruleType);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBenchPage, CPropertyPage)
	//{{AFX_MSG_MAP(CBenchPage)
	ON_BN_CLICKED(IDC_BENCH_BUTTON, OnBenchButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBenchPage message handlers

BOOL CBenchPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	CComboBox	*box;
	CRect		rect;

	CheckRadioButton(IDC_RULE1,IDC_RULE1,IDC_RULE1 );

	m_demoView = (CDpBioMainView *)GetParent()->GetParent();
	box = (CComboBox *)GetDlgItem(IDC_BENCH_TYPE);
	box->GetClientRect(&rect);
	box->SetWindowPos(NULL,0,0,rect.right,256,SWP_NOOWNERZORDER|SWP_NOMOVE);
	m_sourceType = 0;
	m_ruleType = 0;

	UpdateData(false);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CBenchPage::OnBenchButton() 
{
	CString		text;
	CString		srcFolder;
	int			error,nbItems;
	CHiResCounter	timer;

	UpdateData(true);
	if( m_sourceType == 0 )
		text = "Select directory where images for benchmark will be loaded";
	else
		text = "Select directory where templates for benchmark will be loaded";

	CFolderDialog	folderDialog(text);
	error = folderDialog.DoModal();

	if( error == IDOK )
	{
		srcFolder = folderDialog.GetFolderPath();

		text.Format("Please wait analysing directory : %s", srcFolder );
		m_demoView->WriteToInfoCtrl( text );
		BeginWaitCursor();
		if( m_sourceType == 0 )
		{
			timer.startTimer();
			nbItems = getNbFiles( srcFolder, "bmp");
			timer.stopTimer();
			text.Format("Found %d images in directory in %.3f seconds", nbItems, timer.getTimer() );
			m_demoView->WriteToInfoCtrl( text );
			m_progress.Init(nbItems);
			benchFolder( srcFolder, "bmp", nbItems ); 
			m_progress.DestroyWindow();
		}
		else
		{
			timer.startTimer();
			nbItems = getNbFiles( srcFolder, "bin");
			timer.stopTimer();
			text.Format("Found %d templates in directory in %.3f seconds", nbItems, timer.getTimer() );
			m_demoView->WriteToInfoCtrl( text );
			m_progress.Init(nbItems);
			benchFolder( srcFolder, "bin", nbItems ); 
			m_progress.DestroyWindow();
		}

	}

	EndWaitCursor();
	
}

int CBenchPage::benchFolder( CString p_directory, CString p_extension, int p_nbFiles, bool b_first )
{
	WIN32_FIND_DATA	findData;
	HANDLE			findHandle;
	CString			string,filename,text;
	int				cpt=0;
	int				loop;
	static double	matchTime=0.0,rejectTime=0.0;
	static int		sCpt=0,cptMatch,cptReject;
	int				error,size;
	unsigned int	score,nbItems,i,templatePos;
	unsigned char	*buffer;
	CHiResCounter	timer;
	int				farCnt=0,frrCnt=0;

	if( b_first ) 
	{
		matchTime = 0.0;
		rejectTime = 0.0;
		sCpt=0;
		cptMatch = cptReject = 0;
	}

	// recurse subdirectories
	string = p_directory + "\\*";
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( findData.dwFileAttributes|FILE_ATTRIBUTE_DIRECTORY && strcmp(findData.cFileName,".") && strcmp(findData.cFileName,"..") )
			{
				string = CString(p_directory) + "\\" + CString(findData.cFileName);
				cpt+= benchFolder( string, p_extension, p_nbFiles, false );
				string.ReleaseBuffer();
			}
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}

	// now search for "extension" in the directory
	string = p_directory + "\\*" + p_extension;

	size = dpGetMaxTemplateSize();
	buffer = new unsigned char[size];
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			cpt++;
			sCpt++;
			filename = CString(findData.cFileName);
			string = CString(p_directory) + "\\" + filename;
			/*p_parent->m_infos.Format("Fichier : %s", string );
			p_parent->m_progress.StepIt();
			p_parent->UpdateData(false);*/
			if( p_extension == "bmp" )
				error = enrollFromFile( string, buffer );
			else
				error = loadTemplateFile( string, buffer );
			if( error != 1 )
				return -1;

			if( m_benchType == 0 )
			{
			}
			else
			{
				m_progress.m_infos.Format("Identifying file : %s",filename );
				m_progress.UpdateData(false);
				nbItems = m_demoView->m_nbTemplates;
				timer.startTimer();
				error = dpIdentifyTemplate( m_demoView->m_templateTab, nbItems, buffer, &score, &templatePos );
				timer.stopTimer();
				
				m_progress.m_progress.StepIt();
				if( error == 1 ) // matched
				{
					matchTime += m_demoView->m_nbTemplates/timer.getTimer();
					cptMatch++;
					if( filename.Left(filename.GetLength()-m_rule1Nb-4) == m_demoView->m_fileNameTab[templatePos].Left(m_demoView->m_fileNameTab[templatePos].GetLength()-m_rule1Nb) )
					{
						text.Format("CORRECT MATCH : File %s matched with DbTemplate %s in %.3f secondes (match speed %.0f matches/second) - score:%d", filename, m_demoView->m_fileNameTab[templatePos],  timer.getTimer(), m_demoView->m_nbTemplates/timer.getTimer() ,score );
						m_demoView->WriteToInfoCtrl( text );
					}
					else
					{
						text.Format("FALSE MATCH : File %s matched with DbTemplate %s in %.3f secondes (match speed %.0f matches/second) - score:%d", filename, m_demoView->m_fileNameTab[templatePos],  timer.getTimer(), m_demoView->m_nbTemplates/timer.getTimer() ,score );
						m_demoView->WriteToInfoCtrl( text );
						farCnt++;
					}
				}
				else if( error == 2 ) // not matched
				{
					rejectTime += m_demoView->m_nbTemplates/timer.getTimer();
					cptReject++;
					for( i=0; i<nbItems; i++ )
					{
						if( filename.Left(filename.GetLength()-m_rule1Nb-4) == m_demoView->m_fileNameTab[i].Left(m_demoView->m_fileNameTab[i].GetLength()-m_rule1Nb) )
							break;
					}

					if( i != nbItems )
					{
						text.Format("FALSE REJECT : File %s not matched with DbTemplate %s in %.3f secondes (match speed %.0f matches/second) - score:%d", filename, m_demoView->m_fileNameTab[i],timer.getTimer(), m_demoView->m_nbTemplates/timer.getTimer(),score );
						m_demoView->WriteToInfoCtrl( text );
						frrCnt++;
					}
					else
					{
						text.Format("REJECT OK : File %s not matched  in %.3f secondes (match speed %.0f matches/second)", filename, m_demoView->m_fileNameTab[i],timer.getTimer(), m_demoView->m_nbTemplates/timer.getTimer()  );
						m_demoView->WriteToInfoCtrl( text );
					}
				}
				else // error
				{
					text.Format("ERROR DURING IDENTIFICATION : File %s not matched  in %.3f secondes (match speed %.0f matches/second)", filename, m_demoView->m_nbTemplates/timer.getTimer(), timer.getTimer() );
					m_demoView->WriteToInfoCtrl( text );
				}
			}

			text.Format("Found %d FAR pair and %d FRR pairs - Average Matching speed: %.0f matches/seconds", farCnt, frrCnt, (matchTime+rejectTime)/sCpt );
			m_demoView->WriteToInfoCtrl( text );
			text.Format("Average Accept speed: %.0f matches/seconds -  Average Reject speed: %.0f matches/seconds",
				matchTime/cptMatch,rejectTime/cptReject );
			m_demoView->WriteToInfoCtrl( text );
			

			/*p_parent->UpdateData(false);
			p_parent->RedrawWindow();*/
			string.ReleaseBuffer();
			loop = FindNextFile( findHandle, &findData );

		}
		while( loop ); 

		FindClose( findHandle );
	}

	delete[] buffer;
	
	return cpt;
}
