// EnrollPage.cpp : implementation file
//

#include "stdafx.h"
#include "dpBioDemo.h"
#include "EnrollPage.h"
#include "dpBioDemoView.h"
#include "FolderDlg.h"
#include "myFuncs.h"
#include "ProgressDlg.h"
#include "HiResCounter.h"
#include "EnrollDevice.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CDpBioMainView* m_demoView;

/////////////////////////////////////////////////////////////////////////////
// CEnrollPage property page

IMPLEMENT_DYNCREATE(CEnrollPage, CPropertyPage)

CEnrollPage::CEnrollPage() : CPropertyPage(CEnrollPage::IDD)
{
	//{{AFX_DATA_INIT(CEnrollPage)
	m_recurseFolder = FALSE;
	m_keepFolder = FALSE;
	m_enrollMode = -1;
	m_destDb = _T("");
	//}}AFX_DATA_INIT
}

CEnrollPage::~CEnrollPage()
{
}

void CEnrollPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEnrollPage)
	DDX_Check(pDX, IDC_RECURSE_FOLDER, m_recurseFolder);
	DDX_Check(pDX, IDC_KEEP_FOLDER, m_keepFolder);
	DDX_CBIndex(pDX, IDC_ENROLL_MODE, m_enrollMode);
	DDX_Text(pDX, IDC_DEST_DB, m_destDb);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEnrollPage, CPropertyPage)
	//{{AFX_MSG_MAP(CEnrollPage)
	ON_BN_CLICKED(IDC_KEEP_FOLDER, OnKeepFolder)
	ON_BN_CLICKED(IDC_CHOOSE_DB, OnChooseDb)
	ON_BN_CLICKED(IDC_ENROLL_BTN, OnEnrollBtn)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnrollPage message handlers

BOOL CEnrollPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	CComboBox	*box;
	CRect		rect;

	m_demoView = (CDpBioMainView *)GetParent()->GetParent();
	
	m_enrollMode = 0;
	CheckRadioButton(IDC_SIMPLE_TEMPLATES,IDC_COMPOSITE_TEMPLATES,IDC_SIMPLE_TEMPLATES);
	box = (CComboBox *)GetDlgItem(IDC_ENROLL_MODE);
	box->GetClientRect(&rect);
	box->SetWindowPos(NULL,0,0,rect.right,256,SWP_NOOWNERZORDER|SWP_NOMOVE);

	UpdateData(false);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CEnrollPage::OnKeepFolder() 
{
	UpdateData(true);
	if( m_keepFolder )
	{
		m_destDb = m_demoView->m_databaseDir;
	}
	else
	{
		m_destDb = "";	
	}
	UpdateData(false);
}

void CEnrollPage::OnChooseDb() 
{
	CFolderDialog	folderDialog("Select directory where templates will be stored");

	UpdateData(true);
	folderDialog.SetSelectedFolder(m_destDb);

	if( folderDialog.DoModal() == IDOK )
	{
		m_destDb = folderDialog.GetFolderPath();
		// get db path
		/*progConfig->m_databaseDir = folderDialog.GetFolderPath();
		m_progConfig->SaveConfig();
		*/			
	}
	UpdateData(false);
	
}

void CEnrollPage::OnEnrollBtn() 
{
	int				error,cpt;
	CString			txt,fileName;
	CFileDialog		fileDlg(true,".bmp","Please select image file for enrollment",OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
			"Image Files (*.bmp)|*.bmp||",this);
	POSITION		pos;
	CProgressDlg	progress(this);
	CHiResCounter	timer;
	double			time=0;

	UpdateData(true);

	if( m_destDb == "" )
	{
		AfxMessageBox("Please select destination DB first");
		return;
	}

	if( m_enrollMode == 0 )
	{
		
		if( m_recurseFolder )
		{
			CFolderDialog	folderDialog("Select directory where database templates will be loaded/stored");
			folderDialog.SetSelectedFolder(m_destDb);
			
			if( folderDialog.DoModal() == IDOK )
			{
				// get db path
				fileName = folderDialog.GetFolderPath();
				BeginWaitCursor();
				cpt = getNbFiles(fileName,"bmp");
				progress.Init(cpt);
				timer.startTimer();
				genTemplatesFolder( fileName, ".bmp", cpt, m_destDb, &progress );
				timer.stopTimer();
				progress.DestroyWindow();
				time = timer.getTimer();
				txt.Format("%d files enrolled in %.2f seconds",cpt,time);
				m_demoView->WriteToInfoCtrl(txt);
				EndWaitCursor();
				m_demoView->Refresh();
			}
			
		}
		else
		{
			error = fileDlg.DoModal();
			if( error == IDOK )
			{
				// count the number of files
				pos = fileDlg.GetStartPosition();
				cpt=0;
				do
				{
					cpt++;
					fileName = fileDlg.GetNextPathName(pos);
				}
				while( pos != NULL );
				
				progress.Init( cpt );
				
				// then enroll
				pos = fileDlg.GetStartPosition();
				cpt=0;
				BeginWaitCursor();
				do
				{
					cpt++;
					fileName = fileDlg.GetNextPathName(pos);
					progress.m_infos.Format("Please wait...\nEnrolling file : %s",fileName);
					progress.UpdateData(false);
					timer.startTimer();
					error = genWriteTemplate( fileName, m_destDb );
					timer.stopTimer();
					progress.m_progress.StepIt();
					
					time += timer.getTimer();
					if( error == 1 )
					{
						txt.Format("File %s successfully enrolled to directory : %s in %.2f seconds",fileName,m_destDb,timer.getTimer());
						m_demoView->WriteToInfoCtrl(txt,false);
					}
					else
					{
						txt.Format("Enrollment of %s failed",fileName,m_destDb);
						m_demoView->WriteToInfoCtrl(txt,false);
					}
				}
				while( pos != NULL );
				txt.Format("%d files enrolled in %.2f seconds",cpt,time);
				m_demoView->WriteToInfoCtrl(txt);
				EndWaitCursor();
				progress.DestroyWindow();
				m_demoView->Refresh();
			}
			else
				error=0;	
		}
		
	}
	else
	{
		CEnrollDevice	enrollDevice;

		UpdateData(true);
		enrollDevice.m_dbPath = m_destDb;
		if( IsDlgButtonChecked(IDC_COMPOSITE_TEMPLATES) )
			enrollDevice.m_composite = true;
		else
			enrollDevice.m_composite = false;

		if( enrollDevice.DoModal() == IDOK )
			m_demoView->Refresh();
	}
	
}
