/*******************************************************************************
*
*      Company                   :  CAP GEMINI
*      Service Line              :  Product Devlopment
*      Project                   :  FingerChip
*      Task                      :  
*
*      Soft name                 :  $RCSfile: fcpublicstruct.h,v $
*      Soft version              :  $Revision: 1.2 $
*      Last modified             :  $Date: 2004/02/19 16:19:11 $
*
*      Description               :  Public structures and constants definitions
*
*      Authors                   :  Dauvergne Mickal
*									Alain Chambron (AC)
*
*      Platform                  :  Independent 
*      System                    :  Independent 
*
*      Programming language      :  C ANSI
*      Operating System          :  Independent 
*      Date of creation          :  02/14/2001
*
*      Prefix                    :
*      Code size (.o) (KB)       :
*
*      References      :
*
*      History                   :
*		02/14/2001				 : First squeleton
*		28/07/2003				 : cleanup	
*
*     Copyright (c) 
*     All rights reserved
*
******************************************************************************/
#ifndef __FCPUBLICSTRUCT__
#define __FCPUBLICSTRUCT__

//Constants use for FC_DebugFileGeneration
#define FC_LOG_DISABLED			(short)(0x0000)
#define FC_LOG_ENABLE_ERROR		(short)(0x0001)
#define FC_LOG_ENABLE_ALWAYS	(short)(0x0002)
#define FC_LOG_ERASE			(short)(0x8000)

//Default size value for some structure parameters
#define THE_DLL_VERSION_SIZE	20 
#define THE_COPYRIGHT_SIZE		300
#define THE_PART_NUMBER_SIZE	20
#define THE_DEVICE_NAME_SIZE	20

//used to parameter pixel map size
#define USB_XSLICE_SIZE 248
#define USB_YSLICE_SIZE 8
#define USBGENESYS_XSLICE_SIZE 280
#define USBGENESYS_YSLICE_SIZE 8

//Communication port enumeration
//Thus USB stand for 0, USBGenesys for 1, USBTwain for 2, USB_ARM for 3, ETH_ARM for 4 and so on ...
typedef enum
{
	USB=0,			// C103
	USBGenesys,		// C101 + Genesys
	USBTwain,		// C101 + twain ID3
	USB_ARM,		// C104 + USB
	ETH_ARM,			// C104 + ETH
	ETH_SA			// TC2 + ETH
} eCommPort;

//Thermal management policy
typedef enum
{
	THERMManual = 0,
	THERMSoft,
	THERMHard,
	THERMStart
} eThermalManag;

#define THERMStop THERMManual

//Led mode policy
typedef enum
{
	LEDAuto,
	LEDOff,
	LEDOn,
	LEDHard
} eLedMode;

//Wanted bandwidth
typedef enum
{
	BWMODEAutomatic,
	BWMODEFull,
	BWMODEHalf
} eBandwidthMode;

//Effective bandwidth
typedef enum
{
	REALBWNo,
	REALBWFull,
	REALBWHalf
} eRealBandwidth;

#ifdef WIN32
//Backup the data alignment of the structure
#pragma pack( push, enter_include2 )
#endif
//New data aligngment on bytes
//Thus there is no structure reorganisation during the compilation
//as well as no hole between to structure attributes.
//Important for internal process (See ParamAccess::RegReadProcess())
#if defined( WIN32 ) || defined( _WIN32_WCE ) 
#	pragma pack( 1 )
#	define PACK_STRUCT 
#else
#	define PACK_STRUCT __attribute__((packed)) 
#endif

//order in structures are IMPORTANT
typedef struct	t_FC_DeviceInfo
{
	unsigned long	pixelDefaultValue;						//Pixel default value
	eBandwidthMode	bandwidthMode;							//Wanted bandwidth
	eRealBandwidth	realBandwidth;							//Obtained bandwidth
	char			reserved[128-sizeof(eBandwidthMode)-sizeof(eRealBandwidth)];
	eCommPort	communicationPort;						//Port to communicate to.
	char			partNumber[THE_PART_NUMBER_SIZE];		//The device part number
	char			dllVersion[THE_DLL_VERSION_SIZE];		//Version of DLL
	char			copyright[THE_COPYRIGHT_SIZE];			//A copyright message
	char			deviceName[THE_DEVICE_NAME_SIZE];		//The device name
	double			hardVersionBoard;						//version of Hardware board
	double			maxSlicesPerSecond;						//maximum reading speed in frame per second
	double			bytesPerSecond;							//maximum bandwidth.
	unsigned short	xSliceSize;								//x size in pixel of one slice
	unsigned short	ySliceSize;								//y size in pixel of one slice
	unsigned short	pixelPitch;								//pixel pitch in micron (e.g. 50 microns)
	unsigned short	resolution;								//resolution of the device, bit per pixel
} t_FC_DeviceInfo PACK_STRUCT;


typedef struct	t_FC_SliceInfo
{
	unsigned long	thresholdUsed;							//Threshold used by the Dll to detect finger
	unsigned long	numberOfBytesRead;						//size of the stored slice memory, in bytes
	unsigned long	numberOfSlicesStored;					//number of stored slices
	unsigned long	numberOfSlicesRead;						//number of read slices(even when waiting)
	double			timeToReadStoredSlices;					//time to read sslice slices in seconds
	double			timeToReadAllSlices;					//time to read rslice slices in seconds
	unsigned long	slicesPerSecond;						//acquisition speed of the current device
	signed long		currentTemperature;						//slice temperature
	unsigned char	reserved[128];
} t_FC_SliceInfo PACK_STRUCT;

typedef struct _img_
{
	unsigned short x_img ;
	unsigned short y_img ;
	unsigned char *pucData ;
} t_FC_Img PACK_STRUCT;

typedef struct	t_FC_ImageInfo
{
	unsigned long	xReturnSize;		//size in x direction of the returned image in pixel
	unsigned long	yReturnSize;		//size in y direction of the returned image in pixel
	double			yMeanDisplacement; 	//mean value of displacement in y direction in pixel per slice
	double			meanFingerSpeed;	//mean value of finger speed in slices per second.
	unsigned long	xReconstructedSize;	//x size of reconstructed image ( not the returned image)
	unsigned long	yReconstructedSize;	//y size of reconstructed image ( not the returned image)
	t_FC_SliceInfo	lastSliceInfo;		//slice info structures
	unsigned char	backgroundUsed;		//Background color used to fill images
} t_FC_ImageInfo PACK_STRUCT;

typedef struct	t_FCSoftwareConfiguration
{
	char				rotation;			//Horizontal axis symetry
	char				mirror;				//Vertical axis symetry
	char				flip;
	unsigned long		minHeightImage;		//If image height inferior this value, error code generated
	char				autoThreshold;		//Select auto threshold or manual. For manual threshold,
	unsigned long		beginThreshold;		//Minimum level to start acquisition (about 20)
	unsigned long		waitingTimeOut;		//Timeout when waiting for a finger in ms
	char				noiseFilter;		//Remove fixed noise or not
	char				enhanceImage;		//Enhance reconstructed image
	eThermalManag		autoThermalManag;	//Thermal management (Soft, Manual,Hard, Off)
	char				temperatureUnit;
	long				temperatureLow;		//Low temperature boundary in F (by default 25C)
	long				temperatureHigh;	//High temperature boundary in F (by default 45C)
	unsigned long		minSliceNumber;		//Minimum number of slices for an image acquisition
	unsigned char		thermalWatchdog;	//Thermal watchdog for FCUSB only
	eLedMode			ledMode;			// Led control
	char				reserved[123];		// reserved for lated use 128 - sizeof(thermalWatchdog) - sizeof(ledMode)
} t_FC_SoftwareConfiguration PACK_STRUCT;

// Internal parameters,
// computed at OpenDevice step,
// from Communication port value and synchro analysis
typedef struct	t_FC_InternalDeviceInfo
{
	char				invertOddEven;		//Invert odd and even bytes
	unsigned long		startSynchroSize;	//Deducted start synchro size
	unsigned long		endSynchroSize;		//Deducted end synchro size
	char				rotation;			//Deducted horizontal axis symetry
	char				mirror;				//Deducted vertical axis symetry
	eLedMode		ledMode;
} t_FC_InternalDeviceInfo PACK_STRUCT;

// Constants use for FC_CharacGetDataSlices
#define FC_NAV_SLICE   0x00
#define FC_CLICK_SLICE 0x01

typedef struct t_FC_NavigationInfo
{
  unsigned short		unStatus;
  unsigned short		unNavStatus;
  long					lXDisplacement;
  long					lYDisplacement;
  unsigned long			ulPixelClickRegister;
} t_FC_NavigationInfo PACK_STRUCT;

#ifdef WIN32
//Restore data alignment
#pragma pack( pop, enter_include2 )
#endif

#endif
