// IdentifyPage.cpp : implementation file
//

#include "stdafx.h"
#include "dpBioDemo.h"
#include "IdentifyPage.h"
#include "exported.h"
#include "enrollExport.h"
#include "matchExport.h"
#include "errorCodes.h"
#include "DpBioDemoView.h"
#include "myFuncs.h"
#include "HiResCounter.h"
#include "matcher.h"
#include "fxMatcher.h"
#include "identDevice.h"
#include "dynamicParameters.h"
#include "quicksort.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CDpBioMainView* m_demoView;
extern BioEngineParams g_params;

/////////////////////////////////////////////////////////////////////////////
// CIdentifyPage property page

IMPLEMENT_DYNCREATE(CIdentifyPage, CPropertyPage)

CIdentifyPage::CIdentifyPage() : CPropertyPage(CIdentifyPage::IDD)
{
	//{{AFX_DATA_INIT(CIdentifyPage)
	m_identSrc = -1;
	m_searchDepth = 0;
	//}}AFX_DATA_INIT
}

CIdentifyPage::~CIdentifyPage()
{
}

void CIdentifyPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIdentifyPage)
	DDX_CBIndex(pDX, IDC_IDENT_SOURCE, m_identSrc);
	DDX_Text(pDX, IDC_SEARCH_DEPTH, m_searchDepth);
	DDV_MinMaxInt(pDX, m_searchDepth, 0, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIdentifyPage, CPropertyPage)
	//{{AFX_MSG_MAP(CIdentifyPage)
	ON_BN_CLICKED(IDC_IDENT_BUTTON, OnIdentButton)
	ON_BN_CLICKED(IDC_VALIDATE, OnValidate)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIdentifyPage message handlers

void CIdentifyPage::OnIdentButton() 
{
	unsigned char	*templBuf;
	int				i,size,index,error=0,indexOfMax;
	unsigned int	score,maxScore;
	CString			text,fileName;
	CHiResCounter	timer;
	double			totalTime=0;
	CProgressDlg	progress(this);
	int				nbToSearch;

	UpdateData(true);
	
	size = dpGetMaxTemplateSize();
	templBuf = new unsigned char[size];

	CFileDialog	*fileDlg;

	switch(m_identSrc)
	{
	case 0: // FILE LOAD
		fileDlg = new CFileDialog(true,".bin","Please select template file to load",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
			"Template Files (*.bin)|*.bin||",this);
		error = fileDlg->DoModal();
		if( error == IDOK )
		{
			// load file
			fileName = fileDlg->GetPathName();
			timer.startTimer();
			error =loadTemplateFile( fileName , templBuf );
			timer.stopTimer();
			m_demoView->WriteToInfoCtrl("");
			text.Format("Template %s loaded in %.3f seconds",fileName,timer.getTimer());
			m_demoView->WriteToInfoCtrl(text);
		}
		else
			error=0;
		delete fileDlg;
		break;
	case 1: // IMG_LOAD
		fileDlg = new CFileDialog(true,".bmp","Please select template file to load",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
			"Image Files (*.bmp)|*.bmp||",this);
		error = fileDlg->DoModal();
		if( error == IDOK )
		{
			// load file
			fileName = fileDlg->GetPathName();
			timer.startTimer();
			error =enrollFromFile( fileName , templBuf );
			timer.stopTimer();
			m_demoView->WriteToInfoCtrl("");
			text.Format("Image %s loaded + features extraction in %.3f seconds",fileName,timer.getTimer());
			m_demoView->WriteToInfoCtrl(text);
		}
		else
			error=0;
		delete fileDlg;
		break;
	case 2: // IDENT FROM FINGERCHIP
		{
		CIdentDevice identDevice;

		identDevice.m_threadData.m_nbTemplates = m_demoView->m_nbTemplates;
		identDevice.m_threadData.m_fileNameTab = m_demoView->m_fileNameTab;
		identDevice.m_threadData.m_templateTab = m_demoView->m_templateTab;
		UpdateData(true);
		identDevice.m_maxDbSearch = m_searchDepth;
		identDevice.DoModal();
		}
		break;
	default:
		return;
		break;
	}

	text.Format("Seardching at most %d%% of the database (%d templates)",m_searchDepth,m_demoView->m_nbTemplates*m_searchDepth/100);
		m_demoView->WriteToInfoCtrl(text);

	for( i=0; i<m_demoView->m_nbTemplates; i++ )
		m_demoView->m_scores[i] = -1;

	if( error >= 1 )
	{
		totalTime=0;
		timer.startTimer();
		sortTemplates( templBuf, m_demoView->m_templateTab, m_demoView->m_nbTemplates,
		m_demoView->m_coreIndexTab, m_demoView->m_coreGlobDist, m_demoView->m_indexTab, m_demoView->m_globDist );
		timer.stopTimer();
		totalTime += timer.getTimer();


		if( m_demoView->m_coreIndexTab[m_demoView->m_nbTemplates] < m_demoView->m_nbTemplates*m_searchDepth/100 )
			nbToSearch = m_demoView->m_coreIndexTab[m_demoView->m_nbTemplates];
		else
			nbToSearch = m_demoView->m_nbTemplates*m_searchDepth/100;

		text.Format("nbToSearch is finaly : %d\n",nbToSearch);
		m_demoView->WriteToInfoCtrl(text);
	
		//first pass
		maxScore = 0;
		indexOfMax = -1;
		for( i=0; i<nbToSearch; i++ )
		{
			index = m_demoView->m_coreIndexTab[i];

			if( m_demoView->m_coreGlobDist[i] > 300 )
			{
				break;
			}
			timer.startTimer();
			error = fastVerifyTemplate( templBuf, m_demoView->m_templateTab[index], &score );
			timer.stopTimer();
			totalTime += timer.getTimer();
			m_demoView->m_scores[index] = score;
		
			if( score >= 2*g_params.m_defaultSecurity )
			{
				text.Format("Match found for file %s identified as dataBase template %s",fileName,m_demoView->m_fileNameTab[index]);
				m_demoView->WriteToInfoCtrl(text);
				error = -1;
				break;
			}
			else if( score > maxScore )
			{
				maxScore = score;
				indexOfMax = index;
			}
		}
		if( indexOfMax != -1 )
		{
			timer.startTimer();
			error = dpVerifyTemplate( templBuf, m_demoView->m_templateTab[indexOfMax], &score );
			m_demoView->m_scores[indexOfMax] = score;
			timer.stopTimer();
			totalTime += timer.getTimer();
			if( score >= 2*g_params.m_defaultSecurity )
			{
				text.Format("Match found for file %s identified as dataBase template %s",fileName,m_demoView->m_fileNameTab[indexOfMax]);
				m_demoView->WriteToInfoCtrl(text);
				error = -1;
			}
		}

		// second pass
		// sort nocore dists
		sortUINT32( m_demoView->m_globDist, m_demoView->m_indexTab, 0, m_demoView->m_nbTemplates-1 );

		nbToSearch = m_demoView->m_nbTemplates*m_searchDepth/100;
		if( error != -1 )
		{

			maxScore = 0;
			indexOfMax = -1;	
			for( i=0; i<nbToSearch; i++ )
			{
				index = m_demoView->m_indexTab[i];
				timer.startTimer();
				error = dpVerifyTemplate( templBuf, m_demoView->m_templateTab[index], &score );
				timer.stopTimer();
				totalTime += timer.getTimer();
				m_demoView->m_scores[index] = score;
			
				if( score >= 2*g_params.m_defaultSecurity )
				{
					text.Format("Match found for file %s identified as dataBase template %s",fileName,m_demoView->m_fileNameTab[index]);
					m_demoView->WriteToInfoCtrl(text);
					error = -1;
					break;
				}
				else if( score > maxScore )
				{
					maxScore = score;
					indexOfMax = index;
				}
			}
			if( indexOfMax != -1 )
			{
				text.Format("Match found for file %s identified as dataBase template %s",fileName,m_demoView->m_fileNameTab[indexOfMax]);
				m_demoView->WriteToInfoCtrl(text);
				error = -1;
			}
		}

		if( error != -1 )
		{
			text.Format("Match not found for file %s",fileName);
			m_demoView->WriteToInfoCtrl(text);
		}
		text.Format("Total identification time : %.3f seconds (%.3f miliseconds)",totalTime,totalTime*1000.0);
		m_demoView->WriteToInfoCtrl(text);
		text.Format("Matching speed : %.0f matches/sec",m_demoView->m_nbTemplates/totalTime);
		m_demoView->WriteToInfoCtrl(text);
		i++;
		text.Format("%d templates compared (%.2f%% of the database)",i,100.0*i/(float)m_demoView->m_nbTemplates);
		m_demoView->WriteToInfoCtrl(text);

		m_demoView->UpdateListView();
	}
	else
		m_demoView->WriteToInfoCtrl("Source file couldn't be loaded!");

	delete[] templBuf;
	
}

BOOL CIdentifyPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	CComboBox	*box;
	CRect		rect;

	dpInit();
	
	m_searchDepth = 100;
	UpdateData(false);

	m_demoView = (CDpBioMainView *)GetParent()->GetParent();
	box = (CComboBox *)GetDlgItem(IDC_IDENT_SOURCE);
	box->GetClientRect(&rect);
	box->SetWindowPos(NULL,0,0,rect.right,256,SWP_NOOWNERZORDER|SWP_NOMOVE);


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CIdentifyPage::OnValidate() 
{
	CString	text;

	UpdateData(true);
	dpSetDbMaxSearch(m_searchDepth);

	text.Format("Max DB search changed to %d%%",m_searchDepth);
	m_demoView->WriteToInfoCtrl(text);
	
}

void CIdentifyPage::OnDestroy() 
{
	CPropertyPage::OnDestroy();
	
	dpUnInit();
	
}
