// dpBioDemoView.cpp : implementation of the CDpBioMainView class
//

#include "stdafx.h"
#include "dpBioDemo.h"

#include "dpBioDemoDoc.h"
#include "dpBioDemoView.h"
#include "folderDlg.h"
#include "myFuncs.h"
#include "dpBio.h"
#include "HiResCounter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDpBioMainView

IMPLEMENT_DYNCREATE(CDpBioMainView, CFormView)

BEGIN_MESSAGE_MAP(CDpBioMainView, CFormView)
	//{{AFX_MSG_MAP(CDpBioMainView)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_CHOOSE_DATABASE, OnChooseDatabase)
	ON_BN_CLICKED(IDC_SELECT_ALL, OnSelectAll)
	ON_BN_CLICKED(IDC_REFRESH, OnRefresh)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDpBioMainView construction/destruction

CDpBioMainView::CDpBioMainView()
	: CFormView(CDpBioMainView::IDD)
{
	//{{AFX_DATA_INIT(CDpBioMainView)
	m_databaseDir = _T("");
	//}}AFX_DATA_INIT

	CString path;

	dpInit();
	
	m_progConfig = new CDemoConfig( AfxGetApp()->m_pszHelpFilePath );
	m_nbTemplates = 0;
	m_fileNameTab = NULL;
	m_indexTab = NULL;
	m_templateTab = NULL;
	m_globDist = NULL;
	m_scores = NULL;
	m_coreIndexTab = NULL;
	m_coreGlobDist = NULL;

}

CDpBioMainView::~CDpBioMainView()
{	
	int i;

	if( m_fileNameTab != NULL )
		delete[] m_fileNameTab;

	if( m_indexTab != NULL )
		delete[] m_indexTab;

	if( m_coreIndexTab != NULL )
		delete[] m_coreIndexTab;

	if( m_coreGlobDist != NULL )
		delete[] m_coreGlobDist;

	if( m_templateTab != NULL )
	{
		for( i=0; i<m_nbTemplates; i++ )
		{
			if( m_templateTab[i] != NULL )
				delete[] m_templateTab[i];
		}
		delete[] m_templateTab;
	}

	delete m_progConfig;

	dpUnInit();
}

void CDpBioMainView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDpBioMainView)
	DDX_Control(pDX, IDC_INFOS, m_infosCtrl);
	DDX_Control(pDX, IDC_LIST1, m_listCtrl);
	DDX_Control(pDX, IDC_TAB_CONTROL_BOX, m_tabControlBox);
	DDX_Text(pDX, IDC_DATABASE_REP, m_databaseDir);
	//}}AFX_DATA_MAP
}

BOOL CDpBioMainView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style &= ~WS_HSCROLL;
	cs.style &= ~WS_VSCROLL;

	return CFormView::PreCreateWindow(cs);
}

void CDpBioMainView::OnInitialUpdate()
{
	CRect	rect;
	int		i;

	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	EnableScrollBar(SB_BOTH,ESB_DISABLE_BOTH);
	ResizeParentToFit();

	m_progConfig->LoadConfig();

	m_propSheet.Construct("mode select",this);
	m_propSheet.AddPage(&m_enrollPage);
	m_propSheet.AddPage(&m_authentPage);
	m_propSheet.AddPage(&m_identPage);
	m_propSheet.AddPage(&m_benchPage);
	m_propSheet.AddPage(&m_configPage);
	m_propSheet.Create(this,WS_CHILD,WS_EX_LEFT);
	m_tabControlBox.GetClientRect(&rect);
	m_propSheet.SetWindowPos(&wndTop,rect.left,rect.top,rect.right,rect.bottom,SWP_SHOWWINDOW );

	InitListView();

	if( m_progConfig->m_databaseDir != "" )
		LoadDatabase( m_progConfig->m_databaseDir );

	for( i=0; i<5; i++ )
		m_propSheet.SetActivePage(i);

	m_propSheet.SetActivePage(0);
	
}

/////////////////////////////////////////////////////////////////////////////
// CDpBioMainView printing

BOOL CDpBioMainView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDpBioMainView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDpBioMainView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CDpBioMainView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// CDpBioMainView diagnostics

#ifdef _DEBUG
void CDpBioMainView::AssertValid() const
{
	CFormView::AssertValid();
}

void CDpBioMainView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CDpBioDemoDoc* CDpBioMainView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDpBioDemoDoc)));
	return (CDpBioDemoDoc*)m_pDocument;
}
#endif //_DEBUG


void CDpBioMainView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);

	CRect	rect;
	
	if( m_listCtrl.m_hWnd != NULL )
	{
		m_listCtrl.GetWindowRect(&rect);
		ScreenToClient(&rect);
		m_listCtrl.SetWindowPos(&wndTop,0,0,cx-rect.left-10, rect.bottom-rect.top, SWP_NOMOVE|SWP_NOOWNERZORDER);
	}

	if( m_infosCtrl.m_hWnd != NULL )
	{
		m_infosCtrl.GetWindowRect(&rect);
		ScreenToClient(&rect);
		m_infosCtrl.SetWindowPos(&wndTop,0,0,cx-rect.left-10, cy-rect.top-10, SWP_NOMOVE|SWP_NOOWNERZORDER);
	}
}

void CDpBioMainView::OnChooseDatabase() 
{
	CFolderDialog	folderDialog("Select directory where database templates will be loaded/stored");


	UpdateData(true);
	folderDialog.SetSelectedFolder(m_databaseDir);

	if( folderDialog.DoModal() == IDOK )
	{
		// get db path
		m_progConfig->m_databaseDir = folderDialog.GetFolderPath();
		m_progConfig->SaveConfig();

		LoadDatabase( m_progConfig->m_databaseDir );
	}

}

void CDpBioMainView::WriteToInfoCtrl( CString p_message, bool p_redraw )
{
	static CString			oldMessage="";
	static bool				first=true;
	static CStdioFile		file;

	if( first )
	{
		first = false;
		file.Open("c:\\dpBioDemo.log",CFile::modeWrite|CFile::modeCreate);
	}
	
	// check size
	if( oldMessage.GetLength() > 100000 )
		oldMessage = oldMessage.Right(100000);

	// write
	oldMessage += p_message+"\r\n";
	if( p_redraw )
	{
		m_infosCtrl.SetWindowText(oldMessage);
		m_infosCtrl.LineScroll(m_infosCtrl.GetLineCount());
		file.WriteString(p_message+"\r\n");
	}
}

void CDpBioMainView::InitListView()
{
	m_listCtrl.InsertColumn(0,"Index",LVCFMT_LEFT,48,0);
	m_listCtrl.InsertColumn(1,"Template ID",LVCFMT_LEFT,128,1);
	m_listCtrl.InsertColumn(2,"Size",LVCFMT_LEFT,48,2);
	m_listCtrl.InsertColumn(3,"Quality",LVCFMT_LEFT,48,3);
	m_listCtrl.InsertColumn(4,"Global Core Dist",LVCFMT_LEFT,64,4);
	m_listCtrl.InsertColumn(5,"Global Dist",LVCFMT_LEFT,64,5);
	m_listCtrl.InsertColumn(6,"Score",LVCFMT_LEFT,48,6);
}

void CDpBioMainView::UpdateListView()
{
	int			i,j,index,index1;
	CString		text;

	m_listCtrl.DeleteAllItems( );

	for( i=0; i<m_nbTemplates; i++ )
	{
		index = m_indexTab[i];
		// index
		text.Format("%d",i);
		m_listCtrl.InsertItem(i,text);
		// template name
		m_listCtrl.SetItemText(i,1,m_fileNameTab[index]);
		// size
		text.Format("%d",dpGetTemplateSize(m_templateTab[index]));
		m_listCtrl.SetItemText( i, 2, text );
		// quality
		text.Format("%d",dpGetTemplateQuality(m_templateTab[index]));
		m_listCtrl.SetItemText( i, 3, text );
		// global core dist
		index1=-1;
		for( j=0; j<(INT32)m_coreIndexTab[m_nbTemplates]; j++ )
		{
			if( m_coreIndexTab[j] == index )
			{
				index1=j;
				break;
			}
		}
		if( m_coreGlobDist[index1] == -1 || index1 == -1 )
			text.Format("n/a");
		else
			text.Format("%d",m_coreGlobDist[index1]);
		m_listCtrl.SetItemText( i, 4, text );
		// global dist
		if( m_globDist[i] == -1 )
			text.Format("n/a");
		else
			text.Format("%d",m_globDist[i]);
		m_listCtrl.SetItemText( i, 5, text );
		// score
		if( m_scores[index] == -1 )
			text.Format("n/a");
		else if( m_scores[index] == -2 )
			text.Format("FAIL");
		else
			text.Format("%d",m_scores[index]);
		m_listCtrl.SetItemText( i, 6, text );
	}
}

void CDpBioMainView::OnSelectAll() 
{
	int	i;

	for( i=0; i<m_nbTemplates; i++ )
	{
		m_listCtrl.SetItemState( i, LVIS_SELECTED, LVIS_SELECTED ); 
	}
	
}

void CDpBioMainView::LoadDatabase(CString p_dbPath)
{
	int				nbFiles,i,size;
	CString			message;
	CHiResCounter	timer;

	m_databaseDir = p_dbPath;

	message.Format("Please wait, analysing directory %s...",m_databaseDir);
	WriteToInfoCtrl( message );

	// count nb of .bin files
	BeginWaitCursor();
	timer.startTimer();
	nbFiles = getNbFiles(m_databaseDir.GetBuffer(0), ".bin");
	timer.stopTimer();
	m_databaseDir.ReleaseBuffer();
	EndWaitCursor();
	message.Format("Found %d templates with '.bin' extension in directory: %s in %.3f seconds",nbFiles,m_databaseDir,timer.getTimer());
	WriteToInfoCtrl( message );

	// release old memory
	if( m_globDist != NULL )
		delete[] m_globDist;

	if( m_coreIndexTab != NULL )
		delete[] m_coreIndexTab;

	if( m_coreGlobDist != NULL )
		delete[] m_coreGlobDist;

	if( m_scores != NULL )
		delete[] m_scores;

	if( m_fileNameTab != NULL )
		delete[] m_fileNameTab;

	if( m_indexTab != NULL )
		delete[] m_indexTab;

	if( m_templateTab != NULL )
	{
		for( i=0; i<m_nbTemplates; i++ )
		{
			if( m_templateTab[i] != NULL )
				delete[] m_templateTab[i];
		}
		delete[] m_templateTab;
	}

	// allocate new memory
	m_scores = new int[nbFiles+1];
	m_globDist = new unsigned int[nbFiles+1];
	m_fileNameTab = new CString[nbFiles+1];
	m_indexTab = new unsigned int[nbFiles+1];
	m_templateTab = new unsigned char*[nbFiles+1];
	m_coreIndexTab = new unsigned int[nbFiles+1];
	m_coreGlobDist = new unsigned int[nbFiles+1];
	size = dpGetMaxTemplateSize();
	for(i=0; i<nbFiles+1; i++ )
	{
		m_scores[i] = -1;
		m_globDist[i] = -1;
		m_coreIndexTab[i] = -1;
		m_coreGlobDist[i] = -1;
		m_indexTab[i] = -1;
		m_templateTab[i] = new unsigned char[size];
	}m_nbTemplates = nbFiles;

	WriteToInfoCtrl( "Please wait, reading templates from disk" );

	// read files
	CProgressDlg	progress(this);
	progress.Init(nbFiles);
	timer.startTimer();
	fillTabs( m_databaseDir.GetBuffer(0), ".bin", nbFiles, m_fileNameTab, m_indexTab, m_templateTab, &progress );
	timer.stopTimer();
	m_databaseDir.ReleaseBuffer();
	progress.DestroyWindow();

	message.Format("%d templates read from disk in %.3f seconds",nbFiles,timer.getTimer());
	WriteToInfoCtrl( message );

	UpdateListView();
	
	UpdateData(false);
}

void CDpBioMainView::OnRefresh() 
{
	UpdateData(true);
	LoadDatabase(m_databaseDir);
	
}

void CDpBioMainView::Refresh()
{
	OnRefresh();
}
