#include "myFuncs.h"
#include "BMPfiles.h"
#include "dpBio.h"
#include "compatibility.h"
#include "tools.h"

int getNbFiles( CString p_directory, CString p_extension )
{
	WIN32_FIND_DATA	findData;
	HANDLE			findHandle;
	CString			string;
	int				cpt=0;
	int				loop;

	// recurse subdirectories
	string = p_directory + "\\*";
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( findData.dwFileAttributes|FILE_ATTRIBUTE_DIRECTORY && strcmp(findData.cFileName,".") && strcmp(findData.cFileName,"..") )
			{
				string = CString(p_directory) + "\\" + CString(findData.cFileName);
				cpt+= getNbFiles( string.GetBuffer(256), p_extension );
				string.ReleaseBuffer();
			}
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}

	// now search for "extension" in the directory
	string = p_directory + "\\*" + p_extension;
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			cpt++;
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}
	
	return cpt;
}

int genTemplatesFolder( CString p_directory, CString p_extension, int p_nbFiles, CString p_destDir, CProgressDlg *p_parent )
{
	WIN32_FIND_DATA	findData;
	HANDLE			findHandle;
	CString			string;
	int				cpt=0;
	int				loop,t1,t2;
	static float	totalTime=0.0;
	static int		sCpt=0;
	int				error;

	// recurse subdirectories
	string = p_directory + "\\*";
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( findData.dwFileAttributes|FILE_ATTRIBUTE_DIRECTORY && strcmp(findData.cFileName,".") && strcmp(findData.cFileName,"..") )
			{
				string = CString(p_directory) + "\\" + CString(findData.cFileName);
				cpt+= genTemplatesFolder( string, p_extension, p_nbFiles, p_destDir, p_parent );
				string.ReleaseBuffer();
			}
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}

	// now search for "extension" in the directory
	string = p_directory + "\\*" + p_extension;
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			cpt++;
			sCpt++;
			string = CString(p_directory) + "\\" + CString(findData.cFileName);
			p_parent->m_infos.Format("Fichier : %s", string );
			p_parent->m_progress.StepIt();
			p_parent->UpdateData(false);
			t1 = GetTickCount();
			error = genWriteTemplate( string, p_destDir );
			t2 = GetTickCount();
			if( error != 1 )
				return -1;
			totalTime += float((t2-t1)/1000.0);
			//p_parent->m_meanTime = totalTime/sCpt;
			//p_parent->m_totalTime = totalTime*(p_parent->m_nbFiles-sCpt)/(float)sCpt;
			p_parent->UpdateData(false);
			p_parent->RedrawWindow();
			string.ReleaseBuffer();
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}
	
	return cpt;
}

int genWriteTemplate( CString p_fileName, CString p_destDir  )
{
	int				error;
	unsigned int	temSize;
	unsigned char	theTemplate[10240];
	CFile			file;
	CString			string;
	int				pos;

	
	error = enrollFromFile( p_fileName, theTemplate );
	if( error != 1 )
	{
		return error;
	}

	pos = p_fileName.ReverseFind('\\');
	string = p_fileName.Right(p_fileName.GetLength()-pos-1);
	string = string.Left(string.GetLength()-3) + "bin";
	if( p_destDir[p_destDir.GetLength()-1] != '\\' )
		p_destDir += '\\';
	string = p_destDir + string;

	temSize = ((long*)theTemplate)[0];

	if( !file.Open( string, CFile::modeWrite|CFile::modeCreate ) )
	{
		return -1;
	}

	file.Write( theTemplate, temSize );
	
	file.Close();


	return 1;
} 

int enrollFromFile( CString p_fileName, unsigned char *p_buffer )
{
	int				error;
	int				size,width,height;
	unsigned char	*buffer;

	error = getBmpSize( p_fileName.GetBuffer(0), &size );
	p_fileName.ReleaseBuffer();
	if( error != 1 )
		return error;

	buffer = new unsigned char[size];

	error = loadBmp( p_fileName.GetBuffer(0), buffer, size, &width, &height );
	p_fileName.ReleaseBuffer();
	if( error != 1 )
	{
		delete[] buffer;
		return error;
	}

	error = dpGenerateTemplate( buffer, width, height, p_buffer );
	if( error != 1 )
	{
		delete[] buffer;
		return error;
	}

	delete[] buffer;

	return 1;

} 

int fillTabs( CString p_directory, CString p_extension, int p_nbFiles, CString *p_fileNameTab, unsigned int *p_indexTab, unsigned char **p_templateTab, CProgressDlg *p_progress, bool first )
{
	WIN32_FIND_DATA	findData;
	HANDLE			findHandle;
	CString			string;
	int				cpt=0;
	int				loop,pos;
	static int		sCpt=0;
	CFile			file;
	unsigned int	size;
	static CString	path;

	if( first )
	{
		path = p_directory;
		sCpt=0;
	}		

	// recurse subdirectories
	string = p_directory + "\\*";
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( findData.dwFileAttributes|FILE_ATTRIBUTE_DIRECTORY && strcmp(findData.cFileName,".") && strcmp(findData.cFileName,"..") )
			{
				string = CString(p_directory) + "\\" + CString(findData.cFileName);
				cpt+= fillTabs( string.GetBuffer(256), p_extension, p_nbFiles, p_fileNameTab, p_indexTab, p_templateTab, p_progress, false );
				string.ReleaseBuffer();
			}
			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}

	// now search for "extension" in the directory
	string = p_directory + "\\*" + p_extension;
	
	findHandle = FindFirstFile(string,&findData);
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		do
		{
	
			// read file
			string = p_directory + "\\" + CString(findData.cFileName);
			file.Open( string, CFile::modeRead );
			file.Read(&size, sizeof(unsigned int));
			file.SeekToBegin();
			file.Read(p_templateTab[sCpt], size);
			file.Close();

			// unpack template and save infos to arrays
			p_indexTab[sCpt] = sCpt;
			pos = string.ReverseFind('\\');
			string = string.Right(string.GetLength()-pos-1);
			p_fileNameTab[sCpt] = string;
			p_fileNameTab[sCpt] = p_fileNameTab[sCpt].Left(p_fileNameTab[sCpt].GetLength()-4);

			cpt++;
			sCpt++;

			if( p_progress != NULL )
			{
				p_progress->m_infos.Format("Loading files from : %s\r\nCurrent file : %s",p_directory, string );
				p_progress->m_progress.StepIt();
				p_progress->UpdateData(FALSE);
			}

			loop = FindNextFile( findHandle, &findData );
		}
		while( loop ); 

		FindClose( findHandle );
	}

	
	
	return cpt;
}


int segmenter( float *p_tab, int *p_index,  int p_taille )
//	Fonction pour segmenter le tableau en 2 ss-tableaux
//	retourne l'indice du pivot

{
	float pivot = p_tab[ p_taille/2 ];
	int indPivot = p_taille/2;
	int i, j, tmp2;
	float tmp;
	
	i=0;
	j=p_taille-1;
	
	while( i<j )
	{	
		while( p_tab[i] <= pivot && i < indPivot) i++;
		while( p_tab[j] >= pivot && j > indPivot && i<j) j--;

		if( i == indPivot ) indPivot=j;
		else if( j == indPivot ) indPivot=i;
		tmp = p_tab[i];
		p_tab[i] = p_tab[j];
		p_tab[j] = tmp;
		tmp2 = p_index[i];
		p_index[i] = p_index[j];
		p_index[j] = tmp2;
	}
	return indPivot;
}

void trier( float *p_tab, int *p_index, int p_borneInf, int p_borneSup )
// Procedure pour trier un tableau
{
	int i;
	
	i=segmenter( p_tab+p_borneInf, p_index+p_borneInf, p_borneSup-p_borneInf+1 );

	if( p_borneSup-p_borneInf>1 )
	{
		trier( p_tab, p_index, p_borneInf, p_borneInf+i);
		trier( p_tab, p_index, p_borneInf+i, p_borneSup );
	}
}

int loadTemplateFile( CString p_filename, unsigned char *p_buffer )
{
	CFile			file;
	unsigned int	size;

	if( !file.Open(p_filename,CFile::modeRead) )
		return 0;

	file.Read(&size,sizeof(unsigned int));
	file.SeekToBegin();
	file.Read(p_buffer,size);

	file.Close();

	return 1;
}

int saveTemplateFile( CString p_filename, unsigned char *p_buffer )
{
	CFile			file;
	unsigned int	size;

	if( !file.Open(p_filename,CFile::modeWrite|CFile::modeCreate) )
		return 0;

	size = dpGetTemplateSize( p_buffer );

	file.Write(p_buffer,size);

	file.Close();

	return 1;
}

