/*
Company :		ZEFYR Technologies
File :			BMPfiles.c
Author :		Matthieu Perreira Da Silva 
Fisrt release :	15/04/2003

Description : BMP load/save support

Features : 8bit greyscale non compressed BMP files		
    
Functions : 

	main : main testing function, performs unitary tests on dpBio.dll

Function prototypes : 

	void main(void)

Necessary package :
	none

Comments :none

*/

#include <stdlib.h>
#include <stdio.h>
#include "dpBio.h"
#include "errorCodes.h"

void main(void)
{
	int				error;
	unsigned int	result,score,security,nbTemplates=10,sizeX,sizeY,i,templatePos;
	unsigned char	*imgBuf,*templBuff1, *templBuff2, **templArray;
	unsigned char	refined;

	// allocations and initializations
	sizeX = 500;
	sizeY = 500;
	imgBuf = malloc( sizeX*sizeY*sizeof(unsigned char) );
	templBuff1 = malloc( 10000*sizeof(unsigned char) );
	templBuff2 = malloc( 10000*sizeof(unsigned char) );
	templArray = malloc( nbTemplates*sizeof(unsigned char*) );
	for( i=0; i<nbTemplates; i++ )
	{
		templArray[i] = malloc( 10000*sizeof(unsigned char) );
	}

	// testing acces to function without any previous call to dpInit();
	// it should return an error but should not crash
	printf("Calling all functions without initialisation...\n\n");

	printf("Calling getMaxTemplateSize:\t\t");
	result =dpGetMaxTemplateSize();
	printf("result = %d\n", result );

	printf("Calling getTemplateSize:\t\t");
	result =dpGetTemplateSize( NULL );
	printf("result = %d\n", result );

	printf("Calling generateTemplate:\t\t");
	error = dpGenerateTemplate( NULL, 0, 0, NULL ); 
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling verifyTemplate:\t\t");
	error = dpVerifyTemplate( NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling verifyAndRefineTemplate:\t\t");
	error = dpVerifyAndRefineTemplate( NULL, NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getTemplateQuality:\t\t");
	result =dpGetTemplateQuality( NULL );
	printf("result = %d\n", result );

	printf("Calling loadParameters:\t\t");
	error = dpLoadParameters( NULL, 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setSecurity:\t\t");
	error = dpSetSecurity( 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getSecurity:\t\t");
	error = dpGetSecurity( NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling identifyTemplate:\t\t");
	error = dpIdentifyTemplate( NULL, 0, NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setDbMaxSearch:\t\t");
	error = dpSetDbMaxSearch( 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	// then call function with NULL arguments.. it should return an error but not crash
	printf("\nCalling all functions with NULL data...\n\n");
	printf("Calling dpInit:\t\t");
	error = dpInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getMaxTemplateSize:\t\t");
	result =dpGetMaxTemplateSize();
	printf("result = %d\n", result );

	printf("Calling getTemplateSize:\t\t");
	result =dpGetTemplateSize( NULL );
	printf("result = %d\n", result );

	printf("Calling generateTemplate:\t\t");
	error = dpGenerateTemplate( NULL, 0, 0, NULL ); 
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling verifyTemplate:\t\t");
	error = dpVerifyTemplate( NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling verifyAndRefineTemplate:\t\t");
	error = dpVerifyAndRefineTemplate( NULL, NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getTemplateQuality:\t\t");
	result =dpGetTemplateQuality( NULL );
	printf("result = %d\n", result );

	printf("Calling loadParameters:\t\t");
	error = dpLoadParameters( NULL, 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setSecurity:\t\t");
	error = dpSetSecurity( 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getSecurity:\t\t");
	error = dpGetSecurity( NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling identifyTemplate:\t\t");
	error = dpIdentifyTemplate( NULL,0, NULL, NULL, NULL );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setDbMaxSearch:\t\t");
	error = dpSetDbMaxSearch( 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling dpUnInit :\t\t");
	error = dpUnInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	// Some real test with random data, it should not crash also...
	printf("\nCalling all functions with random data...\n\n");
	printf("Calling dpInit:\t\t");
	error = dpInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getMaxTemplateSize:\t\t");
	result = dpGetMaxTemplateSize();
	printf("result = %d\n", result );

	printf("Calling getTemplateSize:\t\t");
	result = dpGetTemplateSize( templBuff1 );
	printf("result = %d\n", result );

	printf("Calling verifyTemplate:\t\t");
	error = dpVerifyTemplate( templBuff1, templBuff2, &score );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling verifyAndRefineTemplate:\t\t");
	error = dpVerifyAndRefineTemplate( templBuff1, templBuff2, &score, &refined );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getTemplateQuality:\t\t");
	result = dpGetTemplateQuality( templBuff1 );
	printf("result = %d\n", result );

	printf("Calling loadParameters:\t\t");
	error = dpLoadParameters( templBuff1, 100 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setSecurity:\t\t");
	error = dpSetSecurity( rand() );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling getSecurity:\t\t");
	error = dpGetSecurity( &security );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling identifyTemplate:\t\t");
	error = dpIdentifyTemplate( templArray, nbTemplates , templBuff1, &score, &templatePos );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling setDbMaxSearch:\t\t");
	error = dpSetDbMaxSearch( rand() );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling generateTemplate:\t\t");
	error = dpGenerateTemplate( imgBuf, 500, 500, templBuff1 ); 
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling dpUnInit :\t\t");
	error = dpUnInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	// Some real test with random data and all size arguments set to zero, it should not crash also...
	printf("\nCalling some functions with random data ans sizes set to zero...\n\n");
	printf("Calling dpInit:\t\t");
	error = dpInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling generateTemplate:\t\t");
	error = dpGenerateTemplate( imgBuf, 0, 0, templBuff1 ); 
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling loadParameters:\t\t");
	error = dpLoadParameters( templBuff1, 0 );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling identifyTemplate:\t\t");
	nbTemplates = 0;
	error = dpIdentifyTemplate( templArray, nbTemplates , templBuff1, &score, &templatePos );
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");

	printf("Calling dpUnInit :\t\t");
	error = dpUnInit();
	if( error < ZEF_OK )
		printf("error # = %d\n", error );
	else
		printf("OK\n");
}