// BinStatDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dpBioTools.h"
#include "BinStatDlg.h"
#include "myFuncs.h"
#include "math.h"
#include "matcher.h"
#include "fxmatcher.h"
#include "zMaths.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBinStatDlg dialog


CBinStatDlg::CBinStatDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBinStatDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBinStatDlg)
	m_bMeanFreq = FALSE;
	m_searchString = _T("");
	m_select = 0;
	m_bIndex = FALSE;
	m_bDistCD = FALSE;
	m_txPenMoy = _T("");
	m_txPenTotal = _T("");
	//}}AFX_DATA_INIT
}


void CBinStatDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBinStatDlg)
	DDX_Control(pDX, IDC_SORTED_LIST, m_sortedList);
	DDX_Control(pDX, IDC_SELECTION, m_selection);
	DDX_Check(pDX, IDC_MEAN_FREQ, m_bMeanFreq);
	DDX_Text(pDX, IDC_SEARCH_STRING, m_searchString);
	DDX_Text(pDX, IDC_SELECT, m_select);
	DDX_Check(pDX, IDC_INDEX, m_bIndex);
	DDX_Check(pDX, IDC_DIST_CD, m_bDistCD);
	DDX_Text(pDX, IDC_TX_PEN_MOY, m_txPenMoy);
	DDX_Text(pDX, IDC_TX_PEN_TOTAL, m_txPenTotal);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBinStatDlg, CDialog)
	//{{AFX_MSG_MAP(CBinStatDlg)
	ON_LBN_SELCHANGE(IDC_SELECTION, OnSelchangeSelection)
	ON_BN_CLICKED(IDC_SEARCH, OnSearch)
	ON_LBN_SELCHANGE(IDC_SORTED_LIST, OnSelchangeSortedList)
	ON_BN_CLICKED(IDC_CALCL_TX_PEN, OnCalclTxPen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBinStatDlg message handlers

BOOL CBinStatDlg::OnInitDialog() 
{
	int  i;

	CDialog::OnInitDialog();

	BeginWaitCursor();

	m_fileNamesTab = new CString[m_nbFiles];
	m_indexTab = new int[m_nbFiles];
	m_templateTab = new Template[m_nbFiles];

	fillTabs( m_sourcePath.GetBuffer(256), ".bin", m_fileNamesTab, m_indexTab, m_templateTab );
	m_sourcePath.ReleaseBuffer();

	for( i=0; i<m_nbFiles; i++ )
	{
		if( m_templateTab[i].m_nbCores == 0 )
			m_selection.AddString( m_fileNamesTab[i] + " *");
		else
			m_selection.AddString( m_fileNamesTab[i] );
	}

	m_bDontRedraw = false;

	EndWaitCursor();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CBinStatDlg::OnSelchangeSelection() 
{
	
	CalcIndex(m_selection.GetCurSel());
	
}

void CBinStatDlg::OnSearch() 
{
	UpdateData(true);
	m_sortedList.SelectString(m_sortedList.GetCurSel(),m_searchString+"\\" ); 
	m_select  = m_sortedList.GetCurSel();
	UpdateData(false);	
}

void CBinStatDlg::OnSelchangeSortedList() 
{
	m_select  = m_sortedList.GetCurSel();
	UpdateData(false);
	
}

void CBinStatDlg::OnCalclTxPen() 
{
	int		i;
	float	txPen=0.0f;

	BeginWaitCursor();
	
	m_bDontRedraw = true;

	for( i=0; i<m_nbFiles; i++ )
	{
		CalcIndex(i);
		txPen += m_txPen;
	}

	txPen /= m_nbFiles;

	m_txPenTotal.Format("Tx Pen Total : %.2f", txPen);

	m_bDontRedraw = false;

	UpdateData(false);

	EndWaitCursor();

}

void CBinStatDlg::CalcIndex(int p_selection)
{
	int				i,i1,i2,selection,x,y,dist1,dist2;
	unsigned int	score;
	float			txFreq;
	CString			str;
	int				txPenMoy=0;

	UpdateData(true);

	selection = p_selection;

	float *diffFreq=new float[m_nbFiles];

	// reinit indexes
	for( i=0; i<m_nbFiles; i++ )
	{
		diffFreq[i] = 0.0f;
		m_indexTab[i] = i;
	}

	

	if( m_bMeanFreq ) 
	{
		
		for( i=0; i<m_nbFiles; i++ )
		{
			txFreq = (float)fabs(m_templateTab[selection].m_meanFreq-m_templateTab[i].m_meanFreq);
			if( m_templateTab[selection].m_meanFreq != 0 && m_templateTab[i].m_meanFreq != 0 )
			{
				txFreq /= 100.0f;
			}
			else
				txFreq = 1;

			diffFreq[i] += 1000*txFreq;
		}
	
	}

	if( m_bIndex && m_templateTab[selection].m_nbCores != 0 )
	{
		for( i=0; i<m_nbFiles; i++ )
		{
			score = scoreIndex( &(m_templateTab[selection]), &(m_templateTab[i]) );

			if( score < 200 )
				diffFreq[i] += score/2;
			else
				diffFreq[i] += 100;

		}
	}

	if( m_bDistCD && m_templateTab[selection].m_nbCores != 0 )
	{
		for( i=0; i<m_nbFiles; i++ )
		{
			score = 1;
			if( m_templateTab[selection].m_nbCores == 2 && m_templateTab[i].m_nbCores == 2)
			{
				x = m_templateTab[selection].m_cores[0].m_x - m_templateTab[selection].m_cores[1].m_x;
				y = m_templateTab[selection].m_cores[0].m_y - m_templateTab[selection].m_cores[1].m_y;
				dist1 = zSqrt(x*x+y*y);

				x = m_templateTab[i].m_cores[0].m_x - m_templateTab[i].m_cores[1].m_x;
				y = m_templateTab[i].m_cores[0].m_y - m_templateTab[i].m_cores[1].m_y;
				dist2 = zSqrt(x*x+y*y);

				if( zAbs(dist1 - dist2) < 12 )
					score++;
			}

			if( m_templateTab[selection].m_nbDeltas > 0 && m_templateTab[i].m_nbDeltas > 0)
			{
				x = m_templateTab[selection].m_cores[0].m_x - m_templateTab[selection].m_deltas[0].m_x;
				y = m_templateTab[selection].m_cores[0].m_y - m_templateTab[selection].m_deltas[0].m_y;
				dist1 = zSqrt(x*x+y*y);

				x = m_templateTab[i].m_cores[0].m_x - m_templateTab[i].m_deltas[0].m_x;
				y = m_templateTab[i].m_cores[0].m_y - m_templateTab[i].m_deltas[0].m_y;
				dist2 = zSqrt(x*x+y*y);

				if( zAbs(dist1 - dist2) < 12 )
					score++;
			}

			diffFreq[i] += 100/score;

		}
	}

	i2 = m_fileNamesTab[selection].Find("\\",0);
	m_searchString = m_fileNamesTab[selection].Left(i2);

	trier( diffFreq, m_indexTab, 0, m_nbFiles-1 );
	m_sortedList.ResetContent();
	for( i=0; i<m_nbFiles; i++ )
	{
		if( !m_bDontRedraw )
			str.Format("%s - %.3f - %d",m_fileNamesTab[m_indexTab[i]],diffFreq[i],m_templateTab[m_indexTab[i]].m_meanFreq);
		if( m_templateTab[m_indexTab[i]].m_nbCores == 0 )
			str += " *";
		if( !m_bDontRedraw )
			m_sortedList.AddString( str );
		i1 = m_fileNamesTab[m_indexTab[i]].Find("\\",0);
		
		if( m_fileNamesTab[m_indexTab[i]].Left(i1+1) == m_fileNamesTab[selection].Left(i2+1) )
			txPenMoy += i;
		m_txPen = 100*(txPenMoy/8.0f)/m_nbFiles;
		if( !m_bDontRedraw )
			m_txPenMoy.Format("Tx Pen Moy : %.2f/100",m_txPen);
	}

	delete[] diffFreq;

	UpdateData(false);

}
