// fxPrecomp.cpp : Defines the entry point for the console application.
//

#include "stdio.h"
#include "customTypes.h"

#include "zMaths.h"
#include "fxEnhancements.h"


int main(int argc, char* argv[])
{
	FILE *file=NULL;
	int	i,j,k;

#if defined(USE_FX_PRECOMP)
	printf("USE_FX_PRCOMP is defined... Please undefine it and relaunch !!!\n");
	goto ErrorHandler;
#endif

	printf("Source file generation : In Progress\n");

	// Maths part
	if( zInitMaths() == ZEF_OK )
	{
		file = fopen("../../src/zFxMaths.c","w");
		if( file == NULL )
			goto ErrorHandler;

		fprintf(file,"// zFxMaths.c : Precomputed maths values for maths functs\n\n");
		fprintf(file,"#include \"customTypes.h\"\n");
		fprintf(file,"#include \"zMaths.h\"\n\n");
		fprintf(file,"#if defined(USE_FX_PRECOMP)\n\n");

		fprintf(file,"// initialize precomputed arrays \n\n");
		
		// sin and cos for spectrum processing
		fprintf(file,"// initialize g_precCos32Dec array\n");
		fprintf(file,"INT32 g_precCosSpectrum[SPECTRUM_RESOLUTION] = {");
		for( i=0; i<SPECTRUM_RESOLUTION-1; i++ )
		{
			fprintf(file,"%d, ",g_precCosSpectrum[i]);
		}
		fprintf(file,"%d };\n",g_precCosSpectrum[SPECTRUM_RESOLUTION-1]);

		fprintf(file,"// initialize g_preSin32Dec array\n");
		fprintf(file,"INT32 g_precSinSpectrum[SPECTRUM_RESOLUTION] = {");
		for( i=0; i<SPECTRUM_RESOLUTION-1; i++ )
		{
			fprintf(file,"%d, ",g_precSinSpectrum[i]);
		}
		fprintf(file,"%d };\n",g_precSinSpectrum[SPECTRUM_RESOLUTION-1]);


		// init precomp Cos and Sin tables
		fprintf(file,"\n// init precomp Cos table\n");
		fprintf(file,"INT8 g_precCos[256] = {");
		for( i=0; i<255; i++ )
		{
			fprintf(file,"%d, ",g_precCos[i]);
		}
		fprintf(file,"%d };\n",g_precCos[255]);

		fprintf(file,"\n// init precomp Sin table\n");
		fprintf(file,"INT8 g_precSin[256] = {");
		for( i=0; i<255; i++ )
		{
			fprintf(file,"%d, ",g_precSin[i]);
		}
		fprintf(file,"%d };\n",g_precSin[255]);

		// init precomp atan and sqrt for spectrum
		fprintf(file,"\n// init precomp atan for spectrum\n");
		fprintf(file,"UINT8 g_sqrtSpec[FFT_BLOC_SIZE][FFT_BLOC_SIZE/2] = { ");
		for( j=0; j<FFT_BLOC_SIZE; j++ )
		{
			fprintf(file,"{ ");
			for( i=0; i<FFT_BLOC_SIZE/2; i++ )
			{
				fprintf(file,"%d",g_sqrtSpec[j][i]);
				if( i != (FFT_BLOC_SIZE/2)-1 )
					fprintf(file,", ");
			}
			fprintf(file," }");
			if( j != FFT_BLOC_SIZE-1 )
				fprintf(file,",\n");
		}
		fprintf(file," };\n");

		fprintf(file,"\n// init precomp sqrt for spectrum\n");
				fprintf(file,"UINT8 g_atanSpec[FFT_BLOC_SIZE][FFT_BLOC_SIZE/2] = { ");
		for( j=0; j<FFT_BLOC_SIZE; j++ )
		{
			fprintf(file,"{ ");
			for( i=0; i<FFT_BLOC_SIZE/2; i++ )
			{
				fprintf(file,"%d",g_atanSpec[j][i]);
				if( i != (FFT_BLOC_SIZE/2)-1 )
					fprintf(file,", ");
			}
			fprintf(file," }");
			if( j != FFT_BLOC_SIZE-1 )
				fprintf(file,",\n");
		}
		fprintf(file," };\n");

		// log table
		fprintf(file,"\n// log table\n");
		fprintf(file,"FIXED32 g_log2Table[1025] = { ");
		for( i=0; i<1024; i++ )
		{
			fprintf(file,"%d, ",g_log2Table[i]);	
		}
		fprintf(file,"%d };\n",g_log2Table[1024]);

		//atan table
		fprintf(file,"\n\t//atan table\n");
		fprintf(file,"UINT8 g_atanTable[257] = { ");
		for( i=0; i<256; i++ )
		{
			fprintf(file,"%d, ",g_atanTable[i]);
		}
		fprintf(file,"%d };\n",g_atanTable[256]);

		// exp(-x) for 0 <= x <= 8
		fprintf(file,"\n//exp(-x) for 0 <= x <= 8\n");
		fprintf(file,"FIXED16 g_invExp[512] = { ");
		for( i=0; i<511; i++ )
		{	
			fprintf(file,"%d, ",g_invExp[i]);	
		}
		fprintf(file,"%d };\n",g_invExp[511]);

		fprintf(file,"#endif //USE_FX_PRECOMP)\n");	

		fclose(file);
	}

	// FFT/enhancement part
	if( initEnhance() == ZEF_OK )
	{
		fopen("../../include/fxEnhancePrecomp.h","w");

		if( file == NULL )
			goto ErrorHandler;

		fprintf(file,"// fxEnhancePrecomp.h : Precomputed values for FFT and enhancement\n\n");
		(file,"#include \"customTypes.h\"\n");

		fprintf(file,"\n#if defined(USE_FX_PRECOMP)\n\n");

		fprintf(file,"// fill precomp arrays with the correct values \n");

		// trigo data for FFT
		fprintf(file,"\n\t// trigo data for FFT\n");
		fprintf(file,"\n\tFIXED16		g_fftTrigo[%d]={",3*FFT_BLOC_SIZE/4);
		for( i=0; i<(3*FFT_BLOC_SIZE/4)-1; i++ )
		{
			fprintf(file,"%d, ",g_preCompData.m_fftTrigo[i]);
		}
		fprintf(file,"%d ",g_preCompData.m_fftTrigo[(3*FFT_BLOC_SIZE/4)-1]);
		fprintf(file,"};\n");

		// in place data for FFT
		fprintf(file,"\n\t// in place data for FFT\n");
		fprintf(file,"\n\tINT32		g_fftInPlace[%d]={",(2+(INT32)zSqrt(FFT_BLOC_SIZE)));
		for( i=0; i<(2+(INT32)zSqrt(FFT_BLOC_SIZE))-1; i++ )
		{
			fprintf(file," %d, ",g_preCompData.m_fftInPlace[i]);
		}
		fprintf(file,"%d ",g_preCompData.m_fftInPlace[(2+(INT32)zSqrt(FFT_BLOC_SIZE))-1]);
		fprintf(file,"};\n");

		// precalc butterworth filter
		fprintf(file,"\n\t// precalc butterworth filter\n");
		fprintf(file,"\n\tFIXED16		g_butterworth[%d]={",FFT_BLOC_SIZE*FFT_BLOC_SIZE/2);
		k=0;
		for( j=0; j<FFT_BLOC_SIZE; j++ )
		{
			for( i=0; i<FFT_BLOC_SIZE/2; i++ )
			{
				fprintf(file,"%d",g_preCompData.m_butterworth[k]);
				k++;
				if( k != FFT_BLOC_SIZE*FFT_BLOC_SIZE/2 )
					fprintf(file,", ");
			}
			fprintf(file,"\n");
		}
		fprintf(file,"};\n");

		fprintf(file,"#endif //USE_FX_PRECOMP)\n");	

		fclose(file);
	}


	printf("Source file generation : OK\n");

	return 0;

ErrorHandler:

	printf("Source file generation : FAILED\n");

	if( file != NULL )
		fclose(file);

	return -1;
}
