/*
Company :		ZEFYR Technologies
File :			linAlgebra.c
Author :		Matthieu Perreira Da Silva 
Fisrt release :	04/07/2003

Description : linear system resolution

Features : finds the solutyions of a linear sysem
    
Functions : 

	luDecompose : transform a matrix into its LU decomposition (used to solve a linear system)
	luBackSubstitute : resolve a system using its LU decomposition

Function prototypes :

	INT32 luDecompose(float **p_matrix, int p_size, int *p_index, float *p_evenOdd);
	INT32 luBackSubstitute(float **p_matrix, int p_size, int *p_index, float p_rightSide[]);

Necessary package : 

	customTypes.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h
	zMaths.h

Comments : none

*/

#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <string.h>

#include "customTypes.h"
#include "errorCodes.h"
#include "staticParameters.h"
#include "dynamicParameters.h"
#include "zMaths.h"

#include "linAlgebra.h"

#if !defined(ZEF_FIXED_POINT) && defined(ZEF_CORRECT_DEFORMATION)

#define TINY 1.0e-20f


INT32 luDecompose(REAL32 **p_matrix, INT32 p_size, INT32 *p_index, REAL32 *p_evenOdd)
{
	
	INT32	i,imax,j,k;
	REAL32	big,dum,sum,temp;
	REAL32	*vv;
	INT32	error;
	
	vv=(REAL32*)MALLOC(p_size*sizeof(REAL32));;

	*p_evenOdd=1.0; 
	for (i=0;i<p_size;i++)
	{ 
		big=0.0;
		for (j=0;j<p_size;j++)
		{
			if ((temp=zfAbs(p_matrix[i][j])) > big) big=temp;
		}
		if (big == 0.0)
		{
			error = ZEF_MATHS_ERROR;
			goto errorHandler;
		}
		
		vv[i]=1.0f/big; 
	}
	for (j=0;j<p_size;j++) { 
		for (i=0;i<j;i++) {
			sum=p_matrix[i][j];
			for (k=0;k<i;k++) {
				sum -= p_matrix[i][k]*p_matrix[k][j];
			}
			p_matrix[i][j]=sum;
		}
		big=0.0; 
		for (i=j;i<p_size;i++) { 
			sum=p_matrix[i][j];
			for (k=0;k<j;k++){
				sum -= p_matrix[i][k]*p_matrix[k][j];
			}
			p_matrix[i][j]=sum;
			if ( (dum=vv[i]*zfAbs(sum)) >= big) {
				
				big=dum;
				imax=i;
			}
		}
		if (j != imax) { 
			for (k=0;k<p_size;k++) { 
				dum=p_matrix[imax][k];
				p_matrix[imax][k]=p_matrix[j][k];
				p_matrix[j][k]=dum;
			}
			*p_evenOdd = -(*p_evenOdd); 
			vv[imax]=vv[j]; 
		}
		p_index[j]=imax;
		if (p_matrix[j][j] == 0.0) p_matrix[j][j]=TINY;
		
		if (j != p_size-1) {
			dum=1.0f/(p_matrix[j][j]);
			for (i=j+1;i<p_size;i++) p_matrix[i][j] *= dum;
		}
	} 

	FREE(vv);

	return ZEF_OK;

	// errorHandling
errorHandler:

	FREE(vv);

	return error;
}


INT32 luBackSubstitute(REAL32 **p_matrix, INT32 p_size, INT32 *p_index, REAL32 p_rightSide[])
{
	INT32 i,ii=-1,ip,j;
	REAL32 sum;
	
	
	for (i=0;i<p_size;i++)
	{ 
		ip=p_index[i];
		sum=p_rightSide[ip];
		p_rightSide[ip]=p_rightSide[i];
		if (ii != -1)
		{
			for (j=ii;j<=i-1;j++) 
				sum -= p_matrix[i][j]*p_rightSide[j];
		}
		else if (sum)
			ii=i;
		p_rightSide[i]=sum;
	}
	for (i=p_size-1;i>=0;i--)
	{ 
		sum=p_rightSide[i];
		for (j=i+1;j<p_size;j++) 
			sum -= p_matrix[i][j]*p_rightSide[j];
		p_rightSide[i]=sum/p_matrix[i][i]; 
	} 

	return ZEF_OK;
}

#endif // ZEF_FIXED_POINT
