/*
Company :		compatibility.c
Author :		Matthieu Perreira Da Silva 
Fisrt release :	15/04/2003

Description : functions for testing new version of the algorithm with the old template format 

Features : template convertion and display functions	
    
Functions : 
	 
	convertToOldGabV2 :	convert a new version template to an old version template
	displayMinutiae :	display a new template to an unsigned char array
	
Prototypes : 

	INT32 convertToOldGabV2( unsigned char *p_src, unsigned char *p_dst );
	INT32 displayMinutiae( UINT8 *p_src, UINT8 *p_dst, UINT8 *p_qual,  UINT32 p_width, UINT32 p_height, 
							Template *p_template, UINT32 p_xCore, UINT32 p_yCore );


Necessary package : 
	customTypes.h
	errorCodes.h
	tools.h
	zMaths.h

Comments : just for tests

*/

#include "stdio.h"

#include "customTypes.h"
#include "errorCodes.h"
#include "tools.h"
#include "zMaths.h"
#include "dynamicParameters.h"

#include "compatibility.h"

INT32 convertToOldGabV2( UINT8 *p_src, UINT8 *p_dst )
{
	UINT32		i;
	GabaritV2	*gab;
	Template	myTemplate;

	gab = (GabaritV2*)p_dst;

	templateInit( &myTemplate );
	templateUnpacker( p_src, &myTemplate );

	gab->m_nbCores = myTemplate.m_nbCores;
	gab->m_nbDeltas = myTemplate.m_nbDeltas;
	for( i=0; i<2; i++ )
	{
		gab->m_cores[i].m_x = myTemplate.m_cores[i].m_x;
		gab->m_cores[i].m_y = myTemplate.m_cores[i].m_y;

		gab->m_deltas[i].m_x = myTemplate.m_deltas[i].m_x;
		gab->m_deltas[i].m_y = myTemplate.m_deltas[i].m_y;
	}

	gab->m_decXtab = myTemplate. m_localShiftX;
	gab->m_decYtab = myTemplate.m_localShiftY;

	gab->m_lTabX = myTemplate.m_localW;
	gab->m_hTabY = myTemplate.m_localH;

	gab->m_freqMoy = myTemplate.m_meanFreq;

	gab->m_nbMinuties = myTemplate.m_nbMinutiae;

	for( i=0; i<myTemplate.m_nbMinutiae; i++ )
	{
		gab->m_minuties[i].m_coord.m_x = myTemplate.m_minutiae[i].m_coord.m_x;
		gab->m_minuties[i].m_coord.m_y = myTemplate.m_minutiae[i].m_coord.m_y;
		gab->m_minuties[i].m_orient = myTemplate.m_minutiae[i].m_ori;
		gab->m_minuties[i].m_quality = myTemplate.m_minutiae[i].m_qual;
	}

	for( i=0; i<(UINT32)myTemplate.m_localW*myTemplate.m_localH; i++ )
	{
		gab->m_tabLocal[i].m_orient = myTemplate.m_ori[i];
		gab->m_tabLocal[i].m_qualAndVarOri = (myTemplate.m_qual[i])<<4;
	}

	gab->m_xMin = myTemplate.m_boundBox.m_left;
	gab->m_xMax = myTemplate.m_boundBox.m_right;
	gab->m_yMin = myTemplate.m_boundBox.m_top;
	gab->m_yMax = myTemplate.m_boundBox.m_bottom;

	templateUninit( &myTemplate );

	return ZEF_OK;
}

INT32 displayMinutiae( UINT8 *p_src, UINT8 *p_dst, UINT8 *p_qual,  UINT32 p_width, UINT32 p_height, Template *p_template,
						UINT32	p_xCore, UINT32 p_yCore )
{
	UINT32	x,y,k,l,qual,xx,yy;
	INT32	theta;

	// init 
	k=0;

	// write skeleton
	for( y=0; y<p_height; y++)
	{
		for( x=0; x<p_width; x++)
		{
			if( p_src[k] == 0 )
				p_dst[k] = 196;
			else
				p_dst[k] = p_qual[(y/FUSION_BLOC_SIZE)*(p_width/FUSION_BLOC_SIZE)+(x/FUSION_BLOC_SIZE)];
			k++;
		}
	}

	//write minutiae
	for( k=0; k<p_template->m_nbMinutiae; k++ )
	{
		x=p_xCore+(p_template->m_minutiae[k].m_coord.m_x*(INT32)g_params.m_globalImgScaling/256);
		y=p_yCore+(p_template->m_minutiae[k].m_coord.m_y*(INT32)g_params.m_globalImgScaling/256);
		theta = p_template->m_minutiae[k].m_ori*2;
		qual = 255-p_template->m_minutiae[k].m_qual;
		p_dst[y*p_width+x] = 0;
		for( l=1; l<6; l++ )
		{
			xx = (UINT32)(x+l*cos(theta*PI/255.0));
			yy = (UINT32)(y+l*sin(theta*PI/255.0));
			p_dst[yy*p_width+xx] = 255-p_dst[yy*p_width+xx];
		}
	}

	return ZEF_OK;
}
