/*
Company :		ZEFYR Technologies
File :			customTypes.c
Author :		Matthieu Perreira Da Silva 
Fisrt release :	05/05/2003

Description : Custom types defined for SDK use

Features : defines some custom types		
    
Functions : 

			swapUInt32 : swap long data type from/to little indian (intel) from/to big indian (motorola) 
			swapUInt16: swap short data type from/to little indian (intel) from/to big indian (motorola) 

Structures : 

Necessary package : none

Comments :none

*/

#include "customTypes.h"

/*---------------------------------------------------------------------------
**  swapUInt32()
**
**  Swap the bytes of the Unsigned Long so the file data is 68000 compatible.
**  This function reverses back and fourth between "little indian" (Intel)
**  and, "big indian" (Motorola) memory format.
---------------------------------------------------------------------------*/
UINT32 swapUInt32(UINT32 p_data)
{
   UINT32 tmpData;

   *((UINT8 *)&tmpData + 3) = *((UINT8 *)&p_data);
   *((UINT8 *)&tmpData + 2) = *((UINT8 *)&p_data + 1);
   *((UINT8 *)&tmpData + 1) = *((UINT8 *)&p_data + 2);
   *((UINT8 *)&tmpData)     = *((UINT8 *)&p_data + 3);
   return(tmpData);
}

/*---------------------------------------------------------------------------
**  swapUInt16()
**
**  Swap the bytes of the Unsigned Short so the file data is 68000 compatible.
**  This function reverses back and fourth between "little indian" (Intel)
**  and, "big indian" (Motorola) memory format.
---------------------------------------------------------------------------*/
UINT16 swapUInt16(UINT16 p_data)
{
   UINT16 tmpData;

   *((UINT8 *)&tmpData + 1) = *((UINT8 *)&p_data);
   *((UINT8 *)&tmpData)     = *((UINT8 *)&p_data + 1);
   return(tmpData);
}

