/*
Company :		ZEFYR Technologies
File :			memMonitor.h
Author :		Matthieu Perreira Da Silva 
Fisrt release :	17/06/2005

Description : memory monitoring tools

Features : check if no memory leak was left + max mem usage, etc...	
    
Functions :

		MALLOC	: our version of malloc
		FREE	: our veriosn of free

Necessary package : 

	staticParameters.h

Comments :none

*/

#include "memMonitor.h"
#include <stdio.h>
#include <string.h>

// static variables for memory management
static long g_allocatedMem = 0;
static long	*g_allocSize;
static void **g_allocPointer;
static char **g_allocFile;
static int	*g_allocLine;
static long	g_nbBlocs = 0;
static long	g_maxNbBlocs = 256;
static long	g_nextBlocPos = 0; 
static long	g_peakMemUsage = 0;

#if defined(MEM_DEBUG)
// memory management functions
void	*myMalloc( size_t p_size, char *p_file, int p_line )
{
	long	i;
	void	*pointer;

	// allocations or reallocation
	if( g_nbBlocs == 0 )
	{
		g_allocSize  = (long*)malloc( g_maxNbBlocs*sizeof(long) );
		g_allocPointer = (void**)malloc( g_maxNbBlocs*sizeof(void*) );
		g_allocFile = (char**)malloc( g_maxNbBlocs*sizeof(char*) );
		g_allocLine = (int*)malloc( g_maxNbBlocs*sizeof(int) );
	}
	else if( g_nextBlocPos >= g_maxNbBlocs )
	{
		g_maxNbBlocs += 256;
		g_allocSize  = (long*)realloc( g_allocSize, g_maxNbBlocs*sizeof(long) );
		g_allocPointer = (void**)realloc( g_allocPointer, g_maxNbBlocs*sizeof(void*) );
		g_allocFile = (char**)realloc( g_allocFile, g_maxNbBlocs*sizeof(char*) );
		g_allocLine = (int*)realloc( g_allocLine, g_maxNbBlocs*sizeof(int) );
	}

	// allocate memory bloc
	pointer = malloc( p_size );

	// update stat arrays
	for( i=0; i<g_nextBlocPos; i++ )
	{
		if( g_allocPointer[i] == NULL )
			break;
	}
	g_allocatedMem += p_size;
	g_allocSize[i] = p_size;
	g_allocPointer[i] = pointer;
	g_allocFile[i] = (char*)malloc((strlen(p_file)+1)*sizeof(char));
	strcpy(g_allocFile[i],p_file);
	g_allocLine[i] = p_line;

	if( g_allocatedMem > g_peakMemUsage )
		g_peakMemUsage = g_allocatedMem;

	if( i == g_nextBlocPos ) 
		g_nextBlocPos++;

	g_nbBlocs++;

	return pointer;
}

void	*myCalloc( size_t p_nb, size_t p_size,  char *p_file, int p_line )
{
	long	i;
	void	*pointer;

	// allocations or reallocation
	if( g_nbBlocs == 0 )
	{
		g_allocSize  = (long*)malloc( g_maxNbBlocs*sizeof(long) );
		g_allocPointer = (void**)malloc( g_maxNbBlocs*sizeof(void*) );
		g_allocFile = (char**)malloc( g_maxNbBlocs*sizeof(char*) );
		g_allocLine = (int*)malloc( g_maxNbBlocs*sizeof(int) );
	}
	else if( g_nextBlocPos >= g_maxNbBlocs )
	{
		g_maxNbBlocs += 256;
		g_allocSize  = (long*)realloc( g_allocSize, g_maxNbBlocs*sizeof(long) );
		g_allocPointer = (void**)realloc( g_allocPointer, g_maxNbBlocs*sizeof(void*) );
		g_allocFile = (char**)realloc( g_allocFile, g_maxNbBlocs*sizeof(char*) );
		g_allocLine = (int*)realloc( g_allocLine, g_maxNbBlocs*sizeof(int) );
	}

	// allocate memory bloc
	pointer = calloc( p_nb, p_size );

	// update stat arrays
	for( i=0; i<g_nextBlocPos; i++ )
	{
		if( g_allocPointer[i] == NULL )
			break;
	}
	g_allocatedMem += p_size;
	g_allocSize[i] = p_size;
	g_allocPointer[i] = pointer;
	g_allocFile[i] = (char*)malloc((strlen(p_file)+1)*sizeof(char));
	strcpy(g_allocFile[i],p_file);
	g_allocLine[i] = p_line;

	if( g_allocatedMem > g_peakMemUsage )
		g_peakMemUsage = g_allocatedMem;

	if( i == g_nextBlocPos ) 
		g_nextBlocPos++;

	g_nbBlocs++;

	return pointer;
}


void	myFree( void *p_pointer, char *p_file, int p_line )
{
	int i;

	// find bloc
	for( i=0; i<g_nextBlocPos; i++ )
	{
		if( g_allocPointer[i] == p_pointer )
			break;
	}

	if( i != g_nextBlocPos )
	{
		g_allocatedMem -= g_allocSize[i];
		g_allocSize[i] = 0;
		g_allocPointer[i] = NULL;
		free(g_allocFile[i]);
		g_nbBlocs--;
		if( i == g_nextBlocPos-1 )
			g_nextBlocPos--;
	}
	else
	{
		// sould never happen !!!
		printf("free unknown bloc : [0x%08x], (%s,%d)\n",p_pointer,p_file,p_line);
	}

	
	free( p_pointer); 
}

void getPeakMem()
{
	printf("Peak Memory  = %d\n",g_peakMemUsage);
}

void getAllocatedMem()
{
	int i;

	printf("Allocated Memory  = %d\n",g_allocatedMem);
	for( i=0; i<g_nextBlocPos; i++ )
	{
		if( g_allocPointer[i] != NULL )
		{
			printf("[0x%08x], size=%d (%s,%d)\n",g_allocPointer[i],g_allocSize[i],g_allocFile[i],g_allocLine[i]);
		}
	}
}

#endif