/*
Company :		ZEFYR Technologies
File :			tools.c
Author :		Matthieu Perreira Da Silva 
Fisrt release :	13/06/2003

Description : some usefull routines

Features : many	
    
Functions :

	templatePacker : fill an array of butes with template structure informations
	templateUnpacker : fill a template structure information with the content of an array of bytes
	templateInit : init template strucuture
	templateUninit : FREE memory allocated for template structure

Function prototypes :

	INT32 templatePacker( Template *p_template, UINT8 *p_buffer );
	INT32 templateUnpacker( UINT8 *p_buffer, Template *p_template );
	INT32 templateInit( Template *p_template );
	INT32 templateUninit( Template *p_template );

Necessary package : 

	customTypes.h
	minutiae.h
	errorCodes.h
	staticParameters.h
	dynamicParameters.h

Comments :none

*/

#include <stdlib.h>
#include <string.h>

#include "customTypes.h"
#include "minutiae.h"
#include "errorCodes.h"
#include "staticParameters.h"
#include "dynamicParameters.h"

#include "tools.h"

// fill buffer with template data
INT32 templatePacker( Template *p_template, UINT8 *p_buffer )
{
	UINT32	offset,i,j;
	UINT8	val;
	UINT32	nbCores;
	UINT16	checksum;

	// check parameters
	if( p_buffer == NULL || p_template == NULL )
		return ZEF_NULL_POINTER;
	
	// init
	offset=0;
	checksum=0;
	
	// size
	WRITE_INT32(p_buffer+offset,p_template->m_size);
	offset += sizeof(UINT32);
	// version
	WRITE_INT32(p_buffer+offset,p_template->m_version);
	offset += sizeof(UINT32);
	// nb cores
	WRITE_INT8(p_buffer+offset,p_template->m_nbCores);
	offset += sizeof(UINT8);
	// nb deltas
	WRITE_INT8(p_buffer+offset,p_template->m_nbDeltas);
	offset += sizeof(UINT8);
	// cores
	nbCores = p_template->m_nbCores;
	if( nbCores == 0 )
		nbCores = 1; // always 1 core
	// other cores
	for( i=0; i<nbCores; i++ )
	{
		WRITE_INT16(p_buffer+offset,p_template->m_cores[i].m_x);
		offset += sizeof(INT16);	
		WRITE_INT16(p_buffer+offset,p_template->m_cores[i].m_y);
		offset += sizeof(INT16);
	}
	// deltas
	for( i=0; i<p_template->m_nbDeltas; i++ )
	{
		WRITE_INT16(p_buffer+offset,p_template->m_deltas[i].m_x);
		offset += sizeof(INT16);	
		WRITE_INT16(p_buffer+offset,p_template->m_deltas[i].m_y);
		offset += sizeof(INT16);
	}

	// 1st core ori
	WRITE_INT8(p_buffer+offset,p_template->m_oriCore);
	offset += sizeof(UINT8);

	// bounding box
	WRITE_INT16(p_buffer+offset,p_template->m_boundBox.m_left);
	offset += sizeof(INT16);
	WRITE_INT16(p_buffer+offset,p_template->m_boundBox.m_right);
	offset += sizeof(INT16);
	WRITE_INT16(p_buffer+offset,p_template->m_boundBox.m_top);
	offset += sizeof(INT16);
	WRITE_INT16(p_buffer+offset,p_template->m_boundBox.m_bottom);
	offset += sizeof(INT16);

	// number of munitiae
	WRITE_INT16(p_buffer+offset,p_template->m_nbMinutiae);
	offset += sizeof(UINT16);

	// minutiae
	for( i=0; i<p_template->m_nbMinutiae; i++ )
	{
		WRITE_INT16(p_buffer+offset,p_template->m_minutiae[i].m_coord.m_x);
		offset += sizeof(INT16);
		WRITE_INT16(p_buffer+offset,p_template->m_minutiae[i].m_coord.m_y);
		offset += sizeof(INT16);
		WRITE_INT8(p_buffer+offset,p_template->m_minutiae[i].m_ori);
		offset += sizeof(UINT8);
		WRITE_INT8(p_buffer+offset,p_template->m_minutiae[i].m_qual);
		offset += sizeof(UINT8);
	}

	// mean frequency
	WRITE_INT16(p_buffer+offset,p_template->m_meanFreq);
	offset += sizeof(UINT16);

	// bloc Size
	WRITE_INT8(p_buffer+offset,p_template->m_blocSize);
	offset += sizeof(UINT8);
		
	// width and height of local infos array
	WRITE_INT8(p_buffer+offset,p_template->m_localW);
	offset += sizeof(UINT8);
	WRITE_INT8(p_buffer+offset,p_template->m_localH);
	offset += sizeof(UINT8);

	// shift between center of the arrays and first core
	WRITE_INT16(p_buffer+offset,p_template->m_localShiftX);
	offset += sizeof(INT16);
	WRITE_INT16(p_buffer+offset,p_template->m_localShiftY);
	offset += sizeof(INT16);

	// ori array
	for( i=0; i<(UINT32)(p_template->m_localW*p_template->m_localH); i++ )
	{
		WRITE_INT8(p_buffer+offset,p_template->m_ori[i]);
		offset += sizeof(UINT8);				
	}

	// qual array
	j=0;
	val=0;
	for( i=0; i<(UINT32)(p_template->m_localW*p_template->m_localH); i++ )
	{
		val = val+(((p_template->m_qual[i]/16)&15)<<j); // encode qual with 4 bits
		j+=4;
		if( j > 4 )
		{
			WRITE_INT8(p_buffer+offset,val);
			offset += sizeof(UINT8);
			val=0;
			j=0;
		}
	}
	if( j!=0 )
	{
		WRITE_INT8(p_buffer+offset,val);
		offset += sizeof(UINT8);
	}

	// template signature
	for( i=0; i<16; i++ )
	{
		WRITE_INT8(p_buffer+offset,p_template->m_sign[i]);
		offset += sizeof(UINT8);
	}

	// update size
	WRITE_INT32(p_buffer,offset+sizeof(UINT16)); // sizeof(UINT16) = sizeof checksum

	// calculate and write checksum
	for( i=0; i<offset; i++ )
		checksum += p_buffer[i];	
	WRITE_INT16(p_buffer+offset,checksum);
		offset += sizeof(UINT16);	
	
	return ZEF_OK;
}

// fill template data with buffer
INT32 templateUnpacker( UINT8 *p_buffer, Template *p_template )
{
	UINT32	offset,i,j;
	UINT8	val;
	UINT32	nbCores;
	UINT16	checksum,templateChecksum;
	INT32	error;
	UINT32	localInfosBlocSize,globalImgScaling,globalTemplateScaling;

	// check parameters
	if( p_buffer == NULL || p_template == NULL )
		return ZEF_NULL_POINTER;

	// init
	LOCK_MUTEX( g_paramsMutex );
		localInfosBlocSize = g_params.m_localInfosBlocSize;
		globalImgScaling = g_params.m_globalImgScaling;
		globalTemplateScaling = g_params.m_globalTemplateScaling;
	UNLOCK_MUTEX( g_paramsMutex );
	offset=0;
	checksum = 0;
	
	// size
	READ_INT32(p_template->m_size,(p_buffer+offset));
	offset += sizeof(UINT32);
	// version
	READ_INT32(p_template->m_version,(p_buffer+offset));
	offset += sizeof(UINT32);

	// check version
	if( p_template->m_version != TEMPLATE_VERSION )
	{
		error = ZEF_BAD_VERSION;
		goto errorHandler;
	}

	// checksum checking
	for( i=0; i<p_template->m_size-sizeof(UINT16); i++ )
		checksum += p_buffer[i];

  READ_INT16(templateChecksum,(p_buffer+p_template->m_size-sizeof(UINT16)));
	if( checksum != templateChecksum )
	{
		error = ZEF_CHECKSUM_FAILED;
		goto errorHandler;
	}

	// nb cores
	READ_INT8(p_template->m_nbCores,(p_buffer+offset));
	offset += sizeof(UINT8);
	// nb deltas
	READ_INT8(p_template->m_nbDeltas,(p_buffer+offset));
	offset += sizeof(UINT8);
  
	// cores
	//allocation
	nbCores = p_template->m_nbCores;
	if( nbCores == 0 )
		nbCores = 1; // there is always 1 core even if it's not specified
	p_template->m_cores = (Coord16*)MALLOC(nbCores*sizeof(Coord16));
	if( p_template->m_cores == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// unpacking
	for( i=0; i<nbCores; i++ )
	{
		READ_INT16(p_template->m_cores[i].m_x,(p_buffer+offset));
		offset += sizeof(INT16);	
		READ_INT16(p_template->m_cores[i].m_y,(p_buffer+offset));
		offset += sizeof(INT16);
	}
	// deltas
	//allocation
	p_template->m_deltas = (Coord16*)MALLOC(p_template->m_nbDeltas*sizeof(Coord16));
	if( p_template->m_deltas == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// unpacking
	for( i=0; i<p_template->m_nbDeltas; i++ )
	{
		READ_INT16(p_template->m_deltas[i].m_x,(p_buffer+offset));
		offset += sizeof(INT16);	
		READ_INT16(p_template->m_deltas[i].m_y,(p_buffer+offset));
		offset += sizeof(INT16);
	}

	// 1st core ori
	READ_INT8(p_template->m_oriCore,(p_buffer+offset));
	offset += sizeof(UINT8);

	// bounding box
	READ_INT16(p_template->m_boundBox.m_left,(p_buffer+offset));
	offset += sizeof(INT16);
	READ_INT16(p_template->m_boundBox.m_right,(p_buffer+offset));
	offset += sizeof(INT16);
	READ_INT16(p_template->m_boundBox.m_top,(p_buffer+offset));
	offset += sizeof(INT16);
	READ_INT16(p_template->m_boundBox.m_bottom,(p_buffer+offset));
	offset += sizeof(INT16);

	// number of munitiae
	READ_INT16(p_template->m_nbMinutiae,(p_buffer+offset));
	offset += sizeof(UINT16);

	// minutiae
	//allocation
	p_template->m_minutiae = (Minutia*)MALLOC(p_template->m_nbMinutiae*sizeof(Minutia));
	if( p_template->m_minutiae == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// unpacking
	for( i=0; i<p_template->m_nbMinutiae; i++ )
	{
		READ_INT16(p_template->m_minutiae[i].m_coord.m_x,(p_buffer+offset));
		offset += sizeof(INT16);
		READ_INT16(p_template->m_minutiae[i].m_coord.m_y,(p_buffer+offset));
		offset += sizeof(INT16);
		READ_INT8(p_template->m_minutiae[i].m_ori,(p_buffer+offset));
		offset += sizeof(UINT8);
		READ_INT8(p_template->m_minutiae[i].m_qual,(p_buffer+offset));
		offset += sizeof(UINT8);
	}

	// mean frequency
	READ_INT16(p_template->m_meanFreq,(p_buffer+offset));
	offset += sizeof(UINT16);

	// bloc Size
	READ_INT8(p_template->m_blocSize,(p_buffer+offset));
	offset += sizeof(UINT8);
		
	// width and height of local infos array
	READ_INT8(p_template->m_localW,(p_buffer+offset));
	offset += sizeof(UINT8);
	READ_INT8(p_template->m_localH,(p_buffer+offset));
	offset += sizeof(UINT8);

	// correct a bug in templates coming from bioEngine before V1.0.0 (ex: 1.0e)
	// p_template->m_blocSize was set to zero during template refinement...
	if( p_template->m_blocSize == 0 && p_template->m_localW != 0 && p_template->m_localH != 0 )
		p_template->m_blocSize = localInfosBlocSize*globalImgScaling*globalTemplateScaling/(256*256);

	// shift between center of the arrays and first core
	READ_INT16(p_template->m_localShiftX,(p_buffer+offset));
	offset += sizeof(INT16);
	READ_INT16(p_template->m_localShiftY,(p_buffer+offset));
	offset += sizeof(INT16);

	// ori array
	//allocation
	p_template->m_ori = (UINT8*)MALLOC(p_template->m_localW*p_template->m_localH*sizeof(UINT8));
	if( p_template->m_ori == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// unpacking
	for( i=0; i<(UINT32)(p_template->m_localW*p_template->m_localH); i++ )
	{
		READ_INT8(p_template->m_ori[i],(p_buffer+offset));
		offset += sizeof(UINT8);				
	}

	// qual array
	//allocation
	p_template->m_qual = (UINT8*)MALLOC(p_template->m_localW*p_template->m_localH*sizeof(UINT8));
	if( p_template->m_qual == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// unpacking
	j=0;
	for( i=0; i<(UINT32)(p_template->m_localW*p_template->m_localH); i++ )
	{
		if( j==0 )
		{
			READ_INT8(val,(p_buffer+offset));
			offset += sizeof(UINT8);
		}

		p_template->m_qual[i] = (val>>j)&15; // decode qual with 4 bits
		j = 4-j;
	}

	// template signature
	for( i=0; i<16; i++ )
	{
		READ_INT8(p_template->m_sign[i],(p_buffer+offset));
		offset += sizeof(UINT8);
	}

	return ZEF_OK;

	//error handling
errorHandler:

	if( p_template->m_cores != NULL )
	{
		FREE( p_template->m_cores );
		p_template->m_cores = NULL;
	}

	if( p_template->m_deltas != NULL )
	{
		FREE( p_template->m_deltas );
		p_template->m_deltas = NULL;
	}

	if( p_template->m_minutiae != NULL )
	{
		FREE( p_template->m_minutiae );
		p_template->m_minutiae = NULL;
	}
	
	if( p_template->m_ori != NULL )
	{
		FREE(p_template->m_ori);
		p_template->m_ori = NULL;
	}

	if( p_template->m_qual != NULL )
	{
		FREE(p_template->m_qual);
		p_template->m_qual = NULL;
	}

	return error;
}

// fill template data with buffer
INT32 templateCopy( Template *p_dest, Template *p_src )
{
	UINT32	i;
	UINT32	nbCores;
	INT32	error;
	
	// size
	p_dest->m_size = p_src->m_size;
	// version
	p_dest->m_version = p_src->m_version;
	// nb cores
	p_dest->m_nbCores = p_src->m_nbCores;
	// nb deltas
	p_dest->m_nbDeltas = p_src->m_nbDeltas;

	// cores
	//allocation
	nbCores = p_dest->m_nbCores;
	if( nbCores == 0 )
		nbCores = 1; // there is always 1 core even if it's not specified
	p_dest->m_cores = (Coord16*)MALLOC(nbCores*sizeof(Coord16));
	if( p_dest->m_cores == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// copy
	for( i=0; i<nbCores; i++ )
	{
		p_dest->m_cores[i].m_x = p_src->m_cores[i].m_x;	
		p_dest->m_cores[i].m_y = p_src->m_cores[i].m_y;
	}
	// deltas
	//allocation
	p_dest->m_deltas = (Coord16*)MALLOC(p_dest->m_nbDeltas*sizeof(Coord16));
	if( p_dest->m_deltas == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// copy
	for( i=0; i<p_dest->m_nbDeltas; i++ )
	{
		p_dest->m_deltas[i].m_x = p_src->m_deltas[i].m_x;	
		p_dest->m_deltas[i].m_y = p_src->m_deltas[i].m_y;
	}

	// 1st core ori
	p_dest->m_oriCore = p_src->m_oriCore;
	// bounding box
	p_dest->m_boundBox.m_left = p_src->m_boundBox.m_left;
	p_dest->m_boundBox.m_right = p_src->m_boundBox.m_right;
	p_dest->m_boundBox.m_top = p_src->m_boundBox.m_top;
	p_dest->m_boundBox.m_bottom = p_src->m_boundBox.m_bottom;
	
	// number of munitiae
	p_dest->m_nbMinutiae = p_src->m_nbMinutiae;
	// minutiae
	//allocation
	p_dest->m_minutiae = (Minutia*)MALLOC(p_dest->m_nbMinutiae*sizeof(Minutia));
	if( p_dest->m_minutiae == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// copy
	for( i=0; i<p_dest->m_nbMinutiae; i++ )
	{
		p_dest->m_minutiae[i].m_coord.m_x = p_src->m_minutiae[i].m_coord.m_x;
		p_dest->m_minutiae[i].m_coord.m_y = p_src->m_minutiae[i].m_coord.m_y;
		p_dest->m_minutiae[i].m_ori = p_src->m_minutiae[i].m_ori;
		p_dest->m_minutiae[i].m_qual = p_src->m_minutiae[i].m_qual;
	}

	// mean frequency
	p_dest->m_meanFreq = p_src->m_meanFreq;
	// bloc Size
	p_dest->m_blocSize = p_src->m_blocSize;		
	// width and height of local infos array
	p_dest->m_localW = p_src->m_localW;
	p_dest->m_localH = p_src->m_localH;
	// shift between center of the arrays and first core
	p_dest->m_localShiftX = p_src->m_localShiftX;
	p_dest->m_localShiftY = p_src->m_localShiftY;

	// ori array
	//allocation
	p_dest->m_ori = (UINT8*)MALLOC(p_dest->m_localW*p_dest->m_localH*sizeof(UINT8));
	if( p_dest->m_ori == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// copy
	for( i=0; i<(UINT32)(p_dest->m_localW*p_dest->m_localH); i++ )
	{
		p_dest->m_ori[i] = p_src->m_ori[i];				
	}

	// qual array
	//allocation
	p_dest->m_qual = (UINT8*)MALLOC(p_dest->m_localW*p_dest->m_localH*sizeof(UINT8));
	if( p_dest->m_qual == NULL )
	{
		error = ZEF_OUT_OF_MEMORY;
		goto errorHandler;
	}
	// copy
	for( i=0; i<(UINT32)(p_dest->m_localW*p_dest->m_localH); i++ )
	{
		p_dest->m_qual[i] = p_src->m_qual[i];
	}

	// template signature
	for( i=0; i<16; i++ )
	{
		p_dest->m_sign[i] = p_src->m_sign[i];
	}

	return ZEF_OK;

	//error handling
errorHandler:

	if( p_dest->m_cores != NULL )
	{
		FREE( p_dest->m_cores );
		p_dest->m_cores = NULL;
	}

	if( p_dest->m_deltas != NULL )
	{
		FREE( p_dest->m_deltas );
		p_dest->m_deltas = NULL;
	}

	if( p_dest->m_minutiae != NULL )
	{
		FREE( p_dest->m_minutiae );
		p_dest->m_minutiae = NULL;
	}
	
	if( p_dest->m_ori != NULL )
	{
		FREE(p_dest->m_ori);
		p_dest->m_ori = NULL;
	}

	if( p_dest->m_qual != NULL )
	{
		FREE(p_dest->m_qual);
		p_dest->m_qual = NULL;
	}

	return error;
}

INT32 templateInit( Template *p_template )
{
	// set everything to all zeros
	memset(p_template, 0, sizeof(Template) );

	// set version and size
	p_template->m_version = TEMPLATE_VERSION;
	p_template->m_size = sizeof( Template );

	return ZEF_OK;
}

INT32 templateUninit( Template *p_template )
{
	// release internal template memory

	if( p_template->m_cores != NULL )
	{
		FREE( p_template->m_cores);
		p_template->m_cores = NULL;
	}
	if( p_template->m_deltas != NULL )
	{
		FREE( p_template->m_deltas);
		p_template->m_deltas = NULL;
	}
	if( p_template->m_minutiae != NULL )
	{
		FREE( p_template->m_minutiae);
		p_template->m_minutiae = NULL;
	}
	if( p_template->m_qual != NULL )
	{
		FREE( p_template->m_qual );
		p_template->m_qual = NULL;
	}
	if( p_template->m_ori != NULL )
	{
		FREE( p_template->m_ori);
		p_template->m_ori = NULL;
	}

	return ZEF_OK;
}

